@@ -1,4 +1,4 @@
-<TeXmacs|1.0.7.20>
+<TeXmacs|1.99.16>
 
 <style|<tuple|source|std>>
 
@@ -34,7 +34,9 @@
 
   <assign|made-by-TeXmacs|<macro|<float|footnote||<with|font-size|0.84|par-mode|justify|par-left|0cm|par-right|0cm|<active*|<move|<image|local:$TEXMACS_PATH/misc/images/tm_gnu3.ps||1fn||>|0fn|-0.2fn>><space|2spc><localize|This
   document has been produced using> GNU <TeXmacs> (<localize|see>
-  <with|font-family|tt|http://www.texmacs.org>).<right-flush>>>>>
+  <with|font-family|tt|https://www.texmacs.org>).<right-flush>>>>>
+
+  <assign|tm-made|<macro|<with|ornament-color||ornament-shadow-color|<value|color>|ornament-sunny-color|<value|color>|ornament-border|1ln|ornament-shape|rounded|ornament-hpadding|0.5spc|ornament-vpadding|0.5spc|<ornament|<math|\<Backsigma\>>>>>>
 
   <assign|TeX|<macro|<active*|T<rsub|<space|-0.4spc><move|<resize|<with|math-level|0|E>||||0.5fn>|0fn|-0.1fn>><space|-0.4spc>X>>>
 
@@ -42,9 +44,11 @@
 
   <assign|LaTeXe|<macro|<active*|L<rsup|<space|-0.8spc><move|A|0fn|-0.1fn>><space|-0.2spc>T<rsub|<space|-0.4spc><move|<resize|<with|math-level|0|E>||||0.5fn>|0fn|-0.1fn>><space|-0.4spc>X<compound|math|>><space|0.2spc>2<move|<math|\<varepsilon\>>|-0.0fn|-0.15fn>>>
 
+  <assign|LaTeX*|<macro|<active*|(L<rsup|<space|-0.8spc><move|A|0fn|-0.1fn>><space|-0.2spc>)<space|-0.2spc>T<rsub|<space|-0.4spc><move|<resize|<with|math-level|0|E>||||0.5fn>|0fn|-0.1fn>><space|-0.4spc>X>>>
+
   <assign|BibTeX|<macro|<active*|<with|font-shape|small-caps|Bib>T<rsub|<space|-0.4spc><move|<resize|<with|math-level|0|E>||||0.5fn>|0fn|-0.1fn>><space|-0.4spc>X>>>
 
-  <assign|hrule|<macro|<no-indent><tabular|<tformat|<cwith|1|-1|1|-1|cell-tborder|1ln>|<twith|table-width|1par>|<cwith|1|-1|1|-1|cell-vmode|exact>|<cwith|1|-1|1|-1|cell-height|1ln>|<cwith|1|-1|1|-1|cell-lsep|0fn>|<cwith|1|-1|1|-1|cell-rsep|0fn>|<cwith|1|-1|1|-1|cell-bsep|0fn>|<cwith|1|-1|1|-1|cell-tsep|0fn>|<cwith|1|-1|1|-1|cell-vcorrect|n>|<table|<row|<cell|<space|1fn|0ln|1ln>>>>>>>>
+  <assign|old-hrule|<macro|<no-indent><tabular|<tformat|<cwith|1|-1|1|-1|cell-tborder|1ln>|<twith|table-width|1par>|<cwith|1|-1|1|-1|cell-vmode|exact>|<cwith|1|-1|1|-1|cell-height|1ln>|<cwith|1|-1|1|-1|cell-lsep|0fn>|<cwith|1|-1|1|-1|cell-rsep|0fn>|<cwith|1|-1|1|-1|cell-bsep|0fn>|<cwith|1|-1|1|-1|cell-tsep|0fn>|<cwith|1|-1|1|-1|cell-vcorrect|n>|<table|<row|<cell|<space|1fn|0ln|1ln>>>>>>>>
 
   <assign|pre-edit|<macro|body|<with|ornament-color|#ffffc0|ornament-sunny-color|#fff0e0|ornament-shadow-color|#ffc080|ornament-border|1px|ornament-hpadding|2px|ornament-vpadding|2px|<smash|<ornament|<with|color|#4040c0|<arg|body>>>>>>>
 
@@ -117,9 +121,9 @@
 
   <assign|text|<macro|body|<with|mode|text|<arg|body>>>>
 
-  <assign|cite*|<macro|body|<with|font-shape|italic|<arg|body>>>>
+  <assign|cite*|<macro|body|<with|font-shape|<if|<equal|<value|font-shape>|italic>|right|italic>|<arg|body>>>>
 
-  <assign|dfn|<macro|body|<with|font-shape|italic|<arg|body>>>>
+  <assign|dfn|<macro|body|<with|font-shape|<if|<equal|<value|font-shape>|italic>|right|italic>|<arg|body>>>>
 
   <assign|code*|<macro|body|<with|font-family|tt|<arg|body>>>>
 
@@ -147,6 +151,8 @@
 
   <assign|chinese|<macro|body|<with|language|chinese|<arg|body>>>>
 
+  <assign|croatian|<macro|body|<with|language|croatian|<arg|body>>>>
+
   <assign|czech|<macro|body|<with|language|czech|<arg|body>>>>
 
   <assign|danish|<macro|body|<with|language|danish|<arg|body>>>>
@@ -155,6 +161,8 @@
 
   <assign|english|<macro|body|<with|language|english|<arg|body>>>>
 
+  <assign|esperanto|<macro|body|<with|language|esperanto|<arg|body>>>>
+
   <assign|finnish|<macro|body|<with|language|finnish|<arg|body>>>>
 
   <assign|french|<macro|body|<with|language|french|<arg|body>>>>
@@ -239,6 +247,14 @@
     </surround>
   </macro>>
 
+  <assign|ultra-compact|<\macro|body>
+    <\surround||<right-flush>>
+      <\with|par-sep|0fn|par-ver-sep|0fn|par-line-sep|0fn|par-par-sep|0fn>
+        <arg|body>
+      </with>
+    </surround>
+  </macro>>
+
   <assign|compact|<\macro|body>
     <\surround||<right-flush>>
       <\with|par-par-sep|0fn>
@@ -247,6 +263,32 @@
     </surround>
   </macro>>
 
+  <assign|compressed|<\macro|body>
+    <\surround||<right-flush>>
+      <\with|par-par-sep|<times|0.5|<value|par-par-sep>>>
+        <arg|body>
+      </with>
+    </surround>
+  </macro>>
+
+  <assign|amplified|<\macro|body>
+    <\surround||<right-flush>>
+      <\with|par-par-sep|<times|1.5|<value|par-par-sep>>>
+        <arg|body>
+      </with>
+    </surround>
+  </macro>>
+
+  <assign|vgroup|<\macro|body>
+    <\surround|<no-break-start>|<no-break-end><right-flush>>
+      <\with|par-min-penalty|100000000>
+        <arg|body>
+      </with>
+    </surround>
+  </macro>>
+
+  <assign|indivisible|<value|vgroup>>
+
   \;
 
   <assign|center|<macro|body|<with|par-mode|center|<arg|body>>>>
@@ -277,6 +319,14 @@
 
   <assign|block*|<macro|body|<tformat|<cwith|1|-1|1|-1|cell-rborder|1ln>|<cwith|1|-1|1|-1|cell-bborder|1ln>|<cwith|1|1|1|-1|cell-tborder|1ln>|<cwith|1|-1|1|1|cell-lborder|1ln>|<cwith|1|-1|1|-1|cell-halign|c>|<arg|body>>>>
 
+  <assign|wide-tabular|<macro|body|<tformat|<twith|table-width|1par>|<twith|table-hmode|exact>|<twith|table-block|yes>|<cwith|1|-1|1|-1|cell-hyphen|t>|<cwith|1|-1|1|-1|cell-hpart|0.001>|<cwith|1|-1|1|1|cell-lsep|0fn>|<cwith|1|-1|-1|-1|cell-rsep|0fn>|<arg|body>>>>
+
+  <assign|wide-block|<macro|body|<tformat|<twith|table-width|1par>|<twith|table-hmode|exact>|<twith|table-block|yes>|<cwith|1|-1|1|-1|cell-hyphen|t>|<cwith|1|-1|1|-1|cell-hpart|0.001>|<cwith|1|-1|1|-1|cell-rborder|1ln>|<cwith|1|-1|1|-1|cell-bborder|1ln>|<cwith|1|1|1|-1|cell-tborder|1ln>|<cwith|1|-1|1|1|cell-lborder|1ln>|<arg|body>>>>
+
+  <assign|par-block|<macro|body|<tabular|<tformat|<twith|table-width|1par>|<twith|table-hmode|exact>|<cwith|1|1|1|1|cell-hyphen|t>|<cwith|1|1|1|1|cell-lsep|0spc>|<cwith|1|1|1|1|cell-rsep|0spc>|<table|<row|<\cell>
+    <arg|body>
+  </cell>>>>>>>
+
   <\active*>
     <\src-comment>
       Transparency.
@@ -301,6 +351,8 @@
 
   <assign|vphantom|<macro|body|<hresize|<phantom|<arg|body>>|0em|0em>>>
 
+  <assign|concealed|<macro|body|>>
+
   <\active*>
     <\src-comment>
       Adjusting the sizes of boxes.
@@ -317,17 +369,25 @@
 
   <assign|smash-bottom|<macro|body|<vresize|<arg|body>|0ex|1t>>>
 
+  <assign|reduce-by|<macro|body|by|<vresize|<arg|body>|<plus|1b|<arg|by>>|<minus|1t|<arg|by>>>>>
+
+  <assign|reduce-top-by|<macro|body|by|<vresize|<arg|body>||<minus|1t|<arg|by>>>>>
+
+  <assign|reduce-bottom-by|<macro|body|by|<vresize|<arg|body>|<plus|1b|<arg|by>>|>>>
+
   \;
 
   <assign|extend|<macro|body|left|bottom|right|top|<style-with|src-compact|none|<resize|<arg|body>|<if|<equal|<arg|left>|>|1l|<minimum|1l|<arg|left>>>|<if|<equal|<arg|bottom>|>|1b|<minimum|1b|<arg|bottom>>>|<if|<equal|<arg|right>|>|1r|<maximum|1r|<arg|right>>>|<if|<equal|<arg|top>|>|1t|<maximum|1t|<arg|top>>>>>>>
 
+  <drd-props|extend|arity|5|accessible|0|length|1|length|2|length|3|length|4>
+
   <assign|extend-right|<macro|body|right|<resize|<arg|body>|||<maximum|1r|<arg|right>>|>>>
 
-  <assign|swell|<macro|body|<resize|<arg|body>||<minimum|1b|1fnbot>||<maximum|1t|1fntop>>>>
+  <assign|inflate|<macro|body|<resize|<arg|body>||<minimum|1b|1fnbot>||<maximum|1t|1fntop>>>>
 
-  <assign|swell-bottom|<macro|body|<resize|<arg|body>||<minimum|1b|1fnbot>||>>>
+  <assign|inflate-bottom|<macro|body|<resize|<arg|body>||<minimum|1b|1fnbot>||>>>
 
-  <assign|swell-top|<macro|body|<resize|<arg|body>||||<maximum|1t|1fntop>>>>
+  <assign|inflate-top|<macro|body|<resize|<arg|body>||||<maximum|1t|1fntop>>>>
 
   \;
 
@@ -366,29 +426,72 @@
 
   <drd-props|underline|with-like|yes|arity|1|accessible|all>
 
-  <assign|strike-through|<macro|body|<quasi|<style-with|src-compact|none|<datoms|<macro|x|<with|color|<unquote|<value|color>>|<repeat|<arg|x>|<resize|-|<plus|1l|0.1fn>||<plus|1r|-0.1fn>|>>>>|<arg|body>>>>>>
+  <assign|stressed-color|black>
+
+  <assign|stressed-distance|0.2fn>
+
+  <assign|stressed|<macro|body|<quasi|<style-with|src-compact|none|<datoms|<macro|x|<with|color|<unquote|<value|stressed-color>>|<repeat*|<with|color|<unquote|<value|color>>|<arg|x>>|<move|<resize|-|<plus|0.6667l|0.3333r>||<plus|0.3333l|0.6667r>|>||<minus|-0.5ex|<unquote|<value|stressed-distance>>>>>>>|<arg|body>>>>>>
+
+  <drd-props|stressed|with-like|yes|arity|1|accessible|all>
+
+  <assign|strike-through|<macro|body|<quasi|<style-with|src-compact|none|<datoms|<macro|x|<with|color|<unquote|<value|color>>|<repeat|<arg|x>|<resize|-|<plus|0.6667l|0.3333r>||<plus|0.3333l|0.6667r>|>>>>|<arg|body>>>>>>
 
   <drd-props|strike-through|with-like|yes|arity|1|accessible|all>
 
+  <assign|hrule|<macro|<strike-through|<htab|5mm>>>>
+
   <assign|repeat-through|<macro|what|body|<quasi|<style-with|src-compact|none|<datoms|<macro|x|<repeat|<arg|x>|<unquote|<arg|what>>>>|<arg|body>>>>>>
 
   <drd-props|repeat-through|arity|2|accessible|1>
 
+  \;
+
   <assign|deleted-color|red>
 
   <assign|deleted|<macro|body|<repeat-through|<with|color|<value|deleted-color>|/>|<arg|body>>>>
 
+  <drd-props|deleted|with-like|yes|arity|1|accessible|all>
+
+  \;
+
   <assign|fill-out-color|black>
 
   <assign|fill-out-distance|0.2fn>
 
-  <assign|fill-out|<macro|body|<repeat-through|<with|color|<value|fill-out-color>|<move|.||<minus|<value|fill-out-distance>>>>|<arg|body>>>>
+  <assign|fill-out-hspace|0fn>
+
+  <assign|fill-out|<macro|body|<repeat-through|<with|color|<value|fill-out-color>|<move|.||<minus|<value|fill-out-distance>>><hspace|<value|fill-out-hspace>>>|<arg|body>>>>
+
+  <assign|fill-out*|<macro|body|<fill-out|<arg|body><htab|5mm>>>>
+
+  <assign|fill-dots|<macro|<fill-out|<htab|5mm>>>>
+
+  <drd-props|fill-out|with-like|yes|arity|1|accessible|all>
+
+  \;
 
   <assign|marked-color|yellow>
 
   <assign|marked-padding|0.2fn>
 
-  <assign|marked|<macro|body|<style-with|src-compact|none|<datoms|<macro|x|<resize|<tabular|<tformat|<cwith|1|1|1|1|cell-background|<value|marked-color>>|<cwith|1|1|1|1|cell-lsep|<value|marked-padding>>|<cwith|1|1|1|1|cell-rsep|<value|marked-padding>>|<cwith|1|1|1|1|cell-bsep|<value|marked-padding>>|<cwith|1|1|1|1|cell-tsep|<value|marked-padding>>|<table|<row|<cell|<arg|x>>>>>>|<plus|1l|<value|marked-padding>>|<plus|1b|<value|marked-padding>>|<minus|1r|<value|marked-padding>>|<minus|1t|<value|marked-padding>>>>|<arg|body>>>>>
+  <assign|marked|<macro|body|<quasi|<style-with|src-compact|none|<datoms|<macro|x|<style-with|src-compact|none|<resize|<tabular|<tformat|<cwith|1|1|1|1|cell-background|<unquote|<value|marked-color>>>|<cwith|1|1|1|1|cell-lsep|<unquote|<value|marked-padding>>>|<cwith|1|1|1|1|cell-rsep|<unquote|<value|marked-padding>>>|<cwith|1|1|1|1|cell-bsep|<unquote|<value|marked-padding>>>|<cwith|1|1|1|1|cell-tsep|<unquote|<value|marked-padding>>>|<cwith|1|-1|1|-1|cell-lborder|0ln>|<cwith|1|-1|1|-1|cell-rborder|0ln>|<cwith|1|-1|1|-1|cell-bborder|0ln>|<cwith|1|-1|1|-1|cell-tborder|0ln>|<twith|table-width|>|<twith|table-hmode|auto>|<twith|table-block|no>|<table|<row|<cell|<arg|x>>>>>>|<plus|1l|<unquote|<value|marked-padding>>>|<plus|1b|<unquote|<value|marked-padding>>>|<minus|1r|<unquote|<value|marked-padding>>>|<minus|1t|<unquote|<value|marked-padding>>>>>>|<arg|body>>>>>>
+
+  <drd-props|marked|with-like|yes|arity|1|accessible|all>
+
+  \;
+
+  <assign|todo-color|dark red>
+
+  <assign|todo-bg-color|pastel red>
+
+  <assign|render-todo|<macro|fg|bg|body|<with|color|<arg|fg>|<quasi|<style-with|src-compact|none|<datoms|<macro|x|<style-with|src-compact|none|<resize|<tabular|<tformat|<cwith|1|1|1|1|cell-background|<unquote|<arg|bg>>>|<cwith|1|1|1|1|cell-lsep|0fn>|<cwith|1|1|1|1|cell-rsep|0fn>|<cwith|1|1|1|1|cell-bsep|<unquote|<value|marked-padding>>>|<cwith|1|1|1|1|cell-tsep|<unquote|<value|marked-padding>>>|<cwith|1|-1|1|-1|cell-lborder|0ln>|<cwith|1|-1|1|-1|cell-rborder|0ln>|<cwith|1|-1|1|-1|cell-bborder|0ln>|<cwith|1|-1|1|-1|cell-tborder|0ln>|<twith|table-width|>|<twith|table-hmode|auto>|<twith|table-block|no>|<table|<row|<cell|<arg|x>>>>>>|<plus|1l|0fn>|<plus|1b|<unquote|<value|marked-padding>>>|<minus|1r|0fn>|<minus|1t|<unquote|<value|marked-padding>>>>>>|[<arg|body>]>>>>>>
+
+  <assign|todo|<macro|body|<render-todo|<value|todo-color>|<value|todo-bg-color>|<arg|body>>>>
+
+  <assign|todo*|<macro|body|<render-todo|dark orange|pastel
+  orange|<arg|body>>>>
+
+  <drd-props|render-todo|arity|3|color|0|color|1|accessible|2>
 
   <\active*>
     <\src-comment>
@@ -400,29 +503,29 @@
 
   <assign|marginal-note-sep|5mm>
 
-  <assign|marginal-note-table|<macro|body|vpos|hyph|<tabular|<tformat|<twith|table-width|<value|marginal-note-width>>|<twith|table-hmode|exact>|<cwith|1|1|1|1|cell-hyphen|<arg|hyph>>|<twith|table-valign|<arg|vpos>>|<cwith|1|1|1|1|cell-lsep|0em>|<cwith|1|1|1|1|cell-rsep|0em>|<table|<row|<\cell>
-    <\with|par-par-sep|0em|par-line-sep|0em>
+  <assign|marginal-note-table|<macro|body|hpos|hhyph|vpos|vhyph|<tabular|<tformat|<twith|table-width|<value|marginal-note-width>>|<twith|table-hmode|exact>|<cwith|1|1|1|1|cell-hyphen|<arg|vhyph>>|<twith|table-valign|<arg|vpos>>|<cwith|1|1|1|1|cell-lsep|0em>|<cwith|1|1|1|1|cell-rsep|0em>|<cwith|1|1|1|1|cell-halign|<arg|hpos>>|<table|<row|<\cell>
+    <\with|par-par-sep|0em|par-line-sep|0em|par-mode|<arg|hhyph>>
       <arg|body>
     </with>
   </cell>>>>>>>
 
-  <assign|left-marginal-note|<macro|body|<line-note|<marginal-note-table|<arg|body>|T|t>|<minus|0cm|<plus|<value|marginal-note-width>|<value|marginal-note-sep>>>|0cm><flag|marginal
+  <assign|marginal-even-left-note|<macro|vpos|body|<line-note|<specific|even|<marginal-note-table|<arg|body>|r|right|<change-case|<arg|vpos>|UPCASE>|<arg|vpos>>>|<minus|0cm|<plus|<value|marginal-note-width>|<value|marginal-note-sep>>>|0cm><line-note|<specific|odd|<marginal-note-table|<arg|body>|l|left|<change-case|<arg|vpos>|UPCASE>|<arg|vpos>>>|<plus|1par|<value|marginal-note-sep>|<value|par-left>|<value|par-right>>|0cm><flag|marginal
   note|dark brown>>>
 
-  <assign|right-marginal-note|<macro|body|<line-note|<marginal-note-table|<arg|body>|T|t>|<plus|1par|<value|marginal-note-sep>|<value|par-left>|<value|par-right>>|0cm><flag|marginal
+  <assign|marginal-even-right-note|<macro|vpos|body|<line-note|<specific|odd|<marginal-note-table|<arg|body>|r|right|<change-case|<arg|vpos>|UPCASE>|<arg|vpos>>>|<minus|0cm|<plus|<value|marginal-note-width>|<value|marginal-note-sep>>>|0cm><line-note|<specific|even|<marginal-note-table|<arg|body>|l|left|<change-case|<arg|vpos>|UPCASE>|<arg|vpos>>>|<plus|1par|<value|marginal-note-sep>|<value|par-left>|<value|par-right>>|0cm><flag|marginal
   note|dark brown>>>
 
-  <assign|left-marginal-note*|<macro|body|<line-note|<marginal-note-table|<arg|body>|B|b>|<minus|0cm|<plus|<value|marginal-note-width>|<value|marginal-note-sep>>>|0cm><flag|marginal
+  <assign|marginal-left-note|<macro|vpos|body|<line-note|<marginal-note-table|<arg|body>|r|right|<change-case|<arg|vpos>|UPCASE>|<arg|vpos>>|<minus|0cm|<plus|<value|marginal-note-width>|<value|marginal-note-sep>>>|0cm><flag|marginal
   note|dark brown>>>
 
-  <assign|right-marginal-note*|<macro|body|<line-note|<marginal-note-table|<arg|body>|B|b>|<plus|1par|<value|marginal-note-sep>|<value|par-left>|<value|par-right>>|0cm><flag|marginal
+  <assign|marginal-right-note|<macro|vpos|body|<line-note|<marginal-note-table|<arg|body>|l|left|<change-case|<arg|vpos>|UPCASE>|<arg|vpos>>|<plus|1par|<value|marginal-note-sep>|<value|par-left>|<value|par-right>>|0cm><flag|marginal
   note|dark brown>>>
 
-  <assign|left-marginal-note**|<macro|body|<line-note|<marginal-note-table|<arg|body>|C|c>|<minus|0cm|<plus|<value|marginal-note-width>|<value|marginal-note-sep>>>|0cm><flag|marginal
-  note|dark brown>>>
+  <assign|marginal-normal-note|<macro|vpos|body|<compound|<if|<and|<equal|<value|page-odd>|<value|page-even>>|<equal|<value|page-odd-shift>|<value|page-even-shift>>>|marginal-left-note|marginal-even-left-note>|<arg|vpos>|<arg|body>>>>
 
-  <assign|right-marginal-note**|<macro|body|<line-note|<marginal-note-table|<arg|body>|C|c>|<plus|1par|<value|marginal-note-sep>|<value|par-left>|<value|par-right>>|0cm><flag|marginal
-  note|dark brown>>>
+  <assign|marginal-note|<macro|hpos|vpos|body|<with|dummy1|<value|marginal-note-width>|dummy2|<value|marginal-note-sep>|<compound|<merge|marginal-|<arg|hpos>|-note>|<arg|vpos>|<arg|body>|<arg|hpos>>>>>
+
+  <drd-props|marginal-note|arity|3|accessible|none>
 
   <\active*>
     <\src-comment>
@@ -430,10 +533,20 @@
     </src-comment>
   </active*>
 
-  <assign|html-div|<macro|name|body|<arg|body>>>
+  <assign|web-title|<macro|title|>>
+
+  <assign|html-tag|<macro|name|body|<arg|body>>>
+
+  <assign|html-attr|<macro|attr|val|body|<arg|body>>>
+
+  <assign|html-div-style|<macro|name|body|<arg|body>>>
+
+  <assign|html-div-class|<macro|name|body|<arg|body>>>
 
   <assign|html-style|<macro|style|body|<arg|body>>>
 
+  <assign|html-class|<macro|style|body|<arg|body>>>
+
   <assign|html-javascript|<macro|script|<small|<colored-frame|pastel
   yellow|<with|font-family|ss|Javascript:
   ><with|font-family|tt|<arg|script>>>>>>
@@ -446,6 +559,86 @@
   yellow|<tabular*|<tformat|<table|<row|<cell|<small|<with|font-family|tt|<arg|dest>>>>>|<row|<cell|<arg|width>
   <math|\<times\>> <arg|height>>>>>>>>>>
 
+  <\active*>
+    <\src-comment>
+      Poor man font effects.
+    </src-comment>
+  </active*>
+
+  <assign|add-font-effect|<macro|prop|val|body|<with|font-effects|<merge|<value|font-effects>|<if|<equal|<value|font-effects>|>||,>|<arg|prop>|=|<arg|val>>|<arg|body>>>>
+
+  \;
+
+  <assign|embold-strength|2>
+
+  <assign|embbb-strength|3>
+
+  <assign|slanted-slope|0.25>
+
+  <assign|hmagnified-factor|1.5>
+
+  <assign|vmagnified-factor|1.5>
+
+  <assign|condensed-factor|0.8>
+
+  <assign|extended-factor|1.2>
+
+  <assign|monospaced-factor|0.75>
+
+  <assign|degraded-threshold|0.667>
+
+  <assign|degraded-frequency|1.0>
+
+  <assign|distorted-strength|0.5>
+
+  <assign|distorted-frequency|1.0>
+
+  <assign|gnawed-strength|0.6>
+
+  <assign|gnawed-frequency|1.0>
+
+  \;
+
+  <assign|embold|<macro|body|<add-font-effect|bold|<value|embold-strength>|<arg|body>>>>
+
+  <assign|embbb|<macro|body|<add-font-effect|bbb|<value|embbb-strength>|<arg|body>>>>
+
+  <assign|slanted|<macro|body|<add-font-effect|slant|<value|slanted-slope>|<arg|body>>>>
+
+  <assign|hmagnified|<macro|body|<add-font-effect|hmagnify|<value|hmagnified-factor>|<arg|body>>>>
+
+  <assign|vmagnified|<macro|body|<add-font-effect|vmagnify|<value|vmagnified-factor>|<arg|body>>>>
+
+  <assign|condensed|<macro|body|<add-font-effect|hextended|<value|condensed-factor>|<arg|body>>>>
+
+  <assign|extended|<macro|body|<add-font-effect|hextended|<value|extended-factor>|<arg|body>>>>
+
+  <assign|monospaced|<macro|body|<add-font-effect|mono|<value|monospaced-factor>|<arg|body>>>>
+
+  <assign|degraded|<macro|body|<add-font-effect|degraded|<merge|<value|degraded-threshold>|;|<value|degraded-frequency>>|<arg|body>>>>
+
+  <assign|distorted|<macro|body|<add-font-effect|distorted|<merge|<value|distorted-strength>|;|<value|distorted-frequency>>|<arg|body>>>>
+
+  <assign|gnawed|<macro|body|<add-font-effect|gnawed|<merge|<value|gnawed-strength>|;|<value|gnawed-frequency>>|<arg|body>>>>
+
+  <\active*>
+    <\src-comment>
+      Artistic font effects.
+    </src-comment>
+  </active*>
+
+  <assign|blurred-radius|0.6pt>
+
+  <assign|shadow-font-color|#222>
+
+  <assign|sunny-font-color|#ddd>
+
+  \;
+
+  <assign|blurred|<macro|body|<add-font-effect|blurred|<value|blurred-radius>|<arg|body>>>>
+
+  <assign|enhanced|<macro|body|<add-font-effect|enhanced|<merge|<value|blurred-radius>|;|<value|shadow-font-color>|;|<value|sunny-font-color>>|<arg|body>>>>
+
   <\active*>
     <\src-comment>
       Miscellaneous.
@@ -457,10 +650,25 @@
   <assign|slink|<macro|body|<hlink|<with|font-family|tt|language|verbatim|<arg|body>>|<arg|body>>>>
 
   <assign|square|<macro|x|<times|<arg|x>|<arg|x>>>>
+
+  <assign|text-dots-sep|0.3333spc>
+
+  <assign|text-spc|<macro|<hspace|<value|text-dots-sep>>>>
+
+  <assign|text-dots|<macro|<text-spc>.<text-spc>.<text-spc>.<text-spc>>>
+
+  <assign|indent-par-first|1.5fn>
+
+  <assign|padded-par-par-sep|0.6666fn>
+
+  <assign|math-color|<value|color>>
+
+  <assign|strong-color|<value|color>>
 </body>
 
 <\initial>
   <\collection>
     <associate|preamble|true>
+    <associate|src-special|normal>
   </collection>
 </initial>
\ No newline at end of file
