@@ -255,9 +255,9 @@
 (menu-bind note-menu
   (when (not (or (inside? 'float) (inside? 'footnote)))
     (when (in-main-flow?)
-      ("Footnote" (make-wrapped 'footnote)))
-    (when (not (selection-active-non-small?))
-      ("Marginal note" (make-marginal-note)))
+      ("Footnote" (make 'footnote))
+      (when (not (selection-active-non-small?))
+        ("Marginal note" (make-marginal-note))))
     ("Balloon" (make-balloon))
     ---
     (when (in-main-flow?)
@@ -718,7 +718,7 @@
   ("Author" (make-doc-data-element 'doc-author))
   ("Date" (make-doc-data-element 'doc-date))
   ("Today" (begin (make-doc-data-element 'doc-date) (make 'date 0)))
-  ("Miscellanous" (make-doc-data-element 'doc-misc))
+  ("Miscellaneous" (make-doc-data-element 'doc-misc))
   ("Note" (make-doc-data-element 'doc-note))
   (-> "Cite TeXmacs" (link cite-texmacs-menu)))
 
@@ -889,12 +889,13 @@
 (tm-menu (focus-extra-icons t)
   (:require (section-context? t))
   (mini #t
+    //
     (=> (eval (tm/section-get-title-string t))
         (link focus-section-menu))))
 
 (tm-define (child-proposals t i)
-  (:require (and (tree-in? t '(bibliography bibliography*)) (== i 1)))
-  (bib-standard-styles))
+  (:require (and (tree-in? t '(bibliography bibliography*)) (<= i 1)))
+  (if (== i 0) (list "bib" :other) (rcons (bib-standard-styles) :other)))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;; Focus menu for lists
@@ -902,7 +903,7 @@
 
 (tm-define (standard-options l)
   (:require (in? l (list-tag-list)))
-  (list "compact-list" "triangle-list"))
+  (list "compact-list" "triangle-list" "prefix-enumerations"))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;; Focus menu for theorems and proofs
@@ -930,7 +931,8 @@
   (:require (dueto-supporting-context? t))
   //
   (when (not (dueto-added? t))
-    ("Due to" (dueto-add t))))
+    (mini #t
+      ("Due to" (dueto-add t)))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;; Focus menus for algorithms
