@@ -11,6 +11,8 @@
 ;;
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
+(use-modules (dynamic session-edit) (dynamic program-edit))
+
 (define (graphviz-serialize lan t)
     (with u (pre-serialize lan t)
       (with s (texmacs->code (stree->tree u) "SourceCode")
@@ -18,15 +20,16 @@
 
 (define (graphviz-launcher)
   (if (url-exists? "$TEXMACS_HOME_PATH/plugins/tmpy")
-      (string-append "python \""
+      (string-append (python-command) " \""
                      (getenv "TEXMACS_HOME_PATH")
                      "/plugins/tmpy/session/tm_graphviz.py\"")
-      (string-append "python \""
+      (string-append (python-command) " \""
                      (getenv "TEXMACS_PATH")
                      "/plugins/tmpy/session/tm_graphviz.py\"")))
 
-(plugin-configure graphviz 
+(plugin-configure dot 
   (:require (url-exists-in-path? "dot"))
+  (:require (python-command))
   (:launch ,(graphviz-launcher))
   (:serializer ,graphviz-serialize)
   (:session "Graphviz"))
