@@ -144,9 +144,9 @@ parse_identifier (string s, int &i) {
 static string
 parse_command_name (string s, int &i) {
   int n= N(s), start= i;
-  if (i<n && s[i] >= 'A' && s[i] <= 'Z') {
+  if (i<n && is_upcase(s[i])) {
     i++;
-    while (i<n && s[i] >= 'a' && s[i] <= 'z') i++;
+    while (i<n && is_locase (s[i])) i++;
   }
   return s (start, i);
 }
@@ -373,19 +373,23 @@ parse_list (string s, int &i) {
 
 static void
 parse_pretty_printing_definition (string s) {
+  (void) s;
 }
 
 static void
 parse_pretty_printing_removal (string s) {
+  (void) s;
 }
 
 static bool
 is_defining_pretty_printing (string s, int i) {
+  (void) s; (void) i;
   return false;
 }
 
 static bool
 is_removing_pretty_printing (string s, int i) {
+  (void) s; (void) i;
   return false;
 }
 
