@@ -34,6 +34,7 @@ extern string WARN_MISSING;
 extern string GLOBAL_TITLE;
 extern string GLOBAL_AUTHOR;
 extern string GLOBAL_SUBJECT;
+extern string LENGTH_MODE;
 
 extern string FONT;
 extern string FONT_FAMILY;
@@ -41,6 +42,7 @@ extern string FONT_SERIES;
 extern string FONT_SHAPE;
 extern string FONT_SIZE;
 extern string FONT_BASE_SIZE;
+extern string FONT_EFFECTS;
 extern string MAGNIFICATION;
 extern string COLOR;
 extern string OPACITY;
@@ -49,6 +51,7 @@ extern string LOCUS_COLOR;
 extern string VISITED_COLOR;
 extern string NO_PATTERNS;
 extern string LANGUAGE;
+extern string SPACING_POLICY;
 extern string ATOM_DECORATIONS;
 extern string LINE_DECORATIONS;
 extern string PAGE_DECORATIONS;
@@ -60,12 +63,20 @@ extern string MATH_FONT;
 extern string MATH_FONT_FAMILY;
 extern string MATH_FONT_SERIES;
 extern string MATH_FONT_SHAPE;
+extern string MATH_FONT_SIZES;
 extern string MATH_LEVEL;
 extern string MATH_DISPLAY;
 extern string MATH_CONDENSED;
 extern string MATH_VPOS;
 extern string MATH_NESTING_MODE;
 extern string MATH_NESTING_LEVEL;
+extern string MATH_FRAC_LIMIT;
+extern string MATH_TABLE_LIMIT;
+extern string MATH_FLATTEN_COLOR;
+extern string MATH_TOP_SWELL_START;
+extern string MATH_TOP_SWELL_END;
+extern string MATH_BOT_SWELL_START;
+extern string MATH_BOT_SWELL_END;
 
 extern string PROG_LANGUAGE;
 extern string PROG_SCRIPTS;
@@ -78,9 +89,13 @@ extern string PROG_SESSION;
 extern string PAR_MODE;
 extern string PAR_FLEXIBILITY;
 extern string PAR_HYPHEN;
+extern string PAR_MIN_PENALTY;
 extern string PAR_SPACING;
+extern string PAR_KERNING_REDUCE;
 extern string PAR_KERNING_STRETCH;
 extern string PAR_KERNING_MARGIN;
+extern string PAR_CONTRACTION;
+extern string PAR_EXPANSION;
 extern string PAR_WIDTH;
 extern string PAR_LEFT;
 extern string PAR_RIGHT;
@@ -94,14 +109,20 @@ extern string PAR_PAR_SEP;
 extern string PAR_FNOTE_SEP;
 extern string PAR_COLUMNS;
 extern string PAR_COLUMNS_SEP;
+extern string PAR_SWELL;
 
 extern string PAGE_MEDIUM;
 extern string PAGE_PRINTED;
 extern string PAGE_TYPE;
 extern string PAGE_ORIENTATION;
+extern string PAGE_CROP_MARKS;
 extern string PAGE_WIDTH_MARGIN;
 extern string PAGE_HEIGHT_MARGIN;
 extern string PAGE_SCREEN_MARGIN;
+extern string PAGE_SINGLE;
+extern string PAGE_PACKET;
+extern string PAGE_OFFSET;
+extern string PAGE_BORDER;
 extern string PAGE_BREAKING;
 extern string PAGE_FLEXIBILITY;
 extern string PAGE_FIRST;
@@ -125,8 +146,11 @@ extern string PAGE_ODD_HEADER;
 extern string PAGE_ODD_FOOTER;
 extern string PAGE_EVEN_HEADER;
 extern string PAGE_EVEN_FOOTER;
+extern string PAGE_THIS_TOP;
+extern string PAGE_THIS_BOT;
 extern string PAGE_THIS_HEADER;
 extern string PAGE_THIS_FOOTER;
+extern string PAGE_THIS_BG_COLOR;
 extern string PAGE_SCREEN_WIDTH;
 extern string PAGE_SCREEN_HEIGHT;
 extern string PAGE_SCREEN_LEFT;
@@ -137,6 +161,7 @@ extern string PAGE_SHOW_HF;
 extern string PAGE_FNOTE_SEP;
 extern string PAGE_FNOTE_BARLEN;
 extern string PAGE_FLOAT_SEP;
+extern string PAGE_FLOAT_ENABLE;
 extern string PAGE_MNOTE_SEP;
 extern string PAGE_MNOTE_WIDTH;
 
@@ -190,6 +215,7 @@ extern string CELL_HYPHEN;
 extern string CELL_BLOCK;
 extern string CELL_ROW_NR;
 extern string CELL_COL_NR;
+extern string CELL_SWELL;
 
 extern string GR_GEOMETRY;
 extern string GR_FRAME;
@@ -201,15 +227,22 @@ extern string GR_GRID_ASPECT;
 extern string GR_EDIT_GRID;
 extern string GR_EDIT_GRID_ASPECT;
 extern string GR_TRANSFORMATION;
+extern string GR_SNAP_DISTANCE;
 
+extern string GR_GID;
+extern string GR_ANIM_ID;
+extern string GR_PROVISO;
 extern string GR_MAGNIFY;
 extern string GR_OPACITY;
 extern string GR_COLOR;
 extern string GR_POINT_STYLE;
+extern string GR_POINT_SIZE;
+extern string GR_POINT_BORDER;
 extern string GR_LINE_WIDTH;
 extern string GR_LINE_JOIN;
 extern string GR_LINE_CAPS;
 extern string GR_LINE_EFFECTS;
+extern string GR_LINE_PORTION;
 extern string GR_DASH_STYLE;
 extern string GR_DASH_STYLE_UNIT;
 extern string GR_ARROW_BEGIN;
@@ -221,14 +254,25 @@ extern string GR_FILL_STYLE;
 extern string GR_TEXT_AT_HALIGN;
 extern string GR_TEXT_AT_VALIGN;
 extern string GR_TEXT_AT_MARGIN;
+extern string GR_DOC_AT_VALIGN;
+extern string GR_DOC_AT_WIDTH;
+extern string GR_DOC_AT_HMODE;
+extern string GR_DOC_AT_PPSEP;
+extern string GR_DOC_AT_BORDER;
+extern string GR_DOC_AT_PADDING;
 
 extern string GID;
+extern string ANIM_ID;
+extern string PROVISO;
 extern string MAGNIFY;
 extern string POINT_STYLE;
+extern string POINT_SIZE;
+extern string POINT_BORDER;
 extern string LINE_WIDTH;
 extern string LINE_JOIN;
 extern string LINE_CAPS;
 extern string LINE_EFFECTS;
+extern string LINE_PORTION;
 extern string DASH_STYLE;
 extern string DASH_STYLE_UNIT;
 extern string ARROW_BEGIN;
@@ -240,11 +284,18 @@ extern string FILL_STYLE;
 extern string TEXT_AT_HALIGN;
 extern string TEXT_AT_VALIGN;
 extern string TEXT_AT_MARGIN;
+extern string DOC_AT_VALIGN;
+extern string DOC_AT_WIDTH;
+extern string DOC_AT_HMODE;
+extern string DOC_AT_PPSEP;
+extern string DOC_AT_BORDER;
+extern string DOC_AT_PADDING;
 
 extern string SRC_STYLE;
 extern string SRC_SPECIAL;
 extern string SRC_COMPACT;
 extern string SRC_CLOSE;
+extern string SRC_TAG_COLOR;
 
 extern string CANVAS_TYPE;
 extern string CANVAS_COLOR;
@@ -257,6 +308,7 @@ extern string ORNAMENT_SHAPE;
 extern string ORNAMENT_TITLE_STYLE;
 extern string ORNAMENT_BORDER;
 extern string ORNAMENT_SWELL;
+extern string ORNAMENT_CORNER;
 extern string ORNAMENT_HPADDING;
 extern string ORNAMENT_VPADDING;
 extern string ORNAMENT_COLOR;
