@@ -109,6 +109,7 @@ converter_rep::load () {
     hashtree_from_dictionary (dic,"cork-unicode-oneway", BIT2BIT, UTF8, false);
     hashtree_from_dictionary (dic,"tmuniversaltounicode", BIT2BIT, UTF8, false);
     hashtree_from_dictionary (dic,"symbol-unicode-oneway", BIT2BIT, UTF8, false);
+    hashtree_from_dictionary (dic,"symbol-unicode-fallback", BIT2BIT, UTF8, false);
     hashtree_from_dictionary (dic,"symbol-unicode-math", BIT2BIT, UTF8, false);
     ht = dic;
   }
@@ -120,6 +121,15 @@ converter_rep::load () {
     hashtree_from_dictionary (dic,"unicode-symbol-oneway", UTF8, BIT2BIT, true);
     ht = dic;
   }
+  if (from=="Strict-Cork" && to=="UTF-8" ) {
+    hashtree<char,string> dic;
+    hashtree_from_dictionary (dic,"corktounicode", BIT2BIT, UTF8, false);
+    hashtree_from_dictionary (dic,"cork-unicode-oneway", BIT2BIT, UTF8, false);
+    hashtree_from_dictionary (dic,"tmuniversaltounicode", BIT2BIT, UTF8, false);
+    hashtree_from_dictionary (dic,"symbol-unicode-oneway", BIT2BIT, UTF8, false);
+    hashtree_from_dictionary (dic,"symbol-unicode-math", BIT2BIT, UTF8, false);
+    ht = dic;
+  }
   else if (from=="UTF-8" && to=="HTML") {
     hashtree<char,string> dic;
     hashtree_from_dictionary (dic, "HTMLlat1"   , CHAR_ENTITY, ENTITY_NAME, true);
@@ -133,6 +143,7 @@ converter_rep::load () {
     hashtree_from_dictionary (dic,"cork-unicode-oneway", BIT2BIT, UTF8, false);
     hashtree_from_dictionary (dic,"tmuniversaltounicode", BIT2BIT, UTF8, false);
     hashtree_from_dictionary (dic,"symbol-unicode-oneway", BIT2BIT, UTF8, false);
+    hashtree_from_dictionary (dic,"symbol-unicode-fallback", BIT2BIT, UTF8, false);
     hashtree_from_dictionary (dic,"symbol-unicode-math", BIT2BIT, UTF8, false);
     hashtree_from_dictionary (dic,"t2atounicode", BIT2BIT, UTF8, false);
     ht = dic;
@@ -181,6 +192,7 @@ converter_rep::load () {
       //hashtree_from_dictionary (dic,"cork-unicode-oneway", BIT2BIT, UTF8, false);
     hashtree_from_dictionary (dic,"tmuniversaltounicode", BIT2BIT, UTF8, false);
     hashtree_from_dictionary (dic,"symbol-unicode-oneway", BIT2BIT, UTF8, false);
+    hashtree_from_dictionary (dic,"symbol-unicode-fallback", BIT2BIT, UTF8, false);
     hashtree_from_dictionary (dic,"symbol-unicode-math", BIT2BIT, UTF8, false);
     hashtree_from_dictionary (dic,"cork-to-real-ascii", BIT2BIT, BIT2BIT, false);
     ht = dic;
@@ -347,6 +359,23 @@ cork_to_utf8 (string input) {
   return r;
 }
 
+string
+strict_cork_to_utf8 (string input) {
+  converter conv= load_converter ("Strict-Cork", "UTF-8");
+  int start= 0, i, n= N(input);
+  string r;
+  for (i=0; i<n; i++)
+    if (input[i] == '<' && i+1<n && input[i+1] == '#') {
+      r << apply (conv, input (start, i));
+      start= i= i+2;
+      while (i<n && input[i] != '>') i++;
+      r << encode_as_utf8 (from_hexadecimal (input (start, i)));
+      start= i+1;
+    }
+  r << apply (conv, input (start, n));
+  return r;
+}
+
 string
 cork_to_sourcecode (string input) {
   converter conv= load_converter ("Cork", "SourceCode");
@@ -453,6 +482,7 @@ cork_to_ascii (string input) {
   return apply (conv, input);
 }
 
+
 #ifdef USE_ICONV
 
 class iconv_converter {
@@ -490,10 +520,10 @@ iconv_converter::~iconv_converter () {
 template<typename T>
 inline size_t
 iconv_adaptor(size_t(*iconv_func)(iconv_t, T, size_t *, char**, size_t*),
-	      iconv_t cd, char **inbuf, size_t *inbytesleft,
-	      char **outbuf, size_t *outbytesleft) {
+              iconv_t cd, char **inbuf, size_t *inbytesleft,
+              char **outbuf, size_t *outbytesleft) {
   return iconv_func (cd, (T) ((void*) inbuf), inbytesleft,
-		     outbuf, outbytesleft);
+                     outbuf, outbytesleft);
 }
 
 string apply (iconv_converter &conv, string input) {
@@ -513,10 +543,10 @@ string apply (iconv_converter &conv, string input) {
     c_string out_cp (out_left);
     char* out_cursor= out_cp;
     size_t r = iconv_adaptor(iconv, conv.cd,
-			     &in_cursor, &in_left, &out_cursor, &out_left);
+                             &in_cursor, &in_left, &out_cursor, &out_left);
     if(r == (size_t)-1 && errno != E2BIG) {
       if (conv.show_errors) {
-	convert_error << "Iconv conversion from " << conv.from
+        convert_error << "Iconv conversion from " << conv.from
                       << " to " << conv.to << " failed\n";
       }
       conv.successful= false;
@@ -568,7 +598,7 @@ convert_using_iconv (string input, string from, string to) {
 void
 put_prefix_code (string key, string value, hashtree<char,string> tree) {
   /*
-  if (DEBUG_STD) {
+  if (DEBUG_CONVERT) {
     hashtree<char,string> ht= find_node (key,tree);
     if (ht->label != "")
       debug_convert << "overwriting " << ht->label << " with " << value << '\n';
@@ -598,7 +628,7 @@ hashtree_from_dictionary (
   hashtree<char,string> dic, string file_name, escape_type key_escape,
   escape_type val_escape, bool reverse)
 {
-  if (DEBUG_STD) debug_convert << "Loading dictionary " << file_name << LF;
+  if (DEBUG_CONVERT) debug_convert << "Loading dictionary " << file_name << LF;
   string key_string, val_string, file;
   file_name = file_name * ".scm";
   if (load_string (url ("$TEXMACS_PATH/langs/encoding", file_name),
@@ -624,14 +654,14 @@ hashtree_from_dictionary (
           key_string = convert_escapes (key_string, false);
         else if (key_escape == UTF8)
           key_string = convert_escapes (key_string, true);
-	else if (key_escape == CHAR_ENTITY)
-	  key_string = convert_char_entities (key_string);
+        else if (key_escape == CHAR_ENTITY)
+          key_string = convert_char_entities (key_string);
         if (val_escape == BIT2BIT)
           val_string = convert_escapes (val_string, false);
         else if (val_escape == UTF8)
           val_string = convert_escapes (val_string, true);
-	else if (val_escape == ENTITY_NAME)
-	  val_string = "&" * val_string * ";";
+        else if (val_escape == ENTITY_NAME)
+          val_string = "&" * val_string * ";";
         //cout << "key: " << key_string << " val: " << val_string << "\n";
         put_prefix_code(key_string,val_string,dic);        
       }
@@ -644,13 +674,6 @@ hashtree_from_dictionary (
 * and HTML/XML character entities to and from UTF-8 byte sequences.
 ***************************************************************************/
 
-bool is_hex_digit (char c) {
-  return
-    (48 <= c && c <= 57) ||
-    (65 <= c && c <= 70) ||
-    (97 <= c && c <= 102);
-}
-
 int hex_digit_to_int(unsigned char c) {
   if (48 <= c && c <= 57)
     return c - 0x30;
@@ -866,3 +889,8 @@ utf8_to_hex_string (string s) {
   }
   return result;
 }
+
+string
+utf8_to_pdf_hex_string (string s) {
+  return "<FEFF" * utf8_to_hex_string (cork_to_utf8 (s)) * ">";
+}
\ No newline at end of file
