@@ -12,7 +12,7 @@
 #ifndef EDIT_INTERFACE_H
 #define EDIT_INTERFACE_H
 #include "editor.hpp"
-#include "timer.hpp"
+#include "tm_timer.hpp"
 #include "widget.hpp"
 
 #define INPUT_NORMAL      0
@@ -28,12 +28,12 @@ protected:
   int           env_change;    // which things have been changed ?
   time_t        last_change;   // time of last processed change
   time_t        last_update;   // time of last update of menu, icons and footer
-  bool          do_animate;    // are we in an animation ?
-  time_t        next_animate;  // time for next animation
+  double        anim_next;     // time for next animation
   bool          full_screen;   // full screen mode ?
   bool          got_focus;     // do we have keyboard focus ?
   string        sh_s;          // current string for shortcuts
   double        sh_mark;       // 0 or mark for undoing shortcut
+  bool          pre_edit_skip; // temporarily disabled pre-edit mechanism
   string        pre_edit_s;    // pre-edit string
   double        pre_edit_mark; // 0 or mark for undoing pre-edit
   widget        popup_win;     // the current popup window
@@ -52,10 +52,13 @@ protected:
   time_t        last_t;
   SI            start_x, start_y;
   SI            end_x, end_y;
+  int           tremble_count;
+  bool          tremble_right;
   bool          table_selection;
   int           mouse_adjusting;  // mask with active key modifiers upon click
   rectangles    selection_rects;
   array<rectangles> alt_selection_rects;
+  rectangle     last_visible;
   rectangles    env_rects;
   rectangles    foc_rects;
   rectangles    sem_rects;
@@ -75,6 +78,7 @@ protected:
   list<string>  focus_ids;
   int           cur_sb, cur_wb;
   SI            cur_wx, cur_wy;
+  rectangles    keys_rects;
 
 public:
   edit_interface_rep ();
@@ -91,6 +95,12 @@ public:
   SI   get_visible_height ();
   SI   get_window_width ();
   SI   get_window_height ();
+  SI   get_window_x ();
+  SI   get_window_y ();
+  SI   get_canvas_x ();
+  SI   get_canvas_y ();
+  SI   get_scroll_x ();
+  SI   get_scroll_y ();
   void set_zoom_factor (double zoom);
   void invalidate (SI x1, SI y1, SI x2, SI y2);
   void invalidate (rectangles rs);
@@ -100,6 +110,7 @@ public:
   void set_extents (SI x1, SI y1, SI x2, SI y2);
 
   /* repainting the window */
+  void draw_background (renderer ren, SI x1, SI y1, SI x2, SI y2);
   void draw_text (renderer ren, rectangles& l);
   void draw_surround (renderer ren, rectangle r);
   void draw_context (renderer ren, rectangle r);
@@ -107,6 +118,7 @@ public:
   void draw_cursor (renderer ren);
   void draw_selection (renderer ren, rectangle r);
   void draw_graphics (renderer ren);
+  void draw_keys (renderer ren);
   void draw_pre (renderer win, renderer ren, rectangle r);
   void draw_post (renderer win, renderer ren, rectangle r);
   void draw_with_shadow (renderer win, rectangle r);
@@ -117,6 +129,8 @@ public:
   bool has_changed (int question);
   int  idle_time (int event_type= ANY_EVENT);
   int  change_time ();
+  void update_menus ();
+  int  find_alt_selection_index (range_set alt_sel, SI y, int b, int e);
   void apply_changes ();
   void animate ();
 
