@@ -14,6 +14,24 @@
 #include "tm_buffer.hpp"
 #include "archiver.hpp"
 
+#ifdef Q_OS_MAC
+extern hashmap<int,string> qtcomposemap;
+#endif
+
+/******************************************************************************
+* Showing the keystrokes while typing
+******************************************************************************/
+
+bool          kbd_show_keys= false;
+array<string> kbd_shown_keys;
+array<string> kbd_last_keys;
+array<time_t> kbd_last_times;
+int           kbd_erase_delay= 1500;
+int           kbd_hide_delay = 3000;
+
+bool get_show_kbd () { return kbd_show_keys; }
+void set_show_kbd (bool flag) { kbd_show_keys= flag; }
+
 /******************************************************************************
 * Basic subroutines for keyboard handling
 ******************************************************************************/
@@ -108,7 +126,7 @@ edit_interface_rep::try_shortcut (string comb) {
     tree rhs= (shorth == rew_s? tree (""): sv->kbd_system_rewrite (shorth));
     //cout << "Shortcut: " << sh_s << " -> " << rew << "\n";
     if ((search_forwards (" ", comb) >= 0 && comb != " ") ||
-	(search_forwards ("-", comb) >= 0 && comb != "-")) {
+        (search_forwards ("-", comb) >= 0 && comb != "-")) {
       tree t= rhs;
       if (is_compound (t, "render-key", 1)) t= t[0];
       if (is_func (t, WITH)) t= t[N(t)-1];
@@ -117,8 +135,9 @@ edit_interface_rep::try_shortcut (string comb) {
         if (cork_to_utf8 (r) != r)
           rhs= tree (CONCAT, rhs, " (" * r(1, N(r)-1) * ")");
       call ("set-temporary-message",
-	    tree (CONCAT, "keyboard shortcut: ", rew), rhs,
-	    shorth == ""? 1: 3000);
+            tree (CONCAT, "keyboard shortcut: ", rew),
+            verbatim (rhs),
+            shorth == ""? 1: 3000);
     }
     if ((status & 1) == 1) cmd ();
     else if (N(shorth) > 0) call ("kbd-insert", shorth);
@@ -129,6 +148,22 @@ edit_interface_rep::try_shortcut (string comb) {
   return false;
 }
 
+static string
+std_accent (string s) {
+  string c= "x";
+  c[0]= '\0';
+  s= replace (s, c, "`");
+  c[0]= '\1';
+  s= replace (s, c, "'");
+  c[0]= '\2';
+  s= replace (s, c, "^");
+  c[0]= '\3';
+  s= replace (s, c, "~");
+  c[0]= '\4';
+  s= replace (s, c, "\"");
+  return s;
+}
+
 void
 edit_interface_rep::key_press (string gkey) {
   string zero= "a"; zero[0]= '\0';
@@ -139,10 +174,8 @@ edit_interface_rep::key_press (string gkey) {
     pre_edit_s= "";
     pre_edit_mark= 0;
   }
-  if (starts (key, "pre-edit:") ) {
-    interrupt_shortcut ();
+  if (starts (key, "pre-edit:")) {
     string s= key (9, N(key));
-    if (s == "") return;
     int i, n= N(s), pos= N(s);
     for (i=0; i<n; i++)
       if (s[i] == ':' && is_int (s (0, i))) {
@@ -153,12 +186,32 @@ edit_interface_rep::key_press (string gkey) {
           tm_char_forwards (s, pos);
         break;
       }
-    pre_edit_s= s;
-    pre_edit_mark= new_marker ();
-    mark_start (pre_edit_mark);
-    archive_state ();
-    insert_tree (compound ("pre-edit", s), path (0, pos));
-    return;
+    if (as_bool (call ("disable-pre-edit?", std_accent (s)))) {
+      pre_edit_skip= false;
+      if (s == "") return;
+      pre_edit_skip= true;
+      key= std_accent (s);
+    }
+    else if (pre_edit_skip) {
+      if (s == "") pre_edit_skip= false;
+      return;
+    }
+    else {
+      if (s == "") return;
+      interrupt_shortcut ();
+      pre_edit_s= s;
+      pre_edit_mark= new_marker ();
+      mark_start (pre_edit_mark);
+      archive_state ();
+      insert_tree (compound ("pre-edit", s), path (0, pos));
+      return;
+    }
+  }
+  else if (pre_edit_skip) {
+    string u= cork_to_utf8 (key);
+    string r= as_string (call ("downgrade-pre-edit", u));
+    if (r == "") return;
+    else key= r;
   }
   
   string new_sh= N(sh_s)==0? key: sh_s * " " * key;
@@ -171,7 +224,7 @@ edit_interface_rep::key_press (string gkey) {
   string rew= sv->kbd_post_rewrite (key);
   if (N(rew) == 1) {
     int i ((unsigned char) rew[0]);
-    if ((i >= 32 && i <= 127) || (i >= 128 && i <= 255) || (i == 25))
+    if ((i >= 1 && i <= 127) || (i >= 128 && i <= 255) || (i == 25))
       if (!inside_active_graphics ()) {
         archive_state ();
         call ("kbd-insert", rew);
@@ -183,6 +236,20 @@ edit_interface_rep::key_press (string gkey) {
     call ("kbd-insert", key);
     interrupt_shortcut ();    
   }
+#ifdef Q_OS_MAC
+  else if (N(key) == 3 && starts (key, "A-") &&
+           qtcomposemap->contains ((int) (unsigned char) key[2])) {
+    string compose_key= qtcomposemap[(int) (unsigned char) key[2]];
+    key_press (compose_key);
+  }
+#endif
+  else if (!occurs (" ", key) && N(key) > 1 && key[1] != '-' &&
+           cork_to_utf8 ("<" * key * ">") != ("<" * key * ">") &&
+           !inside_active_graphics ()) {
+    archive_state ();
+    call ("kbd-insert", "<" * key * ">");
+    interrupt_shortcut ();    
+  }
   else if (DEBUG_KEYBOARD)
     debug_keyboard
       << "unrecognized key " << key << ". "
@@ -202,7 +269,7 @@ edit_interface_rep::emulate_keyboard (string keys, string action) {
   }
   if (N (action) != 0)
     set_message (concat ("You can also obtain ", action, " by typing ", keys),
-		 action);
+                 action);
 }
 
 /******************************************************************************
@@ -226,15 +293,31 @@ edit_interface_rep::kbd_shortcut (string cmd) {
 
 void
 edit_interface_rep::handle_keypress (string key, time_t t) {
+  if (is_nil (buf)) return;
   bool started= false;
 #ifdef USE_EXCEPTIONS
   try {
 #endif
+    if (kbd_show_keys) {
+      if (N(kbd_last_times) > 0 &&
+          kbd_last_times[N(kbd_last_times)-1] + kbd_erase_delay < t) {
+        kbd_last_keys = array<string> ();
+        kbd_last_times= array<time_t> ();
+      }
+      if (!starts (key, "pre-edit:")) {
+        kbd_last_keys  << key;
+        kbd_last_times << t;
+      }
+    }
     if (DEBUG_KEYBOARD) {
       //for (int i=0; i<N(key); i++)
       //  cout << ((int) (unsigned char) key[i]) << " ";
       //cout << "\n";
       debug_keyboard << "Pressed " << key << " at " << t << "\n";
+      debug_keyboard << "  Codes";
+      for (int i=0; i<N(key); i++)
+        debug_keyboard << " " << (unsigned int) (unsigned char) key[i];
+      debug_keyboard << "\n";      
     }
     //time_t t1= texmacs_time ();
     if (is_nil (eb)) apply_changes ();
@@ -242,6 +325,7 @@ edit_interface_rep::handle_keypress (string key, time_t t) {
     started= true;
     string zero= "a"; zero[0]= '\0';
     string gkey= replace (key, zero, "<#0>");
+    if (gkey == "<#3000>") gkey= "space";
     call ("keyboard-press", object (gkey), object ((double) t));
     update_focus_loci ();
     if (!is_nil (focus_ids))
@@ -267,6 +351,7 @@ void drag_right_reset ();
 
 void
 edit_interface_rep::handle_keyboard_focus (bool has_focus, time_t t) {
+  if (is_nil (buf)) return;
   if (DEBUG_KEYBOARD) {
     if (has_focus) debug_keyboard << "Got focus at " << t << "\n";
     else debug_keyboard << "Lost focus at " << t << "\n";
@@ -276,6 +361,7 @@ edit_interface_rep::handle_keyboard_focus (bool has_focus, time_t t) {
     drag_right_reset ();
   }
   got_focus= has_focus; (void) t;
+  notify_change (THE_FREEZE);
   notify_change (THE_FOCUS);
   if (got_focus) {
     focus_on_this_editor ();
