@@ -93,29 +93,6 @@ edit_select_rep::semantic_active (path q) {
   return false;
 }
 
-static bool
-right_most_inside (path p, tree t) {
-  if (is_atomic (t)) return p == path (N(t->label));
-  else if (p == path (1)) return true;
-  if (is_nil (p) || is_nil (p->next)) return false;
-  if (N(t) == 1 && p->item == 0)
-    return right_most_inside (p->next, t[0]);
-  if (is_func (t, WIDE, 2))
-    return p->item == 0 && right_most_inside (p->next, t[0]);
-  if (is_func (t, CONCAT) || is_func (t, DOCUMENT))
-    return p->item == N(t) - 1 && right_most_inside (p->next, t[N(t) - 1]);
-  return false;
-}
-
-static path
-correct_right_most_inside (path p, tree t) {
-  if (is_nil (p) || is_nil (p->next) || is_atomic (t)) return p;
-  if (0 > p->item || p->item >= N(t)) return p;
-  p= path (p->item, correct_right_most_inside (p->next, t[p->item]));
-  if (right_most_inside (p, t)) return path (1);
-  else return p;
-}
-
 bool
 edit_select_rep::semantic_select (path p, path& q1, path& q2, int mode) {
   if (!semantic_active (p)) return false;
@@ -265,7 +242,7 @@ edit_select_rep::select_enlarge_text () {
 
     if (i1 < i2 && (i1 != j1 || i2 != j2)) {
       if (is_concat (subtree (et, q)) && i1 == 0 && i2 == N(s))
-	select (q * 0, q * 1);
+        select (q * 0, q * 1);
       else select (p * i1, p * i2);
       return;
     }
@@ -404,8 +381,7 @@ void
 edit_select_rep::selection_correct (path i1, path i2, path& o1, path& o2) {
   ASSERT (rp <= i1 && rp <= i2, "paths not inside document");
   int old_mode= get_access_mode ();
-  if (get_init_string (MODE) == "src")
-    set_access_mode (DRD_ACCESS_SOURCE);
+  if (in_source ()) set_access_mode (DRD_ACCESS_SOURCE);
   ::selection_correct (subtree (et, rp), i1 / rp, i2 / rp, o1, o2);
   set_access_mode (old_mode);
   o1= rp * o1; o2= rp * o2;
@@ -433,13 +409,13 @@ edit_select_rep::compute_selection (path p1, path p2) {
     rectangle r (0, 0, 0, 0);
     for (i=row1; i<=row2; i++)
       for (j=col1; j<=col2; j++) {
-	path cp= fp * ::table_search_cell (st, i, j);
-	selection sel= eb->find_check_selection (cp * 0, cp * 1);
-	if (sel->valid) {
-	  rectangles rs= sel->rs;
-	  if (r != rectangle (0, 0, 0, 0)) rs= rectangles (r, rs);
-	  r= least_upper_bound (rs);
-	}
+        path cp= fp * ::table_search_cell (st, i, j);
+        selection sel= eb->find_check_selection (cp * 0, cp * 1);
+        if (sel->valid) {
+          rectangles rs= sel->rs;
+          if (r != rectangle (0, 0, 0, 0)) rs= rectangles (r, rs);
+          r= least_upper_bound (rs);
+        }
       }
     return selection (rectangles (r), fp * 0, fp * 1);
   }
@@ -505,6 +481,20 @@ edit_select_rep::selection_get_end () {
   return end (cur_sel);
 }
 
+path
+edit_select_rep::selection_var_get_start () {
+  path p1, p2;
+  selection_get (p1, p2);
+  return p1;
+}
+
+path
+edit_select_rep::selection_var_get_end () {
+  path p1, p2;
+  selection_get (p1, p2);
+  return p2;
+}
+
 tree
 edit_select_rep::selection_get () {
   if (!selection_active_any ()) return "";
@@ -632,9 +622,10 @@ edit_select_rep::selection_set (tree t) {
 
 void
 edit_select_rep::selection_copy (string key) {
+  bool emacs= (get_preference ("look and feel", "default") == "emacs");
   if (inside_active_graphics ()) {
     tree t= as_tree (eval ("(graphics-copy)"));
-    selection_set (key, t);
+    selection_set (key, t, !emacs);
     return;
   }
   if (selection_active_any ()) {
@@ -643,11 +634,18 @@ edit_select_rep::selection_copy (string key) {
     go_to (sel->end);
     tree t= selection_get ();
     go_to (sel->start);
-    selection_set (key, t);
+    selection_set (key, t, !emacs);
     go_to (old_tp);
   }
 }
 
+tree
+edit_select_rep::selection_get (string key) {
+  tree t; string s;
+  (void) ::get_selection (key, t, s, selection_import);
+  return t;
+}
+
 void
 edit_select_rep::selection_paste (string key) {
   tree t; string s;
@@ -672,7 +670,8 @@ edit_select_rep::selection_paste (string key) {
           s= "$" * s * "$";
       }
     string fm;
-    if (selection_import == "default") fm= "texmacs-snippet";
+    if (selection_import == "verbatim" && mode == "prog") fm= "code-snippet";
+    else if (selection_import == "default") fm= "texmacs-snippet";
     else fm= selection_import * "-snippet";
     tree doc= generic_to_tree (s, fm);
     if (is_func (doc, DOCUMENT, 1)) doc= doc[0]; // temporary fix
@@ -695,7 +694,7 @@ edit_select_rep::selection_paste (string key) {
     }
     else {
       if ((t[2] != mode) && (t[2] != "src") && (mode != "src") &&
-	  ((t[2] == "math") || (mode == "math"))) {
+  ((t[2] == "math") || (mode == "math"))) {
         if (t[2] == "math")
           insert_tree (compound ("math", ""), path (0, 0));
         else if (t[2] == "text")
@@ -704,10 +703,10 @@ edit_select_rep::selection_paste (string key) {
           insert_tree (tree (WITH, copy (MODE), copy (t[2]), ""), path (2, 0));
       }
       if (is_func (t[1], TFORMAT) || is_func (t[1], TABLE)) {
-	int row, col;
-	path fp= search_format (row, col);
-	if (is_nil (fp)) insert_tree (compound (copy (TABULAR), t[1]));
-	else table_write_subtable (fp, row, col, t[1]);
+        int row, col;
+        path fp= search_format (row, col);
+        if (is_nil (fp)) insert_tree (compound (copy (TABULAR), t[1]));
+        else table_write_subtable (fp, row, col, t[1]);
       }
       else insert_tree (t[1]);
     }
@@ -946,6 +945,10 @@ edit_select_rep::focus_search (path p, bool skip_flag, bool up_flag) {
       is_compound (st, "shown") ||
       is_func (st, HIDDEN) ||
       is_compound (st, "shared") ||
+      is_compound (st, "slide") ||
+      is_compound (st, "with-screen-color") ||
+      is_compound (st, "mc-field") ||
+      is_compound (st, "live-io*") ||
       up_flag)
     return focus_search (path_up (p), skip_flag, false);
   return p;
@@ -958,8 +961,21 @@ edit_select_rep::focus_get (bool skip_flag) {
     return focus_search (focus_p, skip_flag, false);
   if (selection_active_any ())
     return focus_search (selection_get_path (), skip_flag, false);
-  else
+  else {
+    tree st= subtree (et, path_up (tp));
+    if (is_compound (st, "draw-over")) skip_flag= false;
+    if (is_compound (st, "draw-under")) skip_flag= false;
+    if (is_compound (st, "float")) skip_flag= false;
+    if (is_compound (st, "wide-float")) skip_flag= false;
+    if (is_compound (st, "footnote")) skip_flag= false;
+    if (is_compound (st, "footnote-anchor")) skip_flag= false;
+    if (is_compound (st, "wide-footnote")) skip_flag= false;
+    if (is_compound (st, "note-footnote")) skip_flag= false;
+    if (is_compound (st, "note-footnote*")) skip_flag= false;
+    if (is_compound (st, "folded-comment")) skip_flag= false;
+    if (is_compound (st, "cite-detail")) skip_flag= false;
     return focus_search (path_up (tp), skip_flag, true);
+  }
 }
 
 /******************************************************************************
@@ -979,6 +995,11 @@ edit_select_rep::get_alt_selection (string name) {
   return alt_sels[name];
 }
 
+range_set
+current_alt_selection (string name) {
+  return get_current_editor()->get_alt_selection (name);
+}
+
 void
 edit_select_rep::cancel_alt_selection (string name) {
   if (alt_sels->contains (name)) {
