@@ -58,12 +58,44 @@ public:
   argument_effect_rep (int nr2): nr (nr2) {}
   rectangle get_logical_extents (array<rectangle> rs) { return rs[nr]; }
   rectangle get_extents (array<rectangle> rs) { return rs[nr]; }
-  picture apply (array<picture> pics, SI pixel) { return pics[nr]; }
+  picture apply (array<picture> pics, SI pixel) {
+    (void) pixel; return pics[nr]; }
 };
 
 effect argument_effect (int nr) {
   return tm_new<argument_effect_rep> (nr); }
 
+/******************************************************************************
+* Turbulence effect
+******************************************************************************/
+
+class turbulence_effect_rep: public effect_rep {
+  effect eff;
+  long seed;
+  double w, h;
+  int octaves;
+  bool frac_sum;
+public:
+  turbulence_effect_rep (effect e, long s, double w2, double h2, int o, bool f):
+    eff (e), seed (s), w (w2), h (h2), octaves (o), frac_sum (f) {}
+  rectangle get_logical_extents (array<rectangle> rs) {
+    return eff->get_logical_extents (rs); }
+  rectangle get_extents (array<rectangle> rs) {
+    return eff->get_extents (rs); }
+  picture apply (array<picture> pics, SI pixel) {
+    if (frac_sum)
+      return fractal_noise (eff->apply (pics, pixel), seed,
+                            w / pixel, h / pixel, octaves);
+    else
+      return turbulence (eff->apply (pics, pixel), seed,
+                         w / pixel, h / pixel, octaves); }
+};
+
+effect turbulence (effect e, long s, double w, double h, int oct) {
+  return tm_new<turbulence_effect_rep> (e, s, w, h, oct, false); }
+effect fractal_noise (effect e, long s, double w, double h, int oct) {
+  return tm_new<turbulence_effect_rep> (e, s, w, h, oct, true); }
+
 /******************************************************************************
 * Coordinate transformations
 ******************************************************************************/
@@ -74,6 +106,10 @@ class move_effect_rep: public effect_rep {
 public:
   move_effect_rep (effect eff2, double dx2, double dy2):
     eff (eff2), dx (dx2), dy (dy2) {}
+  rectangle get_logical_extents (array<rectangle> rs) {
+    rectangle r= eff->get_logical_extents (rs);
+    return rectangle ((SI) round (r->x1 + dx), (SI) round (r->y1 + dy),
+                      (SI) round (r->x2 + dx), (SI) round (r->y2 + dy)); }
   rectangle get_extents (array<rectangle> rs) {
     rectangle r= eff->get_extents (rs);
     return rectangle ((SI) floor (r->x1 + dx), (SI) floor (r->y1 + dy),
@@ -88,6 +124,10 @@ class magnify_effect_rep: public effect_rep {
 public:
   magnify_effect_rep (effect eff2, double sx2, double sy2):
     eff (eff2), sx (sx2), sy (sy2) {}
+  rectangle get_logical_extents (array<rectangle> rs) {
+    rectangle r= eff->get_logical_extents (rs);
+    return rectangle ((SI) round (sx * r->x1), (SI) round (sy * r->y1),
+                      (SI) round (sx * r->x2), (SI) round (sy * r->y2)); }
   rectangle get_extents (array<rectangle> rs) {
     rectangle r= eff->get_extents (rs);
     return rectangle ((SI) floor (sx * r->x1), (SI) floor (sy * r->y1),
@@ -113,12 +153,40 @@ public:
     return bubble (eff->apply (pics, pixel), r / pixel, a); }
 };
 
+class crop_effect_rep: public effect_rep {
+  effect eff;
+  double cx1, cy1, cx2, cy2;
+public:
+  crop_effect_rep (effect eff2, double x1, double y1, double x2, double y2):
+    eff (eff2), cx1 (x1), cy1 (y1), cx2 (x2), cy2 (y2) {}
+  rectangle get_logical_extents (array<rectangle> rs) {
+    rectangle r= eff->get_logical_extents (rs);
+    SI x1= (SI) round (r->x1 + cx1 * (r->x2 - r->x1));
+    SI y1= (SI) round (r->y1 + cy1 * (r->y2 - r->y1));
+    SI x2= (SI) round (r->x1 + cx2 * (r->x2 - r->x1));
+    SI y2= (SI) round (r->y1 + cy2 * (r->y2 - r->y1));
+    return rectangle (0, 0, x2 - x1, y2 - y1); }
+  rectangle get_extents (array<rectangle> rs) {
+    rectangle r= eff->get_extents (rs);
+    SI x1= (SI) floor (r->x1 + cx1 * (r->x2 - r->x1));
+    SI y1= (SI) floor (r->y1 + cy1 * (r->y2 - r->y1));
+    SI x2= (SI) ceil  (r->x1 + cx2 * (r->x2 - r->x1));
+    SI y2= (SI) ceil  (r->y1 + cy2 * (r->y2 - r->y1));
+    return rectangle (0, 0, x2 - x1, y2 - y1); }
+  picture apply (array<picture> pics, SI pixel) {
+    return crop (eff->apply (pics, pixel), cx1, cy1, cx2, cy2); }
+};
+
 effect move (effect eff, double dx, double dy) {
   return tm_new<move_effect_rep> (eff, dx, dy); }
 effect magnify (effect eff, double sx, double sy) {
   return tm_new<move_effect_rep> (eff, sx, sy); }
 effect bubble (effect eff, double r, double a) {
   return tm_new<bubble_effect_rep> (eff, r, a); }
+effect crop (effect eff, double cx1, double cy1, double cx2, double cy2) {
+  cx1= max (cx1, 0.0); cy1= max (cy1, 0.0);
+  cx2= min (cx2, 1.0); cy2= min (cy2, 1.0);
+  return tm_new<crop_effect_rep> (eff, cx1, cy1, cx2, cy2); }
 
 /******************************************************************************
 * Pens
@@ -266,6 +334,58 @@ effect thicken (effect eff, effect pen) {
 effect erode (effect eff, effect pen) {
   return tm_new<erode_effect_rep> (eff, pen); }
 
+/******************************************************************************
+* Effects that depend on randomization
+******************************************************************************/
+
+class degrade_effect_rep: public effect_rep {
+  effect eff;
+  double wx, wy, th, sh;
+public:
+  degrade_effect_rep (effect e2, double Wx, double Wy, double Th, double Sh):
+    eff (e2), wx (Wx), wy (Wy), th (Th), sh (Sh) {}
+  rectangle get_extents (array<rectangle> rs) {
+    return eff->get_extents (rs); }
+  picture apply (array<picture> pics, SI pixel) {
+    return degrade (eff->apply (pics, pixel),
+                    wx / pixel, wy / pixel, th, sh); }
+};
+
+effect degrade (effect eff, double wx, double wy, double th, double sh) {
+  return tm_new<degrade_effect_rep> (eff, wx, wy, th, sh); }
+
+class distort_effect_rep: public effect_rep {
+  effect eff;
+  double wx, wy, rx, ry;
+public:
+  distort_effect_rep (effect e2, double Wx, double Wy, double Rx, double Ry):
+    eff (e2), wx (Wx), wy (Wy), rx (Rx), ry (Ry) {}
+  rectangle get_extents (array<rectangle> rs) {
+    return eff->get_extents (rs); }
+  picture apply (array<picture> pics, SI pixel) {
+    return distort (eff->apply (pics, pixel),
+                    wx / pixel, wy / pixel, rx / pixel, ry / pixel); }
+};
+
+effect distort (effect eff, double wx, double wy, double rx, double ry) {
+  return tm_new<distort_effect_rep> (eff, wx, wy, rx, ry); }
+
+class gnaw_effect_rep: public effect_rep {
+  effect eff;
+  double wx, wy, rx, ry;
+public:
+  gnaw_effect_rep (effect e2, double Wx, double Wy, double Rx, double Ry):
+    eff (e2), wx (Wx), wy (Wy), rx (Rx), ry (Ry) {}
+  rectangle get_extents (array<rectangle> rs) {
+    return eff->get_extents (rs); }
+  picture apply (array<picture> pics, SI pixel) {
+    return gnaw (eff->apply (pics, pixel),
+                 wx / pixel, wy / pixel, rx / pixel, ry / pixel); }
+};
+
+effect gnaw (effect eff, double wx, double wy, double rx, double ry) {
+  return tm_new<gnaw_effect_rep> (eff, wx, wy, rx, ry); }
+
 /******************************************************************************
 * Composing various effects
 ******************************************************************************/
@@ -359,13 +479,14 @@ public:
 class make_transparent_effect_rep: public effect_rep {
   effect eff;
   color bgc;
+  double t;
 public:
-  make_transparent_effect_rep (effect eff2, color bgc2):
-    eff (eff2), bgc (bgc2) {}
+  make_transparent_effect_rep (effect eff2, color bgc2, double t2):
+    eff (eff2), bgc (bgc2), t (t2) {}
   rectangle get_extents (array<rectangle> rs) {
     return eff->get_extents (rs); }
   picture apply (array<picture> pics, SI pixel) {
-    return make_transparent (eff->apply (pics, pixel), bgc); }
+    return make_transparent (eff->apply (pics, pixel), bgc, t); }
 };
 
 class make_opaque_effect_rep: public effect_rep {
@@ -380,14 +501,85 @@ public:
     return make_opaque (eff->apply (pics, pixel), bgc); }
 };
 
+class recolor_effect_rep: public effect_rep {
+  effect eff;
+  color col;
+public:
+  recolor_effect_rep (effect eff2, color col2):
+    eff (eff2), col (col2) {}
+  rectangle get_extents (array<rectangle> rs) {
+    return eff->get_extents (rs); }
+  picture apply (array<picture> pics, SI pixel) {
+    return recolor (eff->apply (pics, pixel), col); }
+};
+
+class skin_effect_rep: public effect_rep {
+  effect eff;
+  color col;
+public:
+  skin_effect_rep (effect eff2, color col2):
+    eff (eff2), col (col2) {}
+  rectangle get_extents (array<rectangle> rs) {
+    return eff->get_extents (rs); }
+  picture apply (array<picture> pics, SI pixel) {
+    return apply_skin (eff->apply (pics, pixel), col); }
+};
+
 effect normalize (effect eff) {
   return tm_new<normalize_effect_rep> (eff); }
 effect color_matrix (effect eff, array<double> m) {
   return tm_new<color_matrix_effect_rep> (eff, m); }
-effect make_transparent (effect eff, color bgc) {
-  return tm_new<make_transparent_effect_rep> (eff, bgc); }
+effect make_transparent (effect eff, color bgc, double t) {
+  return tm_new<make_transparent_effect_rep> (eff, bgc, t); }
 effect make_opaque (effect eff, color bgc) {
   return tm_new<make_opaque_effect_rep> (eff, bgc); }
+effect recolor (effect eff, color col) {
+  return tm_new<recolor_effect_rep> (eff, col); }
+effect apply_skin (effect eff, color col) {
+  return tm_new<skin_effect_rep> (eff, col); }
+
+/******************************************************************************
+* Hatching
+******************************************************************************/
+
+class hatch_effect_rep: public effect_rep {
+  effect eff;
+  int sx, sy;
+  double fp;
+  double de;
+public:
+  hatch_effect_rep (effect e, int sx2, int sy2, double fp2, double de2):
+    eff (e), sx (sx2), sy (sy2), fp (fp2), de (de2) {}
+  rectangle get_logical_extents (array<rectangle> rs) {
+    return eff->get_logical_extents (rs); }
+  rectangle get_extents (array<rectangle> rs) {
+    return eff->get_extents (rs); }
+  picture apply (array<picture> pics, SI pixel) {
+    return hatch (eff->apply (pics, pixel), sx, sy, fp, de); }
+};
+
+effect hatch (effect e, int sx, int sy, double fp, double de) {
+  return tm_new<hatch_effect_rep> (e, sx, sy, fp, de); }
+
+class dots_effect_rep: public effect_rep {
+  effect eff;
+  int a, b, c, d;
+  double fp;
+  double de;
+public:
+  dots_effect_rep (effect e, int a2, int b2, int c2, int d2,
+                   double fp2, double de2):
+    eff (e), a (a2), b (b2), c (c2), d (d2), fp (fp2), de (de2) {}
+  rectangle get_logical_extents (array<rectangle> rs) {
+    return eff->get_logical_extents (rs); }
+  rectangle get_extents (array<rectangle> rs) {
+    return eff->get_extents (rs); }
+  picture apply (array<picture> pics, SI pixel) {
+    return dots (eff->apply (pics, pixel), a, b, c, d, fp, de); }
+};
+
+effect dots (effect e, int a, int b, int c, int d, double fp, double de) {
+  return tm_new<dots_effect_rep> (e, a, b, c, d, fp, de); }
 
 /******************************************************************************
 * Building effects from tree description
@@ -403,9 +595,10 @@ build_effects (array<tree> a) {
 
 effect
 build_effect (tree t) {
-  if (is_int (t)) {
+  if (t == "")
+    return argument_effect (0);
+  else if (is_int (t))
     return argument_effect (as_int (t));
-  }
   else if (is_func (t, EFF_MOVE, 3)) {
     effect eff= build_effect (t[0]);
     double dx = as_double (t[1]);
@@ -428,6 +621,40 @@ build_effect (tree t) {
     double r= as_double (t[0]);
     return gaussian_pen_effect (r);
   }
+  else if (is_func (t, EFF_TURBULENCE, 5)) {
+    effect e= build_effect (t[0]);
+    int    s= as_int (t[1]);
+    double w= as_double (t[2]);
+    double h= as_double (t[3]);
+    int    o= as_int (t[4]);
+    return turbulence (e, s, w, h, o);
+  }
+  else if (is_func (t, EFF_FRACTAL_NOISE, 5)) {
+    effect e= build_effect (t[0]);
+    int    s= as_int (t[1]);
+    double w= as_double (t[2]);
+    double h= as_double (t[3]);
+    int    o= as_int (t[4]);
+    return fractal_noise (e, s, w, h, o);
+  }
+  else if (is_compound (t, EFF_HATCH, 5)) {
+    effect e = build_effect (t[0]);
+    int    sx= as_int (t[1]);
+    int    sy= as_int (t[2]);
+    double fp= as_double (t[3]);
+    double de= as_double (t[4]);
+    return hatch (e, sx, sy, fp, de);
+  }
+  else if (is_compound (t, EFF_DOTS, 7)) {
+    effect e = build_effect (t[0]);
+    int    a = as_int (t[1]);
+    int    b = as_int (t[2]);
+    int    c = as_int (t[3]);
+    int    d = as_int (t[4]);
+    double fp= as_double (t[5]);
+    double de= as_double (t[6]);
+    return dots (e, a, b, c, d, fp, de);
+  }
   else if (is_func (t, EFF_GAUSSIAN, 3)) {
     double rx= as_double (t[0]);
     double ry= as_double (t[1]);
@@ -460,7 +687,7 @@ build_effect (tree t) {
     return motion_pen_effect (dx, dy);
   }
   else if (is_func (t, EFF_BLUR, 2)) {
-    effect eff  = build_effect (t[0]);
+    effect eff= build_effect (t[0]);
     effect pen= build_effect (t[1]);
     return blur (eff, pen);
   }
@@ -479,6 +706,30 @@ build_effect (tree t) {
     effect pen= build_effect (t[1]);
     return erode (eff, pen);
   }
+  else if (is_func (t, EFF_DEGRADE, 5)) {
+    effect eff= build_effect (t[0]);
+    double wx = as_double (t[1]);
+    double wy = as_double (t[2]);
+    double th = as_double (t[3]);
+    double sh = as_double (t[4]);
+    return degrade (eff, wx, wy, th, sh);
+  }
+  else if (is_func (t, EFF_DISTORT, 5)) {
+    effect eff= build_effect (t[0]);
+    double wx = as_double (t[1]);
+    double wy = as_double (t[2]);
+    double rx = as_double (t[3]);
+    double ry = as_double (t[4]);
+    return distort (eff, wx, wy, rx, ry);
+  }
+  else if (is_func (t, EFF_GNAW, 5)) {
+    effect eff= build_effect (t[0]);
+    double wx = as_double (t[1]);
+    double wy = as_double (t[2]);
+    double rx = as_double (t[3]);
+    double ry = as_double (t[4]);
+    return gnaw (eff, wx, wy, rx, ry);
+  }
   else if (is_func (t, EFF_SUPERPOSE)) {
     array<effect> effs= build_effects (A(t));
     return superpose (effs);
@@ -536,17 +787,48 @@ build_effect (tree t) {
         v << m (i, j);
     return color_matrix (eff, v);
   }
+  else if (is_func (t, EFF_GRADIENT, 3)) {
+    effect eff= build_effect (t[0]);
+    color  fgc= named_color (as_string (t[1]));
+    color  bgc= named_color (as_string (t[2]));
+    true_color fg (fgc);
+    true_color bg (bgc);
+    array<double> v;
+    v << bg.r - fg.r << 0.0 << 0.0 << 0.0 << fg.r
+      << 0.0 << bg.g - fg.g << 0.0 << 0.0 << fg.g
+      << 0.0 << 0.0 << bg.b - fg.b << 0.0 << fg.b
+      << 0.0 << 0.0 << 0.0 << 1.0 << 0.0;
+    return color_matrix (eff, v);
+  }
   else if (is_func (t, EFF_MAKE_TRANSPARENT)) {
     effect eff= build_effect (t[0]);
     color  bgc= named_color (as_string (t[1]));
-    return make_transparent (eff, bgc);
+    double thr= (N(t)<=2)? 1.0: as_double (t[2]);
+    return make_transparent (eff, bgc, thr);
   }
   else if (is_func (t, EFF_MAKE_OPAQUE)) {
     effect eff= build_effect (t[0]);
     color  bgc= named_color (as_string (t[1]));
     return make_opaque (eff, bgc);
   }
-  else {
-    return argument_effect (0);
+  else if (is_func (t, EFF_RECOLOR)) {
+    effect eff= build_effect (t[0]);
+    color  col= named_color (as_string (t[1]));
+    return recolor (eff, col);
   }
+  else if (is_func (t, EFF_SKIN)) {
+    effect eff= build_effect (t[0]);
+    color  col= named_color (as_string (t[1]));
+    return apply_skin (eff, col);
+  }
+  else if (is_compound (t, "eff-crop", 5)) {
+    effect eff= build_effect (t[0]);
+    double cx1= as_double (t[1]);
+    double cy1= as_double (t[2]);
+    double cx2= as_double (t[3]);
+    double cy2= as_double (t[4]);
+    return crop (eff, cx1, cy1, cx2, cy2);
+  }
+  else
+    return argument_effect (0);
 }
