@@ -10,19 +10,30 @@
 
 #include "url.hpp"
 #include "mac_utilities.h"
-#include "timer.hpp"
+#include "tm_timer.hpp"
 #include "gui.hpp"
 #include "analyze.hpp"
 #include "hashset.hpp"
 #include "iterator.hpp"
-#pragma push_macro("FAILED")  // CoreFoundation will rewrite our macro FAILED
+
+#undef FAILED // redefined by CARBON
 #define extend CARBON_extends // avoid name collision
 #include "Cocoa/mac_cocoa.h"
 #include <Carbon/Carbon.h>
 #include <crt_externs.h>
 #include "HIDRemote.h"
 #undef extend
-#pragma pop_macro("FAILED")   // Restore our definition
+
+#undef FAILED // restore TeXmacs definition
+#ifdef USE_EXCEPTIONS
+#define FAILED(msg) { tm_throw (msg); }
+#else
+#ifdef DEBUG_ASSERT
+#define FAILED(msg) { tm_failure (msg); assert (false); }
+#else
+#define FAILED(msg) { tm_failure (msg); }
+#endif
+#endif
 
 #ifdef QTTEXMACS
 #include <QApplication>
@@ -30,6 +41,7 @@
 #include <QString>
 #include "Qt/QTMWidget.hpp"
 #include "Qt/qt_gui.hpp"
+#include "Qt/qt_utilities.hpp"
 #endif
 
 bool 
@@ -44,6 +56,7 @@ mac_alternate_startup () {
 }
 
 
+#ifdef AQUATEXMACS
 void 
 mac_fix_paths () {
   NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init];
@@ -67,6 +80,7 @@ mac_fix_paths () {
   system("printenv");
   [pool release];  
 }
+#endif
 
 
 
@@ -106,8 +120,8 @@ mac_handler_body (NSEvent *event) {
         if (nsmods &  NSAlternateKeyMask) str.append("Alt+");
         if (nsmods &  NSCommandKeyMask) str.append("Meta+");
         str.append("Tab");
-        cout << str.toAscii().constData() << LF;
-#endif      
+        cout << from_qstring (str) << LF;
+#endif
         
         QKeyEvent *qe = new QKeyEvent(([event type] == NSKeyDown) ? 
                                       QEvent::KeyPress : QEvent::KeyRelease, 
@@ -115,6 +129,21 @@ mac_handler_body (NSEvent *event) {
         QApplication::postEvent(qApp->focusWidget(), qe);
         return nil;
       }
+      if (key == 0x0051 || key == 0x0071) {
+        NSUInteger nsmods = [event modifierFlags];
+        Qt::KeyboardModifiers modifs = 0;
+        if (key == NSBackTabCharacter) modifs |= Qt::ShiftModifier;
+        if (nsmods &  NSControlKeyMask) modifs |= Qt::MetaModifier;
+        if (nsmods &  NSAlternateKeyMask) modifs |= Qt::AltModifier;
+        if (nsmods &  NSCommandKeyMask) modifs |= Qt::ControlModifier;
+        if (nsmods & NSCommandKeyMask) {
+          QKeyEvent *qe = new QKeyEvent(([event type] == NSKeyDown) ? 
+                                        QEvent::KeyPress : QEvent::KeyRelease, 
+                                        Qt::Key_Q, modifs);
+          QApplication::postEvent(qApp->focusWidget(), qe);
+          return nil;
+        }
+      }
     }
   }
   return event;
@@ -216,6 +245,7 @@ from_nsstring (NSString *s) {
          isPressed:(BOOL)isPressed
 fromHardwareWithAttributes:(NSMutableDictionary *)attributes
 {
+  (void) theHidRemote; (void) attributes;
   NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init]; 
   mac_remote_button (from_nsstring([self buttonNameForButtonCode:buttonCode]), isPressed);
   [pool release];
@@ -437,7 +467,6 @@ mac_screen_scale_factor() {
 
 // end scale factor detection
 
-#if defined (MAC_OS_X_VERSION_10_10)
 /* A bug in OSX Yosemite inserts duplicate entries in the environment. This
  affects child processes: in particular, the PATH is not properly inherited
  unless we remove the duplicates and most plugins fail to start (since they are
@@ -447,6 +476,8 @@ mac_screen_scale_factor() {
  */
 void
 mac_fix_yosemite_bug() {
+#if MAC_OS_X_VERSION_MAX_ALLOWED > MAC_OS_X_VERSION_10_9
+#if defined (MAC_OS_X_VERSION_10_10)
   hashset<string> entries;
   hashset<string> duplicates;
   
@@ -468,5 +499,35 @@ mac_fix_yosemite_bug() {
       setenv (name, val, 0); // restore name=val
     }
   }
+#endif   // defined (MAC_OS_X_VERSION_10_10)
+#endif
+}
+
+
+static id background_activity= nil;
+
+void
+mac_begin_server () {
+#if MAC_OS_X_VERSION_MAX_ALLOWED > MAC_OS_X_VERSION_10_8
+#if defined (MAC_OS_X_VERSION_10_9)
+  if (background_activity == nil) {
+    id background_activity = [[NSProcessInfo processInfo]
+                               beginActivityWithOptions: NSActivityBackground
+                              reason: @"TeXmacs server running"];
+    [background_activity retain];
+  }
+#endif
+#endif
+}
+
+void
+mac_end_server () {
+#if MAC_OS_X_VERSION_MAX_ALLOWED > MAC_OS_X_VERSION_10_8
+#if defined (MAC_OS_X_VERSION_10_9)
+  if (background_activity) {
+    [[NSProcessInfo processInfo] endActivity: background_activity];
+    [background_activity release];
+  }
+#endif
+#endif
 }
-#endif   // defined (MAC_OS_X_VERSION_10_10)
\ No newline at end of file
