@@ -16,11 +16,12 @@
 #include "dictionary.hpp"
 #include "file.hpp" // added for copy_as_graphics
 #include "analyze.hpp"
-#include "language.hpp"
+#include "locale.hpp"
 #include "message.hpp"
 #include "scheme.hpp"
 #include "tm_window.hpp"
 #include "new_window.hpp"
+#include "boot.hpp"
 
 #include "qt_gui.hpp"
 #include "qt_utilities.hpp"
@@ -44,6 +45,7 @@
 #include <QLibraryInfo>
 #include <QImage>
 #include <QUrl>
+#include <QDesktopWidget>
 
 #include "QTMGuiHelper.hpp"
 #include "QTMWidget.hpp"
@@ -54,6 +56,29 @@
 #include "MacOS/mac_utilities.h"
 #endif
 
+#if (QT_VERSION >= 0x050000)
+#include <QtPlugin>
+#ifdef qt_static_plugin_qjpeg
+Q_IMPORT_PLUGIN(qjpeg)
+#endif
+#ifdef qt_static_plugin_qgif
+Q_IMPORT_PLUGIN(qgif)
+#endif
+#ifdef qt_static_plugin_qico
+Q_IMPORT_PLUGIN(qico)
+#endif
+#ifdef qt_static_plugin_qsvg
+Q_IMPORT_PLUGIN(qsvg)
+#endif
+
+#ifdef WIN32 
+Q_IMPORT_PLUGIN(QWindowsIntegrationPlugin)
+#endif
+#ifdef QT_MAC_USE_COCOA
+Q_IMPORT_PLUGIN(QCocoaIntegrationPlugin)
+#endif
+#endif
+
 qt_gui_rep* the_gui = NULL;
 int nr_windows = 0; // FIXME: fake variable, referenced in tm_server
 
@@ -119,7 +144,51 @@ needing_update (false)
   updatetimer->setSingleShot (true);
   QObject::connect (updatetimer, SIGNAL (timeout()),
                     gui_helper, SLOT (doUpdate()));
-    //  (void) default_font ();
+  // (void) default_font ();
+
+  if (!retina_manual) {
+    retina_manual= true;
+#ifdef MACOSX_EXTENSIONS
+#if (QT_VERSION < 0x050000)
+    double mac_hidpi = mac_screen_scale_factor();
+    if (DEBUG_STD)
+      debug_boot << "Mac Screen scaleFfactor: " << mac_hidpi <<  "\n";
+          
+    if (mac_hidpi == 2) {
+      if (DEBUG_STD) debug_boot << "Setting up HiDPI mode\n";
+      retina_factor= 2;
+      retina_scale = 1.4;
+      if (!retina_iman) {
+        retina_iman  = true;
+        retina_icons = 2;
+        // retina_icons = 1;
+        // retina_icons = 2;  // FIXME: why is this not better?
+      }
+    }
+#endif
+#else
+    SI w, h;
+    get_extents (w, h);
+    if (DEBUG_STD)
+      debug_boot << "Screen extents: " << w/PIXEL << " x " << h/PIXEL << "\n";
+    if (min (w, h) >= 1440 * PIXEL) {
+      retina_zoom = 2;
+      retina_scale= 1.0;
+      if (!retina_iman) {
+        retina_iman  = true;
+        retina_icons = 2;
+      }
+    }
+#endif
+  }
+  if (has_user_preference ("retina-factor"))
+    retina_factor= get_user_preference ("retina-factor") == "on"? 2: 1;
+  if (has_user_preference ("retina-zoom"))
+    retina_zoom= get_user_preference ("retina-zoom") == "on"? 2: 1;
+  if (has_user_preference ("retina-icons"))
+    retina_icons= get_user_preference ("retina-icons") == "on"? 2: 1;
+  if (has_user_preference ("retina-scale"))
+    retina_scale= as_double (get_user_preference ("retina-scale"));
 }
 
 /* important routines */
@@ -246,8 +315,9 @@ qt_gui_rep::get_selection (string key, tree& t, string& s, string format) {
   if (input_format == "picture") {
     tree t (IMAGE);
     QSize size= qvariant_cast<QImage>(md->imageData()).size ();
-    string w= as_string (size.width ()) * "px";
-    string h= as_string (size.height ()) * "px";
+    int ww= size.width (), hh= size.height ();
+    string w, h;
+    qt_pretty_image_size (ww, hh, w, h);
     t << tuple (tree (RAW_DATA, s), "png") << w << h << "" << "";
     s= as_string (call ("convert", t, "texmacs-tree", "texmacs-snippet"));
   }
@@ -274,7 +344,7 @@ qt_gui_rep::set_selection (string key, tree t,
   cb->clear (mode);
   
   c_string selection (s);
-  cb->setText (QString::fromAscii (selection), mode);
+  cb->setText (QString::fromLatin1 (selection), mode);
   QMimeData *md = new QMimeData;
   
   if (format == "verbatim" || format == "default") {
@@ -283,7 +353,7 @@ qt_gui_rep::set_selection (string key, tree t,
       
       QString pid_str;
       pid_str.setNum (QCoreApplication::applicationPid ());
-      md->setData ("application/x-texmacs-pid", pid_str.toAscii());
+      md->setData ("application/x-texmacs-pid", pid_str.toLatin1());
       
       (void) sh;
         //selection = c_string (sh);
@@ -302,10 +372,10 @@ qt_gui_rep::set_selection (string key, tree t,
     else if (enc == "iso-8859-1" || enc == "ISO-8859-1")
       md->setText (QString::fromLatin1 (selection));
     else
-      md->setText (QString::fromAscii (selection));
+      md->setText (QString::fromLatin1 (selection));
   }
   else
-    md->setText (QString::fromAscii (selection));
+    md->setText (QString::fromLatin1 (selection));
   cb->setMimeData (md, mode);
     // according to the docs, ownership of mimedata is transferred to clipboard
     // so no memory leak here
@@ -443,6 +513,22 @@ gui_open (int& argc, char** argv) {
 #ifdef MACOSX_EXTENSIONS
   mac_begin_remote();
 #endif
+
+  // Qt and Guile want to change the locale.
+  // We reset it to have POSIX functions parse correctly the configuration files
+  // (see as_double() in string.cpp)
+
+  setlocale (LC_NUMERIC, "C");
+
+  // From Qt docs:
+  // On Unix/Linux Qt is configured to use the system locale settings by
+  // default. This can cause a conflict when using POSIX functions, for
+  // instance, when converting between data types such as floats and strings,
+  // since the notation may differ between locales. To get around this problem,
+  // call the POSIX function setlocale(LC_NUMERIC,"C") right after initializing
+  // QApplication, QGuiApplication or QCoreApplication to reset the locale
+  // that is used for number formatting to "C"-locale.
+  // See https://doc.qt.io/qt-5/qcoreapplication.html#locale-settings
 }
 
 void
@@ -480,6 +566,18 @@ gui_refresh () {
   the_gui->refresh_language();
 }
 
+string
+gui_version () {
+#if (QT_VERSION >= 0x060000)
+  return "qt6";
+#else
+#if (QT_VERSION >= 0x050000)
+  return "qt5";
+#else
+  return "qt4";
+#endif
+#endif
+}
 
 /******************************************************************************
  * Queued processing
@@ -495,6 +593,9 @@ gui_refresh () {
  Likewise this function is just a hack to get things working properly.
  */
 
+static int keyboard_events = 0;
+static int keyboard_special= 0;
+
 void
 qt_gui_rep::process_queued_events (int max) {
   int count = 0;
@@ -503,8 +604,8 @@ qt_gui_rep::process_queued_events (int max) {
     if (ev.x1 == qp_type::QP_NULL) break;
 #ifdef QT_CPU_FIX
     if (ev.x1 != qp_type::QP_NULL &&
-	ev.x1 != qp_type::QP_SOCKET_NOTIFICATION &&
-	ev.x1 != qp_type::QP_DELAYED_COMMANDS)
+        ev.x1 != qp_type::QP_SOCKET_NOTIFICATION &&
+        ev.x1 != qp_type::QP_DELAYED_COMMANDS)
       tm_wake_up ();
 #endif
     switch (ev.x1) {
@@ -514,8 +615,11 @@ qt_gui_rep::process_queued_events (int max) {
       {
         typedef triple<widget, string, time_t > T;
         T x = open_box <T> (ev.x2);
-        if (!is_nil (x.x1))
+        if (!is_nil (x.x1)) {
           concrete_simple_widget (x.x1)->handle_keypress (x.x2, x.x3);
+          keyboard_events++;
+          if (N(x.x2) > 1) keyboard_special++;
+        }
       }
         break;
       case qp_type::QP_KEYBOARD_FOCUS :
@@ -685,7 +789,7 @@ qt_gui_rep::update () {
   int std_delay= 1;
   tm_sleep ();
 #else
-  int std_delay= 1000 / 6;
+  int std_delay= 90 / 6;
 #endif
 
   if (updating) {
@@ -700,11 +804,11 @@ qt_gui_rep::update () {
   updating = true;
   
   static int count_events    = 0;
-  static int max_proc_events = 10;
+  static int max_proc_events = 40;
   
   time_t     now = texmacs_time();
   needing_update = false;
-  time_credit    = 100 / (waiting_events.size() + 1);
+  time_credit    = 9 / (waiting_events.size() + 1);
   
     // 1.
     // Check if a wait dialog is active and in that case remove it.
@@ -737,22 +841,25 @@ qt_gui_rep::update () {
   
   if (waiting_events.size() == 0) {
       // If there are no waiting events call the interpose handler at least once
-    if (the_interpose_handler) the_interpose_handler();
-  } else while (waiting_events.size() > 0 && count_events < max_proc_events) {
+    //if (the_interpose_handler) the_interpose_handler();
+  }
+  else while (waiting_events.size() > 0 && count_events < max_proc_events) {
     process_queued_events (1);
     count_events++;
-    if (the_interpose_handler) the_interpose_handler();
+    //if (the_interpose_handler) the_interpose_handler();
   }
-    // Repaint invalid regions and redraw
-  count_events = 0;
+  // Repaint invalid regions and redraw
+  bool postpone_treatment= (keyboard_events > 0 && keyboard_special == 0);
+  keyboard_events = 0;
+  keyboard_special= 0;
+  count_events    = 0;
   
   interrupted  = false;
   timeout_time = texmacs_time() + time_credit;
-  
-  QSetIterator<QTMWidget*> i (QTMWidget::all_widgets);
-  while (i.hasNext()) {
-    QTMWidget* w = i.next();
-    if (w->isVisible()) w->repaint_invalid_regions();
+
+  if (!postpone_treatment) {
+    if (the_interpose_handler) the_interpose_handler();
+    qt_simple_widget_rep::repaint_all ();
   }
   
   if (waiting_events.size() > 0) needing_update = true;
@@ -762,7 +869,8 @@ qt_gui_rep::update () {
   time_t delay = delayed_commands.lapse - texmacs_time();
   if (needing_update) delay = 0;
   else                delay = max (0, min (std_delay, delay));
-  
+  if (postpone_treatment) delay= 9; // NOTE: force occasional display
+ 
   updatetimer->start (delay);
   updating = false;
   
@@ -837,7 +945,7 @@ qt_gui_rep::show_help_balloon (widget wid, SI x, SI y) {
   // the texmacs-input widget whenever there is one)
   get_position (get_window (concrete_window()->win), winx, winy);
   set_position (_popup_wid, x+winx, y+winy);
-  popup_wid_time = texmacs_time() + 666;
+  popup_wid_time = texmacs_time() + 66;
     // update() will eventually show the widget
 }
 
@@ -910,7 +1018,7 @@ qt_gui_rep::put_graphics_on_clipboard (url file) {
       (extension == "jpg") || (extension == "jpeg")) {
     QClipboard *clipboard = QApplication::clipboard();
     c_string tmp (concretize (file));
-    clipboard->setImage (QImage (tmp));
+    clipboard->setImage (QImage (QString (tmp)));
   }
   else {
       // vector formats
@@ -994,6 +1102,7 @@ command_queue::exec (object cmd) {
   start_times << (((time_t) texmacs_time ()) - 1000000000);
   lapse = texmacs_time();
   the_gui->need_update();
+  wait= true;
 }
 
 void
@@ -1002,6 +1111,7 @@ command_queue::exec_pause (object cmd) {
   start_times << ((time_t) texmacs_time ());
   lapse = texmacs_time();
   the_gui->need_update();
+  wait= true;
 }
 
 void
@@ -1098,5 +1208,3 @@ int
 event_queue::size() const {
   return n;
 }
-
-
