@@ -132,7 +132,7 @@ qt_tm_widget_rep::qt_tm_widget_rep(int mask, command _quit)
   // which has been fixed in 4.6.2 (at least)
   // this is why we change dimension of icons
   
-#if (defined(Q_WS_MAC)&&(QT_VERSION>=QT_VERSION_CHECK(4,6,0))&&(QT_VERSION<QT_VERSION_CHECK(4,6,2)))
+#if (defined(Q_OS_MAC)&&(QT_VERSION>=QT_VERSION_CHECK(4,6,0))&&(QT_VERSION<QT_VERSION_CHECK(4,6,2)))
   mw->setIconSize (QSize (22, 30));  
 #else
   mw->setIconSize (QSize (17, 17));
@@ -163,14 +163,19 @@ qt_tm_widget_rep::qt_tm_widget_rep(int mask, command _quit)
   //    trying to figure this out :)
   
   bar->setMinimumWidth (2);
-#ifdef Q_WS_X11
+#ifdef Q_OS_LINUX
   int min_h= (int) floor (28 * retina_scale);
   bar->setMinimumHeight (min_h);
+#else
+#if (QT_VERSION >= 0x050000)
+  int min_h= (int) floor (24 * retina_scale);
+  bar->setMinimumHeight (min_h);
 #else
   if (retina_scale > 1.0) {
     int min_h= (int) floor (20 * retina_scale);
     bar->setMinimumHeight (min_h);
   }
+#endif
 #endif
   mw->setStatusBar (bar);
  
@@ -211,7 +216,29 @@ qt_tm_widget_rep::qt_tm_widget_rep(int mask, command _quit)
     sz = (pxm ? pxm->size() : QSize (16, 16));
     tweak_iconbar_size (sz);
     focusToolBar->setIconSize (sz);
-  }  
+  }
+
+  // Why we need fixed height:
+  // The height of the toolbar is actually determined by the font height.
+  // And the font height is not fixed. If the height of the toolbar is not
+  // fixed, the stretching of it will make the document area floating and
+  // triggers the re-rendering of the full document.
+  //
+  // NOTICE: setFixedHeight must be after setIconSize
+  // TODO: the size of the toolbar should be calculated dynamically
+#if (QT_VERSION >= 0x050000)
+  int toolbarHeight= 30;
+  mainToolBar->setFixedHeight (toolbarHeight + 8);
+  modeToolBar->setFixedHeight (toolbarHeight + 4);
+  focusToolBar->setFixedHeight (toolbarHeight);
+#else
+#ifndef Q_OS_MAC
+  int toolbarHeight= 30 * retina_icons;
+  mainToolBar->setFixedHeight (toolbarHeight + 8);
+  modeToolBar->setFixedHeight (toolbarHeight + 4);
+  focusToolBar->setFixedHeight (toolbarHeight);  
+#endif
+#endif
   
   QWidget *cw= new QWidget();
   cw->setObjectName("central widget");  // this is important for styling toolbars.
@@ -330,9 +357,13 @@ qt_tm_widget_rep::qt_tm_widget_rep(int mask, command _quit)
   sideTools->setVisible (false);
   bottomTools->setVisible (false);
   mainwindow()->statusBar()->setVisible (true);
-#ifndef Q_WS_MAC
+#ifndef Q_OS_MAC
   mainwindow()->menuBar()->setVisible (false);
 #endif  
+  QPalette pal;
+  QColor bgcol (160, 160, 160); // same as tm_background
+  pal.setColor (QPalette::Mid, bgcol);
+  mainwindow()->setPalette(pal);
 }
 
 qt_tm_widget_rep::~qt_tm_widget_rep () {
@@ -346,7 +377,7 @@ qt_tm_widget_rep::~qt_tm_widget_rep () {
 
 void
 qt_tm_widget_rep::tweak_iconbar_size (QSize& sz) {
-#ifdef Q_WS_X11
+#ifdef Q_OS_LINUX
   if (sz.height () >= 24) {
     sz.setWidth (sz.width () + 2);
     sz.setHeight (sz.height () + 8);
@@ -378,7 +409,8 @@ qt_tm_widget_rep::tweak_iconbar_size (QSize& sz) {
  \param name A unique identifier for the window (e.g. "TeXmacs:3")
  */
 widget
-qt_tm_widget_rep::plain_window_widget (string name, command _quit) {
+qt_tm_widget_rep::plain_window_widget (string name, command _quit, int b) {
+  (void) b;
   (void) _quit; // The widget already has a command. Don't overwrite. 
   orig_name = name;
   return this;
@@ -419,7 +451,7 @@ qt_tm_widget_rep::update_visibility () {
   if ( XOR(old_statusVisibility,  new_statusVisibility) )
     mainwindow()->statusBar()->setVisible (new_statusVisibility);
 
-#ifndef Q_WS_MAC
+#ifndef Q_OS_MAC
   bool old_menuVisibility = mainwindow()->menuBar()->isVisible();
   bool new_menuVisibility = visibility[0];
 
@@ -851,11 +883,28 @@ qt_tm_widget_rep::write (slot s, blackbox index, widget w) {
     case SLOT_FOCUS_ICONS:
       check_type_void (index, s);
     {
-      focus_icons_widget = concrete (w);
-      QList<QAction*>* list = focus_icons_widget->get_qactionlist();
-      if (list) {
-        replaceButtons (focusToolBar, list);
-        update_visibility();
+      bool can_update = true;
+#if (QT_VERSION >= 0x050000)
+      // BUG:
+      // there is a problem with updateActions  which apparently
+      // reset a running input method in Qt5.
+      //
+      // This is (probably) also relate to
+      // bug #47338 [CJK] input disappears immediately
+      // see http://lists.gnu.org/archive/html/texmacs-dev/2017-09/msg00000.html
+      
+      // HACK: we just disable the focus bar updating while preediting.
+      // This seems enough since the other toolbars are not usually updated
+      // while performing an input method keyboard sequence
+      if (canvas()) can_update = !canvas()->isPreediting();
+#endif
+      if (can_update) {
+        focus_icons_widget = concrete (w);
+        QList<QAction*>* list = focus_icons_widget->get_qactionlist();
+        if (list) {
+          replaceButtons (focusToolBar, list);
+          update_visibility();
+        }
       }
     }
       break;
@@ -937,6 +986,8 @@ qt_tm_widget_rep::set_full_screen(bool flag) {
     }
     else {
       QPalette pal;
+      QColor bgcol (160, 160, 160); // same as tm_background
+      pal.setColor (QPalette::Mid, bgcol);
       mainwindow()->setPalette(pal);
       bool cache = visibility[0];
       visibility[0] = false;
