@@ -182,6 +182,34 @@ qt_glue_widget_rep::as_qwidget() {
 }
 
 
+/******************************************************************************
+ * The following hack has been implemented by Joris in order to avoid
+ * a focus bug when a menu contains a text input field.  To provoke
+ * this bug without the hack, put your cursor behind a citation,
+ * open the right-most (search) popup menu just at the left of
+ * the 'Identifier' text field and then close it again by re-clicking
+ * on the magnifying glass
+ ******************************************************************************/
+
+static list<QAction*> to_be_destroyed;
+static time_t last_addition;
+
+void
+schedule_destruction (QAction* a) {
+  time_t now= texmacs_time ();
+  if (!is_nil (to_be_destroyed) && last_addition + 3000 < now) {
+    to_be_destroyed= reverse (to_be_destroyed);
+    while (!is_nil (to_be_destroyed)) {
+      //cout << "Destroy\n";
+      delete to_be_destroyed->item;
+      to_be_destroyed= to_be_destroyed->next;
+    }
+  }
+  //cout << "Postpone\n";
+  last_addition= now;
+  to_be_destroyed= list<QAction*> (a, to_be_destroyed);
+}
+
 /******************************************************************************
  * qt_ui_element_rep
  ******************************************************************************/
@@ -193,8 +221,9 @@ qt_ui_element_rep::~qt_ui_element_rep() {
   if (cachedActionList) {
     while (!cachedActionList->empty()) {
       QAction *a = cachedActionList->takeFirst();
-      if (a) delete a;
-     }
+      //if (a) delete a;
+      if (a) schedule_destruction (a);
+    }
     delete cachedActionList;
   }
 }
@@ -673,14 +702,14 @@ qt_ui_element_rep::as_qlayoutitem () {
         // Used only for non-colored glue widgets (skips qt_glue_widget_rep)
     {
       typedef quartet<bool, bool, SI, SI> T;
-      T x = open_box<T>(load);
-
+      T x = open_box<T> (load);
+      QSize sz = QSize (x.x3, x.x4);
       QSizePolicy::Policy hpolicy = x.x1 ? QSizePolicy::MinimumExpanding
                                          : QSizePolicy::Minimum;
       QSizePolicy::Policy vpolicy = x.x2 ? QSizePolicy::MinimumExpanding
                                          : QSizePolicy::Minimum;
 
-      return new QSpacerItem (x.x3, x.x4, hpolicy, vpolicy);
+      return new QSpacerItem (sz.width (), sz.height (), hpolicy, vpolicy);
     }
       break;
     default:
@@ -765,12 +794,28 @@ qt_ui_element_rep::as_qwidget () {
       
     case menu_separator: 
     case menu_group:
-    case glue_widget:
     {
       qwid = new QWidget();
     }
       break;
       
+    case glue_widget:
+    // Used only for non-colored glue widgets (skips qt_glue_widget_rep)
+    {
+      typedef quartet<bool, bool, SI, SI> T;
+      T x = open_box<T>(load);
+      QSize sz = QSize (x.x3, x.x4);
+      QSizePolicy::Policy hpolicy = x.x1 ? QSizePolicy::MinimumExpanding
+                                         : QSizePolicy::Minimum;
+      QSizePolicy::Policy vpolicy = x.x2 ? QSizePolicy::MinimumExpanding
+                                         : QSizePolicy::Minimum;
+      qwid = new QWidget();
+      qwid->setMinimumSize (sz);
+      qwid->setSizePolicy (hpolicy, vpolicy);
+    }
+      break;
+
+      
     case pulldown_button:
     case pullright_button:
     {
@@ -841,6 +886,7 @@ qt_ui_element_rep::as_qwidget () {
       qwid->setStyle (qtmstyle());
       qt_apply_tm_style (qwid, style);
       qwid->setEnabled (! (style & WIDGET_STYLE_INERT));
+      qwid->setFocusPolicy (Qt::StrongFocus);
     }
       break;
       
@@ -909,6 +955,7 @@ qt_ui_element_rep::as_qwidget () {
       QCheckBox* w  = new QCheckBox (NULL);  
       w->setCheckState (check ? Qt::Checked : Qt::Unchecked);
       qt_apply_tm_style (w, style);
+      w->setFocusPolicy (Qt::StrongFocus);
       
       command tcmd = tm_new<qt_toggle_command_rep> (w, cmd);
       QTMCommand* c = new QTMCommand (w, tcmd);
