@@ -33,6 +33,13 @@
  * qt_widget_rep: the base widget for the Qt port.
  ******************************************************************************/
 
+template<> void
+tm_delete<qt_widget_rep> (qt_widget_rep* ptr) {
+  void *mem= ptr->derived_this ();
+  ptr -> ~qt_widget_rep ();
+  fast_delete (mem);
+}
+
 static long widget_counter = 0;
 
 qt_widget_rep::qt_widget_rep(types _type, QWidget* _qwid)
@@ -155,14 +162,8 @@ qt_widget_rep::as_qlayoutitem () {
 }
 
 
-/*! Returns the QMenu associated if any.
- 
- This method must not give ownership of the menu to the caller, thus
- allowing menu caching at the TeXmacs level. See the implementations in
- qt_ui_element_rep and qt_menu_rep.
- */
-QMenu*
-qt_widget_rep::get_qmenu () {
+QList<QAction*>*
+qt_widget_rep::get_qactionlist() {
   return NULL;
 }
 
@@ -185,7 +186,7 @@ qt_widget_rep::get_qmenu () {
  \return The new qt_window_widget.
 */
 widget
-qt_widget_rep::plain_window_widget (string name, command quit) {
+qt_widget_rep::plain_window_widget (string name, command quit, int border) {
   if (DEBUG_QT_WIDGETS)
     debug_widgets << "qt_widget_rep::plain_window_widget() around a "
                   << type_as_string() << LF;
@@ -212,7 +213,7 @@ qt_widget_rep::plain_window_widget (string name, command quit) {
   
   int l,t,r,b;
   win->layout()->getContentsMargins (&l, &t, &r, &b);
-  win->layout()->setContentsMargins (l+3, t+3, r+3, b+3);
+  win->layout()->setContentsMargins (l+border, t+border, r+border, b+border);
   win->setWindowTitle (to_qstring (name));  // HACK: remove me (see bug#40837)
   
   qt_window_widget_rep* wid = tm_new<qt_window_widget_rep> (win, name, quit);
@@ -229,7 +230,7 @@ qt_widget_rep::plain_window_widget (string name, command quit) {
  */
 widget
 qt_widget_rep::make_popup_widget () {
-  return tm_new<qt_popup_widget_rep> (this, command());
+  return tm_new<qt_popup_widget_rep> ((widget_rep*)this, command());
 }
 
 /*! Interface for the creation of popups.
@@ -242,6 +243,13 @@ qt_widget_rep::popup_window_widget (string s) {
   return concrete(wid)->popup_window_widget(s);
 }
 
+widget
+qt_widget_rep::tooltip_window_widget (string s) {
+  widget wid= make_popup_widget();
+  ASSERT(concrete(wid) != this, "Loop in call to tooltip_window_widget()");
+  return concrete(wid)->tooltip_window_widget(s);
+}
+
 tm_ostream& operator << (tm_ostream& out, qt_widget w) {
   return out << "qt_widget of type: " << w.rep->type_as_string();
 }
@@ -285,6 +293,11 @@ popup_window_widget (widget w, string s) {
   return concrete(w)->popup_window_widget (s);
 }
 
+widget
+tooltip_window_widget (widget w, string s) {
+  return concrete(w)->tooltip_window_widget (s);
+}
+
 /*! A factory for a popup widget container whose contents are to be unmapped as 
  soon as the mouse exits the widget. 
  
