@@ -23,7 +23,7 @@ protected:
   url appcast;
   int interval;
   
-  tm_updater () { }
+  tm_updater () : interval (0) { }
   tm_updater (const tm_updater&);
   void operator= (const tm_updater&);
   virtual ~tm_updater () { };
@@ -35,12 +35,9 @@ public:
   virtual bool checkInForeground () { return false; }  // non-blocking
   virtual bool isRunning () const   { return false; }
    
-  virtual bool setAutomaticChecks (bool enable) { (void) enable; return false; }
   virtual time_t lastCheck () const { return 0; }
   virtual bool getCheckInterval () const { return interval; }
   virtual bool setCheckInterval (int hours) { (void) hours; return false; }
-  virtual url  getAppcast () const { return appcast; }
-  virtual bool setAppcast (url _appcast) { (void) _appcast; return false; }
 };
 
 
@@ -52,9 +49,7 @@ bool updater_supported ();
 bool updater_is_running ();
 bool updater_check_background ();
 bool updater_check_foreground ();
-bool updater_set_appcast (url appcast);
 bool updater_set_interval (int hours);
-bool updater_set_automatic (bool enable);
 time_t updater_last_check ();
 
 #endif    // TM_UPDATER_HPP
