@@ -24,18 +24,27 @@
   (updater-check-background updater_check_background (bool))
   (updater-check-foreground updater_check_foreground (bool))
   (updater-last-check updater_last_check (long))
-  (updater-set-appcast updater_set_appcast (bool url))
   (updater-set-interval updater_set_interval (bool int))
-  (updater-set-automatic updater_set_automatic (bool bool))
+  (get-original-path get_original_path (string))
   (os-win32? os_win32 (bool))
   (os-mingw? os_mingw (bool))
   (os-macos? os_macos (bool))
+  (has-printing-cmd? has_printing_cmd (bool))
   (x-gui? gui_is_x (bool))
   (qt-gui? gui_is_qt (bool))
+  (gui-version gui_version (string))
   (default-look-and-feel default_look_and_feel (string))
   (default-chinese-font default_chinese_font_name (string))
   (default-japanese-font default_japanese_font_name (string))
   (default-korean-font default_korean_font_name (string))
+  (get-retina-factor get_retina_factor (int))
+  (get-retina-zoom get_retina_zoom (int))
+  (get-retina-icons get_retina_icons (int))
+  (get-retina-scale get_retina_scale (double))
+  (set-retina-factor set_retina_factor (void int))
+  (set-retina-zoom set_retina_zoom (void int))
+  (set-retina-icons set_retina_icons (void int))
+  (set-retina-scale set_retina_scale (void double))
   (tm-output tm_output (void string))
   (tm-errput tm_errput (void string))
   (win32-display win32_display (void string))
@@ -63,6 +72,8 @@
   (bench-print bench_print (void string))
   (bench-print-all bench_print (void))
   (system-wait system_wait (void string string))
+  (get-show-kbd get_show_kbd (bool))
+  (set-show-kbd set_show_kbd (void bool))
   (set-latex-command set_latex_command (void string))
   (set-bibtex-command set_bibtex_command (void string))
   (number-latex-errors number_latex_errors (int url))
@@ -80,6 +91,7 @@
   (cpp-set-preference set_user_preference (void string string))
   (cpp-reset-preference reset_user_preference (void string))
   (save-preferences save_user_preferences (void))
+  (get-default-printing-command get_printing_default (string))
   (set-input-language set_input_language (void string))
   (get-input-language get_input_language (string))
   (set-output-language gui_set_output_language (void string))
@@ -93,11 +105,14 @@
   (color named_color (int string))
   (get-hex-color get_hex_color (string string))
   (named-color->xcolormap named_color_to_xcolormap (string string))
+  (rgba->named-color named_rgb_color (string array_int))
+  (named-color->rgba get_named_rgb_color (array_int string))
   (new-author new_author (double))
   (set-author set_author (void double))
   (get-author get_author (double))
   (debug-set debug_set (void string bool))
   (debug-get debug_get (bool string))
+  (debug-message debug_message (void string string))
   (get-debug-messages get_debug_messages (tree string int))
   (clear-debug-messages clear_debug_messages (void))
   (cout-buffer cout_buffer (void))
@@ -109,7 +124,10 @@
   (new-fonts? get_new_fonts (bool))
   (tmtm-eqnumber->nonumber eqnumber_to_nonumber (tree tree))
   (busy-versioning? is_busy_versioning (bool))
-
+  (players-set-elapsed players_set_elapsed (void tree double))
+  (players-set-speed players_set_speed (void tree double))
+  (apply-effect apply_effect (void content array_url url int int))
+  
   ;; routines for the font database
   (tt-exists? tt_font_exists (bool string))
   (tt-dump tt_dump (void url))
@@ -117,6 +135,7 @@
   (tt-analyze tt_analyze (array_string string))
   (font-database-build font_database_build (void url))
   (font-database-build-local font_database_build_local (void))
+  (font-database-extend-local font_database_extend_local (void url))
   (font-database-build-global font_database_build_global (void))
   (font-database-build-characteristics
    font_database_build_characteristics (void bool))
@@ -162,8 +181,9 @@
   (logical-font-substitute apply_substitutions (array_string array_string))
   (font-family-main main_family (string string))
 
-  ;; routines for images
+  ;; routines for images and animations
   (image->psdoc image_to_psdoc (string url))
+  (anim-control-times get_control_times (array_double content))
 
   ;; routines for trees
   (tree->stree tree_to_scheme_tree (scheme_tree tree))
@@ -207,15 +227,25 @@
   (tree-child-name get_child_name (string content int ))
   (tree-child-long-name get_child_long_name (string content int))
   (tree-child-type get_child_type (string content int))
+  (tree-child-env* get_env_child (tree content int content))
   (tree-child-env get_env_child (tree content int string content))
-  (tree-descendant-env get_env_descendant (tree content path content))
+  (tree-descendant-env* get_env_descendant (tree content path content))
+  (tree-descendant-env get_env_descendant (tree content path string content))
   (tree-load-inclusion load_inclusion (tree url))
   (tree-as-string tree_as_string (string content))
   (tree-extents tree_extents (tree content))
   (tree-empty? is_empty (bool content))
+  (tree-multi-line? is_multi_line (bool content))
   (tree-is-buffer? admits_edit_observer (bool tree))
   (tree-search-sections search_sections (array_tree tree))
   (tree-search-tree search (array_path content content path int))
+  (tree-search-tree-at search (array_path content content path path int))
+  (tree-spell spell (array_path string content path int))
+  (tree-spell-at spell (array_path string content path path int))
+  (tree-spell-selection spell (array_path string content path path path int))
+  (previous-search-hit previous_search_hit (array_path array_path path bool))
+  (next-search-hit next_search_hit (array_path array_path path bool))
+  (navigate-search-hit navigate_search_hit (array_path path bool bool bool))
   (tag-minimal-arity minimal_arity (int tree_label))
   (tag-maximal-arity maximal_arity (int tree_label))
   (tag-possible-arity? correct_arity (bool tree_label int))
@@ -382,6 +412,10 @@
   (unescape-guile unescape_guile (string string))
   (string-quote scm_quote (string string))
   (string-unquote scm_unquote (string string))
+  (string-trim-spaces-left trim_spaces_left (string string))
+  (string-trim-spaces-right trim_spaces_right (string string))
+  (string-trim-spaces trim_spaces (string string))
+  (downgrade-math-letters downgrade_math_letters (string string))
   (string-convert convert (string string string string))
   (encode-base64 encode_base64 (string string))
   (decode-base64 decode_base64 (string string))
@@ -404,6 +438,8 @@
   (hexadecimal->integer from_hexadecimal (int string))
   (cpp-string-tokenize tokenize (array_string string string))
   (cpp-string-recompose recompose (string array_string string))
+  (string-differences differences (array_int string string))
+  (string-distance distance (int string string))
   
   ; Program bracket matching
   (find-left-bracket find_left_bracket (path path string string))
@@ -426,6 +462,20 @@
   (tmstring-upcase-first uni_upcase_first (string string))
   (tmstring-locase-all uni_locase_all (string string))
   (tmstring-upcase-all uni_upcase_all (string string))
+  (tmstring-unaccent-all uni_unaccent_all (string string))
+  (tmstring-letter? uni_is_letter (bool string))
+  (tmstring-before? uni_before (bool string string))
+
+  ;; Spell checking
+  (multi-spell-start spell_start (void))
+  (multi-spell-done spell_done (void))
+  (single-spell-start spell_start (string string))
+  (single-spell-done spell_done (void string))
+  (spell-check spell_check (tree string string))
+  (spell-check? check_word (bool string string))
+  (spell-accept spell_accept (void string string))
+  (spell-var-accept spell_accept (void string string bool))
+  (spell-insert spell_insert (void string string))
 
   ;; Packrat grammar and parsing tools
   (packrat-define packrat_define (void string string tree))
@@ -467,8 +517,10 @@
                            (tree string content string))
   (conservative-bib-export conservative_bib_export
                            (string content string content))
+  (clean-html clean_html (tree content))
   (upgrade-tmml tmml_upgrade (tree scheme_tree))
   (upgrade-mathml upgrade_mathml (tree content))
+  (retrieve-mathjax retrieve_mathjax (tree int))
   (vernac->texmacs vernac_to_tree (tree string))
   (vernac-document->texmacs vernac_document_to_tree (tree string))
   (compute-keys-string compute_keys (array_string string string))
@@ -526,9 +578,11 @@
   (url-complete complete (url url string))
   (url-resolve resolve (url url string))
   (url-resolve-in-path resolve_in_path (url url))
+  (url-resolve-pattern resolve_pattern (url url))
   (url-exists? exists (bool url))
   (url-exists-in-path? exists_in_path (bool url))
   (url-exists-in-tex? exists_in_tex (bool url))
+  (url-concretize* concretize_url (url url))
   (url-concretize concretize (string url))
   (url-materialize materialize (string url string))
   (url-test? is_of_type (bool url string))
@@ -536,6 +590,7 @@
   (url-directory? is_directory (bool url))
   (url-link? is_symbolic_link (bool url))
   (url-newer? is_newer (bool url url))
+  (url-size file_size (int url))
   (url-last-modified last_modified (int url))
   (url-temp url_temp (url))
   (url-scratch url_scratch (url string string int))
@@ -555,6 +610,7 @@
   (system-url->string sys_concretize (string url))
   (url-grep grep (url string url))
   (url-search-upwards search_file_upwards (url url string array_string))
+  (picture-cache-reset picture_cache_reset (void))
 
   ;; Persistent data
   (persistent-set persistent_set (void url string string))
@@ -652,6 +708,7 @@
   (object->promise-widget as_promise_widget (promise_widget object))
   (tree-bounding-rectangle get_bounding_rectangle (array_int tree))
   (widget-size get_widget_size (array_int widget))
+  (texmacs-widget-size get_texmacs_widget_size (array_int widget))
   (show-balloon show_help_balloon (void widget int int))
   (get-style-menu get_style_menu (object))
   (hidden-package? hidden_package (bool string))
@@ -659,6 +716,7 @@
   (get-remove-package-menu get_remove_package_menu (object))
   (get-toggle-package-menu get_toggle_package_menu (object))
   (refresh-now windows_refresh (void string))
+  (get-screen-size get_screen_size (array_int))
 
   ;; buffers
   (buffer-list get_all_buffers (array_url))
@@ -684,12 +742,14 @@
   (buffer-attach-notifier attach_buffer_notifier (void url))
   (buffer-has-name? buffer_has_name (bool url))
   (buffer-aux? is_aux_buffer (bool url))
+  (buffer-embedded? is_embedded_buffer (bool url))
   (buffer-import buffer_import (bool url url string))
   (buffer-load buffer_load (bool url))
   (buffer-export buffer_export (bool url url string))
   (buffer-save buffer_save (bool url))
   (tree-import-loaded import_loaded_tree (tree string url string))
   (tree-import import_tree (tree url string))
+  (tree-inclusion load_inclusion (tree url))
   (tree-export export_tree (bool tree url string))
   (tree-load-style load_style_tree (tree string))
   (buffer-focus focus_on_buffer (bool url))
@@ -706,6 +766,7 @@
   (view-delete delete_view (void url))
   (window-set-view window_set_view (void url url bool))
   (switch-to-buffer switch_to_buffer (void url))
+  (set-drd set_current_drd (void url))
 
   (window-list windows_list (array_url))
   (windows-number get_nr_windows (int))
@@ -731,8 +792,10 @@
 
   ;; transitional alternative windows; to be replaced by better solution
   (alt-window-handle window_handle (int))
-  (alt-window-create window_create (void int widget string bool))
   (alt-window-create-quit window_create (void int widget string command))
+  (alt-window-create-plain window_create_plain (void int widget string))
+  (alt-window-create-popup window_create_popup (void int widget string))
+  (alt-window-create-tooltip window_create_tooltip (void int widget string))
   (alt-window-delete window_delete (void int))
   (alt-window-show window_show (void int))
   (alt-window-hide window_hide (void int))
@@ -740,8 +803,10 @@
   (alt-window-set-size window_set_size (void int int int))
   (alt-window-get-position window_get_position (scheme_tree int))
   (alt-window-set-position window_set_position (void int int int))
+  (alt-window-search window_search (path url))
 
   ;; routines for BibTeX
+  (supports-bibtex? bibtex_present (bool))
   (bibtex-run bibtex_run (tree string string url array_string))
   (bib-add-period bib_add_period (scheme_tree scheme_tree))
   (bib-locase-first bib_locase_first (scheme_tree scheme_tree))
