@@ -27,7 +27,8 @@
   (selection-tree selection_get (tree))
 
   ;; low-level modification routines
-  (path->tree the_subtree (tree path))
+  (path-exists? test_subtree (bool path))
+  (cpp-path->tree the_subtree (tree path))
   (path-correct-old correct (void path))
   (path-insert-with insert_with (void path string content))
   (path-remove-with remove_with (void path string))
@@ -88,10 +89,22 @@
   (get-page-height get_page_height (int bool))
   (get-total-width get_total_width (int bool))
   (get-total-height get_total_height (int bool))
+  (get-reference get_ref (tree string))
+  (set-reference set_ref (void string content))
+  (reset-reference reset_ref (void string))
+  (find-references find_refs (array_string string))
+  (list-references list_refs (array_string))
+  (list-references* list_refs (array_string bool))
+  (get-auxiliary get_aux (tree string))
+  (set-auxiliary set_aux (void string content))
+  (reset-auxiliary reset_aux (void string))
+  (list-auxiliaries list_auxs (array_string))
+  (list-auxiliaries* list_auxs (array_string bool))
   (get-attachment get_att (tree string))
   (set-attachment set_att (void string content))
   (reset-attachment reset_att (void string))
   (list-attachments list_atts (array_string))
+  (list-attachments* list_atts (array_string bool))
 
   ;; modify text
   (make-htab make_htab (void string))
@@ -108,6 +121,9 @@
 
   (length-decode as_length (int string))
   (length-add add_lengths (string string string))
+  (length-sub sub_lengths (string string string))
+  (length-max max_lengths (string string string))
+  (length-min min_lengths (string string string))
   (length-mult multiply_length (string double string))
   (length? is_length (bool string))
   (length-divide divide_lengths (double string string))
@@ -192,6 +208,7 @@
   (go-page-down go_page_down (void))
   (go-start-paragraph go_start_paragraph (void))
   (go-end-paragraph go_end_paragraph (void))
+  (label->path search_label (path string))
   (go-to-label go_to_label (void string))
   (cursor-accessible? cursor_is_accessible (bool))
   (cursor-show-if-hidden show_cursor_if_hidden (void))
@@ -292,6 +309,12 @@
   (view-get-property get_property (scheme_tree scheme_tree))
   (get-window-width get_window_width (int))
   (get-window-height get_window_height (int))
+  (get-window-x get_window_x (int))
+  (get-window-y get_window_y (int))
+  (get-canvas-x get_canvas_x (int))
+  (get-canvas-y get_canvas_y (int))
+  (get-scroll-x get_scroll_x (int))
+  (get-scroll-y get_scroll_y (int))
   (clear-buffer clear_buffer (void))
   (tex-buffer tex_buffer (void))
   (clear-local-info clear_local_info (void))
@@ -304,6 +327,7 @@
   (generate-aux generate_aux (void string))
   (notify-page-change notify_page_change (void))
   (notify-change notify_change (void int))
+  (notified-change? has_changed (bool int))
   (get-metadata get_metadata (string string))
   (cpp-nr-pages nr_pages (int))
   (print-to-file print_to_file (void url))
