@@ -23,6 +23,7 @@
 #include "universal.hpp"
 #include "convert.hpp"
 #include "file.hpp"
+#include "locale.hpp"
 #include "iterator.hpp"
 #include "Freetype/tt_tools.hpp"
 #include "Database/database.hpp"
@@ -53,6 +54,13 @@ template<class T> tmscm boxP (tmscm t) {
 * Miscellaneous routines for use by glue only
 ******************************************************************************/
 
+string original_path;
+
+string
+get_original_path () {
+  return original_path;
+}
+
 string
 texmacs_version (string which) {
   if (which == "tgz") return TM_DEVEL;
@@ -63,6 +71,7 @@ texmacs_version (string which) {
   if (which == "stable") return TM_STABLE;
   if (which == "devel-release") return TM_DEVEL_RELEASE;
   if (which == "stable-release") return TM_STABLE_RELEASE;
+  if (which == "revision") return TEXMACS_REVISION;
   return TEXMACS_VERSION;
 }
 
@@ -134,6 +143,15 @@ is_busy_versioning () {
   return busy_versioning;
 }
 
+array<SI>
+get_screen_size () {
+  array<SI> r;
+  SI w, h;
+  gui_root_extents (w, h);
+  r << w << h;
+  return r;
+}
+
 /******************************************************************************
 * Redirections
 ******************************************************************************/
@@ -580,7 +598,6 @@ tmscm_to_command (tmscm o) {
   return open_box<command> (tmscm_to_blackbox (o));
 }
 
-
 /******************************************************************************
 *  Widget Factory
 ******************************************************************************/
@@ -625,7 +642,7 @@ url_to_tmscm (url u) {
 url
 tmscm_to_url (tmscm obj) {
   if (tmscm_is_string (obj))
-#ifdef __MINGW32__
+#ifdef OS_MINGW
     return url_system (tmscm_to_string (obj));
 #else
   return tmscm_to_string (obj);
@@ -734,7 +751,7 @@ var_apply (tree& t, patch p) {
 
 typedef hashmap<string,string> table_string_string;
 
-static bool
+bool
 tmscm_is_table_string_string (tmscm p) {
   if (tmscm_is_null (p)) return true;
   else if (!tmscm_is_pair (p)) return false;
@@ -1174,7 +1191,7 @@ tmscm_to_list_tree (tmscm p) {
 #include "tm_frame.hpp"
 #include "Concat/concater.hpp"
 #include "converter.hpp"
-#include "timer.hpp"
+#include "tm_timer.hpp"
 #include "Metafont/tex_files.hpp"
 #include "Freetype/tt_file.hpp"
 #include "LaTeX_Preview/latex_preview.hpp"
