@@ -22,6 +22,7 @@ struct concat_box_rep: public composite_box_rep {
   concat_box_rep (path ip, array<box> bs, array<SI> spc, bool indent);
   operator tree ();
   box adjust_kerning (int mode, double factor);
+  box expand_glyphs (int mode, double factor);
 
   void      finalize ();
   void      clear_incomplete (rectangles& rs, SI pixel, int i, int i1, int i2);
@@ -45,6 +46,7 @@ struct concat_box_rep: public composite_box_rep {
   SI        sup_lo_lim  (int level);
   SI        sup_lo_base (int level);
   SI        sup_hi_lim  (int level);
+  SI        wide_correction (int mode);
   void      get_bracket_extents (SI& lo, SI& hi);
 
   int       find_any_child (SI x, SI y, SI delta, SI& delta_out);
@@ -140,6 +142,18 @@ concat_box_rep::adjust_kerning (int mode, double factor) {
   return concat_box (ip, adj, spa, indent);
 }
 
+box
+concat_box_rep::expand_glyphs (int mode, double factor) {
+  int n= N(bs);
+  array<box> adj (n);
+  array<SI > spa (n);
+  for (int i=0; i<n; i++) {
+    adj[i]= bs[i]->expand_glyphs (mode, factor);
+    spa[i]= (SI) tm_round ((1 + factor) * spc[i]);
+  }
+  return concat_box (ip, adj, spa, indent);
+}
+
 bool
 concat_box_rep::access_allowed () {
   return false;
@@ -256,48 +270,68 @@ concat_box_rep::rsup_correction () {
 SI
 concat_box_rep::sub_lo_base (int level) {
   int i=0, n=N(bs);
-  SI  y=y1;
+  SI  y= y2;
   for (i=0; i<n; i++)
     y= min (y, bs[i]->sub_lo_base (level));
+  if (y == y2) y= y1;
   return y;
 }
 
 SI
 concat_box_rep::sub_hi_lim  (int level) {
   int i=0, n=N(bs);
-  SI  y= y1 + (y2-y1)/4;
+  SI  y= y1;
   for (i=0; i<n; i++)
     y= max (y, bs[i]->sub_hi_lim (level));
+  if (y == y1) y= y1 + (y2-y1)/4;
   return y;
 }
 
 SI
 concat_box_rep::sup_lo_lim  (int level) {
   int i=0, n=N(bs);
-  SI  y=y2 - (y2-y1)/4;
+  SI  y= y2;
   for (i=0; i<n; i++)
     y= min (y, bs[i]->sup_lo_lim (level));
+  if (y == y2) y= y2 - (y2-y1)/4;
   return y;
 }
 
 SI
 concat_box_rep::sup_lo_base (int level) {
   int i=0, n=N(bs);
-  SI  y=y2 - (y2-y1)/4;
+  SI  y= y2;
   for (i=0; i<n; i++)
     y= min (y, bs[i]->sup_lo_base (level));
+  if (y == y2) y= y2 - (y2-y1)/4;
   return y;
 }
 
 SI
 concat_box_rep::sup_hi_lim  (int level) {
   int i=0, n=N(bs);
-  SI  y=y2;
+  SI  y= y1;
   for (i=0; i<n; i++)
     y= max (y, bs[i]->sup_hi_lim (level));
+  if (y == y1) y= y2;
   return y;
 }
 
+SI
+concat_box_rep::wide_correction (int mode) {
+  SI current= 0;
+  bool done= false;
+  int i, n= N(bs);
+  for (i=0; i<n; i++)
+    if (bs[i]->w() != 0) {
+      if (done) return 0;
+      current= bs[i]->wide_correction (mode);
+      done= true;
+    }
+  if (!done && mode == 0) return 1;
+  return current;
+}
+
 void
 concat_box_rep::get_bracket_extents (SI& lo, SI& hi) {
   int i, n= N(bs);
