@@ -55,6 +55,7 @@ initialize_default_env () {
   env (GLOBAL_TITLE)     = "";          // global document title
   env (GLOBAL_AUTHOR)    = "";          // global document author
   env (GLOBAL_SUBJECT)   = "";          // global document subject
+  env (LENGTH_MODE)      = "magnified"; // length units under magnification
 
   env (FONT)             = "roman";     // the font name in text mode
   env (FONT_FAMILY)      = "rm";        // the font family in text mode
@@ -62,6 +63,7 @@ initialize_default_env () {
   env (FONT_SHAPE)       = "right";     // the font shape in text mode
   env (FONT_SIZE)        = "1";         // the font size multiplier
   env (FONT_BASE_SIZE)   = "10";        // the font base size
+  env (FONT_EFFECTS)     = "";          // additional effects applied to font
   env (MAGNIFICATION)    = "1";         // magnification (slides for instance)
   env (COLOR)            = "black";     // the color
   env (OPACITY)          = "100%";      // the opacity
@@ -70,6 +72,7 @@ initialize_default_env () {
   env (VISITED_COLOR)    = "global";    // the color of visited loci
   env (NO_PATTERNS)      = "false";     // disable background patterns
   env (LANGUAGE)         = "english";   // the language
+  env (SPACING_POLICY)   = "default";   // the spacing policy
   env (ATOM_DECORATIONS) = DATOMS;      // dots, underline, hyperlinks?, etc.
   env (LINE_DECORATIONS) = DLINES;      // boxed pars, nested envs, etc.
   env (PAGE_DECORATIONS) = DPAGES;      // future headers, footers, etc.
@@ -81,15 +84,23 @@ initialize_default_env () {
   env (MATH_FONT_FAMILY) = "mr";        // the font family in math mode
   env (MATH_FONT_SERIES) = "medium";    // the font series in math mode
   env (MATH_FONT_SHAPE)  = "normal";    // the font shape in math mode
+  env (MATH_FONT_SIZES)  = "default";   // various script sizes
   env (MATH_LEVEL)       = "0";         // the index level (0, 1 or 2)
   env (MATH_DISPLAY)     = "false";     // true if we are in display style
   env (MATH_CONDENSED)   = "false";     // ignore spaces between operators ?
   env (MATH_VPOS)        = "0";         // used in fractions (-1, 0 or 1)
   env (MATH_NESTING_MODE)= "off";       // color nested brackets?
   env (MATH_NESTING_LEVEL)= "0";        // nesting level inside brackets
+  env (MATH_FRAC_LIMIT)  = "100par";    // maximal width of 2D fraction
+  env (MATH_TABLE_LIMIT) = "100par";    // maximal width of 2D math table
+  env (MATH_FLATTEN_COLOR)  = "#448";   // bracket color when 2D width excess
+  env (MATH_TOP_SWELL_START)= "1.7ex";  // start padding above this level
+  env (MATH_TOP_SWELL_END)  = "3.5ex";  // maximal padding reached here
+  env (MATH_BOT_SWELL_START)= "-0.7ex"; // start padding below this level
+  env (MATH_BOT_SWELL_END)  = "-2.5ex"; // maximal padding reached here
 
   env (PROG_LANGUAGE)    = "scheme";    // the default programming language
-  env (PROG_SCRIPTS)     = "scheme";    // the scripting language
+  env (PROG_SCRIPTS)     = "none";      // the scripting language
   env (PROG_FONT)        = "roman";     // the font name in prog mode
   env (PROG_FONT_FAMILY) = "tt";        // the font family in prog mode
   env (PROG_FONT_SERIES) = "medium";    // the font series in prog mode
@@ -99,9 +110,13 @@ initialize_default_env () {
   env (PAR_MODE)         = "justify";   // outline method
   env (PAR_FLEXIBILITY)  = "1000";      // threshold for switching to ragged
   env (PAR_HYPHEN)       = "professional"; // quality of hyphenation
+  env (PAR_MIN_PENALTY)  = "0";         // minimal page break penalty
   env (PAR_SPACING)      = "plain";     // spacing mode (for CJK)
+  env (PAR_KERNING_REDUCE) = "auto";    // reduced kerning around characters
   env (PAR_KERNING_STRETCH)= "auto";    // extra kerning around characters
   env (PAR_KERNING_MARGIN) = "false";   // use marginal kerning (protrusion)
+  env (PAR_CONTRACTION)  = "auto";      // maximal glyph contraction
+  env (PAR_EXPANSION)    = "auto";      // maximal glyph expansion
   env (PAR_WIDTH)        = "auto";      // width of paragraph
   env (PAR_LEFT)         = "0cm";       // left indentation
   env (PAR_RIGHT)        = "0cm";       // right indentation
@@ -115,14 +130,20 @@ initialize_default_env () {
   env (PAR_FNOTE_SEP)    = "0.2fn";     // min space between diff footnotes
   env (PAR_COLUMNS)      = "1";         // number of columns
   env (PAR_COLUMNS_SEP)  = "2fn";       // separation between columns
+  env (PAR_SWELL)        = "0ex";       // extra padding around large lines
 
   env (PAGE_MEDIUM)      = "papyrus";   // paper medium: paper, papyrus, auto
   env (PAGE_PRINTED)     = "false";     // printed version?
   env (PAGE_TYPE)        = "a4";        // paper type (-> width & height)
   env (PAGE_ORIENTATION) = "portrait";  // paper orientation
+  env (PAGE_CROP_MARKS)  = "";          // paper size on which we print
   env (PAGE_WIDTH_MARGIN)  = "false";   // compute margins from par-width?
   env (PAGE_HEIGHT_MARGIN) = "false";   // compute margins from par-width?
   env (PAGE_SCREEN_MARGIN) = "true";    // special margins for screen editing?
+  env (PAGE_SINGLE)      = "false";     // display page by page
+  env (PAGE_PACKET)      = "1";         // display pages by packets of 1,2,...
+  env (PAGE_OFFSET)      = "0";         // first page's offset in first packet
+  env (PAGE_BORDER)      = "default";   // page border decorations
   env (PAGE_BREAKING)    = "professional";  // quality of page breaking
   env (PAGE_FLEXIBILITY) = "1";         // flexibility factor of stretch
   env (PAGE_FIRST)       = "1";         // number of first page
@@ -146,6 +167,7 @@ initialize_default_env () {
   env (PAGE_ODD_FOOTER)  = "";          // footer on odd pages
   env (PAGE_EVEN_HEADER) = "";          // header on even pages
   env (PAGE_EVEN_FOOTER) = "";          // footer on even pages
+  env (PAGE_THIS_BG_COLOR) = "";        // background color of this page
   env (PAGE_SCREEN_WIDTH)  = "10cm";    // width of current window (for auto)
   env (PAGE_SCREEN_HEIGHT) = "10cm";    // height of current window (for auto)
   env (PAGE_SCREEN_LEFT) = "5mm";       // left margin for screen editing
@@ -156,6 +178,7 @@ initialize_default_env () {
   env (PAGE_FNOTE_SEP)   = "1.0fn";     // space between text & footnotes
   env (PAGE_FNOTE_BARLEN)= "7.5fn";     // length of footnote separating bar
   env (PAGE_FLOAT_SEP)   = "1.5fn";     // space between text & floats
+  env (PAGE_FLOAT_ENABLE)= "paper";     // enable floating objects
   env (PAGE_MNOTE_SEP)   = "5mm";       // space between text & marginal notes
   env (PAGE_MNOTE_WIDTH) = "15mm";      // width of marginal notes
 
@@ -183,7 +206,7 @@ initialize_default_env () {
   env (TABLE_MAX_COLS)   = "";          // suggested maximal number of columns
 
   env (CELL_DECORATION)  = "";          // decorating table of cell
-  env (CELL_FORMAT)      = TFORMAT;     // format of cell
+  env (CELL_FORMAT)      = tree(TFORMAT); // format of cell
   env (CELL_BACKGROUND)  = "";          // background color of cell
   env (CELL_ORIENTATION) = "portrait";  // orientation of cell  
   env (CELL_WIDTH)       = "";          // width of cell
@@ -209,6 +232,7 @@ initialize_default_env () {
   env (CELL_COL_SPAN)    = "1";         // column span of cell
   env (CELL_ROW_NR)      = "1";         // row coordinate of cell
   env (CELL_COL_NR)      = "1";         // column coordinate of cell
+  env (CELL_SWELL)       = "0ex";       // extra padding around large cells
 
   env (GR_GEOMETRY)         = gr_geometry;    // geometry of graphics
   env (GR_FRAME)            = gr_frame;       // coordinate frame for graphics
@@ -220,15 +244,22 @@ initialize_default_env () {
   env (GR_EDIT_GRID)        = gr_edit_grid;   // edit grid
   env (GR_EDIT_GRID_ASPECT) = gr_grid_aspect; // edit grid (subdivisions)
   env (GR_TRANSFORMATION)   = gr_transf;      // 3D transformation
+  env (GR_SNAP_DISTANCE)    = "10px";         // default snap distance
 
+  env (GR_GID)           = "default";   // graphical identifier of new objects
+  env (GR_ANIM_ID)       = "default";   // animation identifier of new objects
+  env (GR_PROVISO)       = "default";   // visibility condition for new objects
   env (GR_MAGNIFY)       = "default";   // magnify of new objects
   env (GR_OPACITY)       = "default";   // opacity of new objects
   env (GR_COLOR)         = "default";   // color of new objects
   env (GR_POINT_STYLE)   = "default";   // point style of new objects
+  env (GR_POINT_SIZE)    = "default";   // point size of new objects
+  env (GR_POINT_BORDER)  = "default";   // point border width for new objects
   env (GR_LINE_WIDTH)    = "default";   // line width for new objects
   env (GR_LINE_JOIN)     = "default";   // line join for new objects
   env (GR_LINE_CAPS)     = "default";   // line caps for new objects
   env (GR_LINE_EFFECTS)    = "default";   // line effects for new objects
+  env (GR_LINE_PORTION)    = "default";   // curve portion for new objects
   env (GR_DASH_STYLE)      = "default";   // dash style for new objects
   env (GR_DASH_STYLE_UNIT) = "default";   // dash style unit for new objects
   env (GR_ARROW_BEGIN)     = "default";   // arrow begin for new objects
@@ -240,14 +271,25 @@ initialize_default_env () {
   env (GR_TEXT_AT_HALIGN)  = "default";   // horiz. alignment for new text-ats
   env (GR_TEXT_AT_VALIGN)  = "default";   // vert. alignment for new text-ats
   env (GR_TEXT_AT_MARGIN)  = "default";   // margins for new text-ats
+  env (GR_DOC_AT_VALIGN)   = "default";   // vert. al. for new document-ats
+  env (GR_DOC_AT_WIDTH)    = "default";   // width of new document-ats
+  env (GR_DOC_AT_HMODE)    = "default";   // width mode for new document-ats
+  env (GR_DOC_AT_PPSEP)    = "default";   // par-par-sep for new document-ats
+  env (GR_DOC_AT_BORDER)   = "default";   // border of new document-ats
+  env (GR_DOC_AT_PADDING)  = "default";   // padding for new document-ats
 
   env (GID)              = "default";   // graphical identifier
+  env (ANIM_ID)          = "";          // identifier inside animations
+  env (PROVISO)          = "true";      // visibility condition
   env (MAGNIFY)          = "1";         // magnification for graphical objects
   env (POINT_STYLE)      = "disk";      // point style (square, circle...)
+  env (POINT_SIZE)       = "2.5ln";     // point size
+  env (POINT_BORDER)     = "1ln";       // point border width
   env (LINE_WIDTH)       = "1ln";       // line width in graphics
   env (LINE_JOIN)        = "normal";    // junctions in multilines
   env (LINE_CAPS)        = "normal";    // caps at ends
   env (LINE_EFFECTS)     = "none";      // effects to be applied on line
+  env (LINE_PORTION)     = "1";         // portion of curve to be drawn
   env (DASH_STYLE)       = "none";      // dash style
   env (DASH_STYLE_UNIT)  = "5ln";       // dash style unit
   env (ARROW_BEGIN)      = "none";      // arrow at beginning of line
@@ -258,12 +300,19 @@ initialize_default_env () {
   env (FILL_STYLE)       = "plain";     // fill style
   env (TEXT_AT_HALIGN)   = "left";      // horizontal text-at alignment
   env (TEXT_AT_VALIGN)   = "base";      // vertical text-at alignment
-  env (TEXT_AT_MARGIN)   = "base";      // margin for smart guides
+  env (TEXT_AT_MARGIN)   = "1spc";      // margin for smart guides
+  env (DOC_AT_VALIGN)    = "top";       // vertical document-at alignment
+  env (DOC_AT_WIDTH)     = "1par";      // width of document-ats
+  env (DOC_AT_HMODE)     = "min";       // width mode for document-ats
+  env (DOC_AT_PPSEP)     = "0fn";       // par-par-sep for document-ats
+  env (DOC_AT_BORDER)    = "0ln";       // border of document-ats
+  env (DOC_AT_PADDING)   = "0spc";      // padding for document-ats
 
   env (SRC_STYLE)        = "angular";   // style for "source" tags
   env (SRC_SPECIAL)      = "normal";    // special treatment of certain tags
   env (SRC_COMPACT)      = "normal";    // compact inline/multi-paragraph tags?
   env (SRC_CLOSE)        = "compact";   // how to close long tags
+  env (SRC_TAG_COLOR)    = "blue";      // default source tag color
 
   env (CANVAS_TYPE)           = "plain";        // which kind of scrollbars
   env (CANVAS_COLOR)          = "white";        // canvas colour
@@ -276,6 +325,7 @@ initialize_default_env () {
   env (ORNAMENT_TITLE_STYLE)  = "classic";      // title style
   env (ORNAMENT_BORDER)       = "1ln";          // border width
   env (ORNAMENT_SWELL)        = "0.5";          // border swell
+  env (ORNAMENT_CORNER)       = "100%";         // corner rounding
   env (ORNAMENT_HPADDING)     = "1spc";         // horizontal padding of body
   env (ORNAMENT_VPADDING)     = "1spc";         // vertical padding of body
   env (ORNAMENT_COLOR)        = "";             // background color
@@ -299,6 +349,9 @@ initialize_default_env () {
   tree ln2 (LINK, "action", copy (src_id), copy (dest_script));
   tree ln3 (LINK, "hyperlink", copy (ref_id), copy (dest_ref));
   tree ln4 (LINK, "anchor", anchor);
+  //tree sc1 (SCRIPT, "preview-reference",
+  //          tree (QUOTE_ARG, "Id"), tree (ARG, "Id"));
+  //tree ln5 (LINK, "mouse-over", copy (ref_id), sc1);
   tree labflag (FLAG, tree (ARG, "Id"), "blue", "Id");
   tree labtxt (SET_BINDING, tree (ARG, "Id"), tree (VALUE, THE_LABEL));
   tree merged (MERGE, tree (VALUE, THE_TAGS), tuple (tree (ARG, "Id")));
@@ -322,9 +375,11 @@ initialize_default_env () {
 			   tree (SURROUND, tagflag, "",
                                  tree (ARG, "body"))));
   env ("reference")= tree (MACRO, "Id",
-			   tree (LOCUS, copy (ref_id), ln3, reftxt));
+			   tree (LOCUS, copy (ref_id), ln3,
+                                 /*ln5,*/ reftxt));
   env ("pageref")= tree (MACRO, "Id",
-			 tree (LOCUS, copy (ref_id), copy (ln3), preftxt));
+			 tree (LOCUS, copy (ref_id), copy (ln3),
+                               /*copy (ln5),*/ preftxt));
   env ("include")= tree (MACRO, "name",
 			 tree (VAR_INCLUDE, tree (ARG, "name")));
 
@@ -364,12 +419,18 @@ initialize_default_env () {
     tree (MACRO, "body", tree (WITH, COLOR, "#4040c0", tree (ARG, "body")));
   env ("mutator")=
     tree (MACRO, "body", "y", tree (ARG, "body"));
+  env ("phantom-float")=
+    tree (MACRO, "type", "pos",
+          tree (CONCAT, tree (FLAG, "float", "brown"),
+                tree (_FLOAT,  tree (ARG, "type"), tree (ARG, "pos"),
+                      tree (DOCUMENT))));
 
   /* syntactic highlighting */
   env ("src-regular")=
     tree (MACRO, "body", tree (WITH, COLOR, "black", tree (ARG, "body")));
   env ("src-macro")=
-    tree (MACRO, "body", tree (WITH, MODE, "src", COLOR, "blue",
+    tree (MACRO, "body", tree (WITH, MODE, "src",
+                               COLOR, tree (VALUE, SRC_TAG_COLOR),
                                FONT_FAMILY, "ss", tree (ARG, "body")));
   env ("src-var")=
     tree (MACRO, "body", tree (WITH, MODE, "src", COLOR, "dark green",
