@@ -39,22 +39,29 @@ initialize_default_var_type () {
   var_type (FONT_SHAPE)         = Env_Font;
   var_type (FONT_SIZE)          = Env_Font_Size;
   var_type (FONT_BASE_SIZE)     = Env_Font_Size;
+  var_type (FONT_EFFECTS)       = Env_Font;
   var_type (MAGNIFICATION)      = Env_Magnification;
   var_type (MAGNIFY)            = Env_Magnify;
   var_type (COLOR)              = Env_Color;
   var_type (OPACITY)            = Env_Color;
   var_type (NO_PATTERNS)        = Env_Pattern_Mode;
   var_type (LANGUAGE)           = Env_Language;
+  var_type (SPACING_POLICY)     = Env_Spacing;
 
   var_type (MATH_LANGUAGE)      = Env_Language;
   var_type (MATH_FONT)          = Env_Font;
   var_type (MATH_FONT_FAMILY)   = Env_Font;
   var_type (MATH_FONT_SERIES)   = Env_Font;
   var_type (MATH_FONT_SHAPE)    = Env_Font;
+  var_type (MATH_FONT_SIZES)    = Env_Font_Sizes;
   var_type (MATH_LEVEL)         = Env_Index_Level;
   var_type (MATH_DISPLAY)       = Env_Display_Style;
   var_type (MATH_CONDENSED)     = Env_Math_Condensed;
   var_type (MATH_VPOS)          = Env_Vertical_Pos;
+  var_type (MATH_NESTING_LEVEL) = Env_Math_Nesting;
+  var_type (MATH_FRAC_LIMIT)    = Env_Math_Width;
+  var_type (MATH_TABLE_LIMIT)   = Env_Math_Width;
+  var_type (MATH_FLATTEN_COLOR) = Env_Math_Width;
 
   var_type (PROG_LANGUAGE)      = Env_Language;
   var_type (PROG_FONT)          = Env_Font;
@@ -65,9 +72,13 @@ initialize_default_var_type () {
   var_type (PAR_MODE)           = Env_Paragraph;
   var_type (PAR_FLEXIBILITY)    = Env_Paragraph;
   var_type (PAR_HYPHEN)         = Env_Paragraph;
+  var_type (PAR_MIN_PENALTY)    = Env_Paragraph;
   var_type (PAR_SPACING)        = Env_Paragraph;
+  var_type (PAR_KERNING_REDUCE) = Env_Paragraph;
   var_type (PAR_KERNING_STRETCH)= Env_Paragraph;
   var_type (PAR_KERNING_MARGIN) = Env_Paragraph;
+  var_type (PAR_CONTRACTION)    = Env_Paragraph;
+  var_type (PAR_EXPANSION)      = Env_Paragraph;
   var_type (PAR_HYPHEN)         = Env_Paragraph;
   var_type (PAR_WIDTH)          = Env_Paragraph;
   var_type (PAR_LEFT)           = Env_Paragraph;
@@ -86,6 +97,7 @@ initialize_default_var_type () {
   var_type (PAGE_FIRST)         = Env_Fixed;
   var_type (PAGE_WIDTH)         = Env_Page_Extents;
   var_type (PAGE_HEIGHT)        = Env_Page_Extents;
+  var_type (PAGE_CROP_MARKS)    = Env_Page_Extents;
   var_type (PAGE_WIDTH_MARGIN)  = Env_Page;
   var_type (PAGE_SCREEN_MARGIN) = Env_Page;
   var_type (PAGE_NR)            = Env_Page;
@@ -106,8 +118,11 @@ initialize_default_var_type () {
   var_type (PAGE_ODD_FOOTER)    = Env_Page;
   var_type (PAGE_EVEN_HEADER)   = Env_Page;
   var_type (PAGE_EVEN_FOOTER)   = Env_Page;
+  var_type (PAGE_THIS_TOP)      = Env_Page;
+  var_type (PAGE_THIS_BOT)      = Env_Page;
   var_type (PAGE_THIS_HEADER)   = Env_Page;
   var_type (PAGE_THIS_FOOTER)   = Env_Page;
+  var_type (PAGE_THIS_BG_COLOR) = Env_Page;
   var_type (PAGE_FNOTE_SEP)     = Env_Page;
   var_type (PAGE_FNOTE_BARLEN)  = Env_Page;
   var_type (PAGE_FLOAT_SEP)     = Env_Page;
@@ -115,6 +130,8 @@ initialize_default_var_type () {
   var_type (PAGE_MNOTE_WIDTH)   = Env_Page;
 
   var_type (POINT_STYLE)        = Env_Point_Style;
+  var_type (POINT_SIZE)         = Env_Point_Size;
+  var_type (POINT_BORDER)       = Env_Point_Size;
   var_type (LINE_WIDTH)         = Env_Line_Width;
   var_type (DASH_STYLE)         = Env_Dash_Style;
   var_type (DASH_STYLE_UNIT)    = Env_Dash_Style_Unit;
@@ -123,8 +140,10 @@ initialize_default_var_type () {
   var_type (ARROW_END)          = Env_Line_Arrows;
   var_type (ARROW_LENGTH)       = Env_Line_Arrows;
   var_type (ARROW_HEIGHT)       = Env_Line_Arrows;
+  var_type (LINE_PORTION)       = Env_Line_Portion;
   var_type (TEXT_AT_HALIGN)     = Env_Text_At_Halign;
   var_type (TEXT_AT_VALIGN)     = Env_Text_At_Valign;
+  var_type (DOC_AT_VALIGN)      = Env_Doc_At_Valign;
   var_type (GR_FRAME)           = Env_Frame;
   var_type (GR_GEOMETRY)        = Env_Geometry;
   var_type (GR_GRID)            = Env_Grid;
@@ -136,6 +155,7 @@ initialize_default_var_type () {
   var_type (SRC_SPECIAL)        = Env_Src_Special;
   var_type (SRC_COMPACT)        = Env_Src_Compact;
   var_type (SRC_CLOSE)          = Env_Src_Close;
+  var_type (SRC_TAG_COLOR)      = Env_Src_Color;
 }
 
 /******************************************************************************
@@ -149,6 +169,9 @@ initialize_default_var_type () {
 
 void
 edit_env_rep::update_page_pars () {
+  double magn_old= magn_len;
+  magn_len= 1.0;
+
   page_type         = get_string (PAGE_TYPE);
   page_landscape    = (get_string (PAGE_ORIENTATION) == "landscape");
   page_automatic    = (get_string (PAGE_MEDIUM) == "automatic");
@@ -156,6 +179,10 @@ edit_env_rep::update_page_pars () {
   string height_flag= get_string (PAGE_HEIGHT_MARGIN);
   bool   screen_flag= get_bool   (PAGE_SCREEN_MARGIN);
 
+  page_floats       = (get_string (PAGE_FLOAT_ENABLE) == "true");
+  if (get_string (PAGE_FLOAT_ENABLE) == get_string (PAGE_MEDIUM))
+    page_floats= true;
+
   if (page_automatic) {
     page_width        = get_length (PAGE_SCREEN_WIDTH);
     page_height       = get_length (PAGE_SCREEN_HEIGHT);
@@ -227,6 +254,25 @@ edit_env_rep::update_page_pars () {
       page_height= page_user_height + page_top_margin + page_bottom_margin;
     }
   }
+
+  string crop_marks= get_string (PAGE_CROP_MARKS);
+  page_real_type  = page_type;
+  page_real_width = page_width;
+  page_real_height= page_height;
+  if (crop_marks != "" && get_string (PAGE_MEDIUM) == "paper") {
+    page_real_type= crop_marks;
+    page_real_width=
+      as_length (page_get_feature (crop_marks, PAGE_WIDTH, page_landscape));
+    page_real_height=
+      as_length (page_get_feature (crop_marks, PAGE_HEIGHT, page_landscape));
+  }
+  
+  page_single= get_bool (PAGE_SINGLE);
+  page_packet= get_int (PAGE_PACKET);
+  page_offset= get_int (PAGE_OFFSET);
+  page_border= read (PAGE_BORDER);
+
+  magn_len= magn_old;
 }
 
 void
@@ -244,8 +290,11 @@ edit_env_rep::get_page_pars (SI& w, SI& h, SI& width, SI& height,
 
   int nr_cols= get_int (PAR_COLUMNS);
   if (nr_cols > 1) {
+    double magn_old= magn_len;
+    magn_len= 1.0;
     SI col_sep= get_length (PAR_COLUMNS_SEP);
     w= ((w+col_sep) / nr_cols) - col_sep;
+    magn_len= magn_old;
   }
 
   /*
@@ -261,6 +310,38 @@ edit_env_rep::get_page_pars (SI& w, SI& h, SI& width, SI& height,
   */
 }
 
+SI
+edit_env_rep::get_page_width (bool deco) {
+  SI w= page_user_width + page_odd_margin + page_right_margin;
+  if (get_string (PAGE_MEDIUM) == "paper" &&
+      get_string (PAGE_BORDER) != "none" &&
+      deco) w += 20 * pixel;
+  return w;
+}
+
+SI
+edit_env_rep::get_pages_width (bool deco) {
+  SI w= page_user_width + page_odd_margin + page_right_margin;
+  if (get_string (PAGE_MEDIUM) == "paper" &&
+      get_string (PAGE_BORDER) != "attached" &&
+      get_string (PAGE_BORDER) != "none" &&
+      deco) w += 20 * pixel;
+  w= w * page_packet;
+  if (get_string (PAGE_MEDIUM) == "paper" &&
+      get_string (PAGE_BORDER) == "attached" &&
+      deco) w += 20 * pixel;
+  return w;
+}
+
+SI
+edit_env_rep::get_page_height (bool deco) {
+  SI h= page_user_height + page_top_margin + page_bottom_margin;
+  if (get_string (PAGE_MEDIUM) == "paper" &&
+      get_string (PAGE_BORDER) != "none" &&
+      deco) h += 20 * pixel;
+  return h;
+}
+
 /******************************************************************************
 * Retrieving ornament parameters
 ******************************************************************************/
@@ -285,6 +366,7 @@ edit_env_rep::get_ornament_parameters () {
   int   a     = alpha;
   tree  w     = read (ORNAMENT_BORDER);
   tree  ext   = read (ORNAMENT_SWELL);
+  tree  cor   = read (ORNAMENT_CORNER);
   tree  xpad  = read (ORNAMENT_HPADDING);
   tree  ypad  = read (ORNAMENT_VPADDING);
 
@@ -340,13 +422,126 @@ edit_env_rep::get_ornament_parameters () {
   }
   else bpad= tpad= as_length (ypad);
 
+
+  double corf= 1.0;
+  if (shape != "classic") {
+    if (shape == "rounded") corf= 2.0;
+    if (shape == "angular") corf= 1.5;
+  }
+  
+  SI lcor, bcor, rcor, tcor;
+  if (is_percentage (cor)) {
+    double a= as_percentage (cor) * corf;
+    lcor= rcor= (SI) round (a * min (lpad, rpad));
+    bcor= tcor= (SI) round (a * min (bpad, tpad));
+  }
+  else if (is_atomic (cor) && !occurs (",", cor->label))
+    lcor= bcor= rcor= tcor= as_length (cor);
+  else {
+    if (is_atomic (cor))
+      cor= tuplify (cor);
+    if (is_func (cor, TUPLE, 2))
+      cor= tuple (cor[0], cor[1], cor[0], cor[1]);
+    if (!is_func (cor, TUPLE, 4))
+      lcor= bcor= rcor= tcor= 0;
+    else {
+      if (is_percentage (cor[0]))
+        lcor= (SI) round (as_percentage (cor[0]) * corf * lpad);
+      else lcor= as_length (cor[0]);
+      if (is_percentage (cor[1])) 
+        bcor= (SI) round (as_percentage (cor[1]) * corf * bpad);
+      else bcor= as_length (cor[1]);
+      if (is_percentage (cor[2])) 
+        rcor= (SI) round (as_percentage (cor[2]) * corf * rpad);
+      else rcor= as_length (cor[2]);
+      if (is_percentage (cor[3])) 
+        tcor= (SI) round (as_percentage (cor[3]) * corf * tpad);
+      else tcor= as_length (cor[3]);
+    }
+  }
+  
   return ornament_parameters (shape, tst,
                               lw, bw, rw, tw,
 			      lx, bx, rx, tx,
 			      lpad, bpad, rpad, tpad,
+                              lcor, bcor, rcor, tcor,
                               brush (bg, a), brush (xc, a), border);
 }
 
+art_box_parameters
+edit_env_rep::get_art_box_parameters (tree t) {
+  tree data (TUPLE);
+  SI lpad= 0, rpad= 0, bpad= 0, tpad= 0;
+  for (int i=1; i<N(t); i++)
+    if (is_func (t[i], TUPLE) && N(t[i]) >= 2) {
+      tree u= t[i], r (TUPLE);
+      bool main_text= (u[0] == "text");
+      for (int j=0; j+1<N(u); j+=2) {
+        tree var= exec (u[j]);
+        tree val= exec (u[j+1]);
+        if (is_atomic (val) && is_length (val->label))
+          val= as_tmlen (val);
+        r << var << val;
+        if (main_text) {
+          if      (var == "lpadding") lpad= as_length (val);
+          else if (var == "rpadding") rpad= as_length (val);
+          else if (var == "bpadding") bpad= as_length (val);
+          else if (var == "tpadding") tpad= as_length (val);
+        }
+      }
+      data << r;
+    }
+  return art_box_parameters (data, lpad, bpad, rpad, tpad);
+}
+
+/******************************************************************************
+* Various font sizes for scripts
+******************************************************************************/
+
+static array<int>
+determine_sizes (tree szt, int sz) {
+  array<int> r;
+  r << sz;
+  if (is_tuple (szt))
+    for (int i=0; i<N(szt); i++)
+      if (is_tuple (szt[i]) && N(szt[i]) >= 1)
+	if (szt[i][0] == "all" ||
+	    szt[i][0] == as_string (sz)) {
+	  for (int j=1; j<N(szt[i]); j++)
+	    if (is_atomic (szt[i][j])) {
+	      string s= szt[i][j]->label;
+	      if (is_int (s)) r << as_int (s);
+	      else if (starts (s, "*")) {
+		s= s (1, N(s));
+		double x= 1.0;
+		int d= search_forwards ("/", s);
+		if (d >= 0 && is_double (s(0, d)) && is_double (s(d+1, N(s))))
+		  x= as_double (s (0, d)) / as_double (s (d+1, N(s)));
+		else if (is_double (s)) x= as_double (s);
+		int xsz= (int) ceil ((x - 0.001) * sz);
+		r << xsz;
+	      }
+	    }
+	  //cout << szt << ", " << sz << " -> " << r << LF;
+	  return r;
+	}
+  r << script (sz, 1);
+  r << script (sz, 2);
+  //cout << szt << ", " << sz << " -> " << r << LF;
+  return r;
+}
+
+int
+edit_env_rep::get_script_size (int sz, int level) {
+  while (sz >= N(size_cache)) {
+    int xsz= N(size_cache);
+    size_cache << determine_sizes (math_font_sizes, xsz);
+  }
+  array<int>& a (size_cache[sz]);
+  if (level < N(a)) return a[level];
+  else return a[N(a) - 1];
+}
+
 /******************************************************************************
 * Updating the environment from the variables
 ******************************************************************************/
@@ -360,23 +555,25 @@ edit_env_rep::update_font () {
   case 1:
     fn= smart_font (get_string (FONT), get_string (FONT_FAMILY),
                     get_string (FONT_SERIES), get_string (FONT_SHAPE),
-                    script (fn_size, index_level), (int) (magn*dpi));
+                    get_script_size (fn_size, index_level), (int) (magn*dpi));
     break;
   case 2:
     fn= smart_font (get_string (MATH_FONT), get_string (MATH_FONT_FAMILY),
                     get_string (MATH_FONT_SERIES), get_string (MATH_FONT_SHAPE),
                     get_string (FONT), get_string (FONT_FAMILY),
                     get_string (FONT_SERIES), "mathitalic",
-                    script (fn_size, index_level), (int) (magn*dpi));
+                    get_script_size (fn_size, index_level), (int) (magn*dpi));
     break;
   case 3:
     fn= smart_font (get_string (PROG_FONT), get_string (PROG_FONT_FAMILY),
                     get_string (PROG_FONT_SERIES), get_string (PROG_FONT_SHAPE),
                     get_string (FONT), get_string (FONT_FAMILY) * "-tt",
                     get_string (FONT_SERIES), get_string (FONT_SHAPE),
-                    script (fn_size, index_level), (int) (magn*dpi));
+                    get_script_size (fn_size, index_level), (int) (magn*dpi));
     break;
   }
+  string eff= get_string (FONT_EFFECTS);
+  if (N(eff) != 0) fn= apply_effects (fn, eff);
 }
 
 int
@@ -403,9 +600,15 @@ edit_env_rep::update_color () {
   tree pc= env [COLOR];
   tree fc= env [FILL_COLOR];
   if (pc == "none") pen= pencil (false);
-  else pen= pencil (pc, alpha, get_length (LINE_WIDTH));
+  else {
+    if (L(pc) == PATTERN) pc= exec (pc);
+    pen= pencil (pc, alpha, get_length (LINE_WIDTH));
+  }
   if (fc == "none") fill_brush= brush (false);
-  else fill_brush= brush (fc, alpha);
+  else {
+    if (L(fc) == PATTERN) fc= exec (fc);
+    fill_brush= brush (fc, alpha);
+  }
 }
 
 void
@@ -473,8 +676,10 @@ edit_env_rep::update_geometry () {
   gh= as_length ("0.6par");
   gvalign= as_string ("center");
   if (is_tuple (t, "geometry", 2) || is_tuple (t, "geometry", 3)) {
-    if (is_length (as_string (t[1]))) gw= as_length (t[1]);
-    if (is_length (as_string (t[2]))) gh= as_length (t[2]);
+    if (is_length (as_string (t[1])) || is_anylen (t[1]))
+      gw= as_length (t[1]);
+    if (is_length (as_string (t[2])) || is_anylen (t[2]))
+      gh= as_length (t[2]);
     if (is_tuple (t, "geometry", 3))
       gvalign= as_string (t[3]);
   }
@@ -560,6 +765,7 @@ void
 edit_env_rep::update_dash_style () {
   tree t= env [DASH_STYLE];
   dash_style= array<bool> (0);
+  dash_motif= array<point> (0);
   if (is_string (t)) {
     string s= as_string (t);
     if (N(s) > 0 && (s[0] == '0' || s[0] == '1')) {
@@ -568,14 +774,67 @@ edit_env_rep::update_dash_style () {
       for (i=0; i<n; i++)
         dash_style[i]= (s[i] != '0');
     }
+    else if (N(s) > 0) {
+      if (s == "zigzag")
+        dash_motif << point (0.25, 0.5) << point (0.75, -0.5);
+      else if (s == "wave")
+        for (int k=1; k<=11; k++)
+          dash_motif << point (k / 12.0, sin (3.141592*k / 6.0) / 2.0);
+      else if (s == "pulse")
+        dash_motif << point (0.0, 0.5) << point (0.5, 0.5)
+                   << point (0.5, -0.5) << point (1.0, -0.5);
+      else if (s == "loops")
+        for (int k=1; k<=11; k++) {
+          double c= (1.0 - cos (3.141592*k / 6.0)) / 2.0;
+          double s= sin (3.141592*k / 6.0) / 2.0;
+          dash_motif << point (k / 12.0 + c, s);
+        }
+      else if (s == "meander") {
+        double u= 1.0 / 12.0;
+        dash_motif << point (-0.5*u, 0.0)
+                   << point (5*u, 0.0) << point (5*u, 5*u)
+                   << point (u, 5*u) << point (u, 2*u)
+                   << point (3*u, 2*u) << point (3*u, 3*u)
+                   << point (2*u, 3*u) << point (2*u, 4*u)
+                   << point (4*u, 4*u) << point (4*u, u)
+                   << point (0.0, u) << point (0.0, 6*u)
+                   << point (5.5*u, 6*u)
+                   << point (11*u, 6*u) << point (11*u, u)
+                   << point (7*u, u) << point (7*u, 4*u)
+                   << point (9*u, 4*u) << point (9*u, 3*u)
+                   << point (8*u, 3*u) << point (8*u, 2*u)
+                   << point (10*u, 2*u) << point (10*u, 5*u)
+                   << point (6*u, 5*u) << point (6*u, 0.0);
+        for (int i=0; i<N(dash_motif); i++)
+          dash_motif[i][0] += 0.5*u;
+      }
+      for (int i=0; i<N(dash_motif); i++)
+        dash_motif[i][1] *= dash_style_ratio;
+    }
   }
 }
 
+void
+edit_env_rep::update_dash_style_unit () {
+  tree t= read (DASH_STYLE_UNIT);
+  if (is_tuple (t) && N(t) == 2) {
+    SI hunit= as_length (t[0]);
+    SI vunit= as_length (t[1]);
+    dash_style_unit = hunit;
+    dash_style_ratio= ((double) vunit) / max (((double) hunit), 1.0);
+  }
+  else {
+    dash_style_unit= as_length (t);
+    dash_style_ratio= 1.0;
+  }
+  if (N(dash_motif) != 0) update_dash_style ();
+}
+
 void
 decompose_length (string s, double& x, string& un) {
   int i;
   for (i=0; i<N(s); i++)
-    if ((s[i]>='a') && (s[i]<='z')) break;
+    if (is_locase (s[i])) break;
   x = as_double (s (0, i));
   un= s (i, N(s));
 }
@@ -667,6 +926,10 @@ edit_env_rep::update_line_arrows () {
   string h= get_string (ARROW_HEIGHT);
   line_arrows[0]= decode_arrow (env [ARROW_BEGIN], l, h);
   line_arrows[1]= decode_arrow (env [ARROW_END], l, h);
+  if (line_arrows[0] != "")
+    line_arrows[0]= tree (WITH, LINE_PORTION, "1", line_arrows[0]);
+  if (line_arrows[1] != "")
+    line_arrows[1]= tree (WITH, LINE_PORTION, "1", line_arrows[1]);
 }
 
 void
@@ -674,11 +937,16 @@ edit_env_rep::update () {
   zoomf          = normal_zoom (get_double (ZOOM_FACTOR));
   pixel          = (SI) tm_round ((std_shrinkf * PIXEL) / zoomf);
   magn           = get_double (MAGNIFICATION);
+  magn_len       = (get_string (LENGTH_MODE) == "fixed"? 1.0: magn);
   index_level    = get_int (MATH_LEVEL);
   display_style  = get_bool (MATH_DISPLAY);
   math_condensed = get_bool (MATH_CONDENSED);
   vert_pos       = get_int (MATH_VPOS);
+  nesting_level  = get_int (MATH_NESTING_LEVEL);
   preamble       = get_bool (PREAMBLE);
+  spacing_policy = get_spacing_id (env[SPACING_POLICY]);
+  math_font_sizes= env[MATH_FONT_SIZES];
+  size_cache     = array<array<int> > ();
 
   update_mode ();
   update_info_level ();
@@ -687,19 +955,29 @@ edit_env_rep::update () {
 
   update_geometry ();
   update_frame ();
-  point_style= get_string (POINT_STYLE);
+  point_style = get_string (POINT_STYLE);
+  point_size  = get_length (POINT_SIZE);
+  point_border= get_length (POINT_BORDER);
   update_color ();
   update_pattern_mode ();
   update_dash_style ();
-  dash_style_unit= get_length (DASH_STYLE_UNIT);
+  update_dash_style_unit ();
   update_line_arrows ();
+  line_portion= get_double (LINE_PORTION);
   text_at_halign= get_string (TEXT_AT_HALIGN);
   text_at_valign= get_string (TEXT_AT_VALIGN);
+  doc_at_valign= get_string (DOC_AT_VALIGN);
 
   update_src_style ();
   update_src_special ();
   update_src_compact ();
   update_src_close ();
+  src_tag_color= get_string (SRC_TAG_COLOR);
+  src_tag_col= named_color (src_tag_color);
+
+  frac_max   = get_length (MATH_FRAC_LIMIT);
+  table_max  = get_length (MATH_TABLE_LIMIT);
+  flatten_pen= pencil (env[MATH_FLATTEN_COLOR], alpha, get_length (LINE_WIDTH));
 }
 
 /******************************************************************************
@@ -719,11 +997,16 @@ edit_env_rep::update (string s) {
     break;
   case Env_Magnification:
     magn= get_double (MAGNIFICATION);
+    magn_len= (get_string (LENGTH_MODE) == "fixed"? 1.0: magn);
     update_font ();
     update_color ();
+    update_dash_style_unit ();
     break;
   case Env_Magnify:
     mgfy= get_double (MAGNIFY);
+    update_font ();
+    update_color ();
+    update_dash_style_unit ();
     break;
   case Env_Language:
     update_language ();
@@ -742,6 +1025,11 @@ edit_env_rep::update (string s) {
   case Env_Font_Size:
     update_font ();
     break;
+  case Env_Font_Sizes:
+    math_font_sizes= env[MATH_FONT_SIZES];
+    size_cache= array<array<int> > ();
+    update_font ();
+    break;
   case Env_Index_Level:
     index_level= get_int (MATH_LEVEL);
     update_font ();
@@ -755,12 +1043,23 @@ edit_env_rep::update (string s) {
   case Env_Vertical_Pos:
     vert_pos= get_int (MATH_VPOS);
     break;
+  case Env_Math_Nesting:
+    nesting_level= get_int (MATH_NESTING_LEVEL);
+    break;
+  case Env_Math_Width:
+    frac_max= get_length (MATH_FRAC_LIMIT);
+    table_max= get_length (MATH_TABLE_LIMIT);
+    flatten_pen= pencil (env[MATH_FLATTEN_COLOR], alpha, get_length (LINE_WIDTH));
+    break;
   case Env_Color:
     update_color ();
     break;
   case Env_Pattern_Mode:
     update_pattern_mode ();
     break;
+  case Env_Spacing:
+    spacing_policy= get_spacing_id (env[SPACING_POLICY]);
+    break;
   case Env_Paragraph:
     break;
   case Env_Page:
@@ -780,6 +1079,10 @@ edit_env_rep::update (string s) {
   case Env_Point_Style:
     point_style= get_string (POINT_STYLE);
     break;
+  case Env_Point_Size:
+    point_size= get_length (POINT_SIZE);
+    point_border= get_length (POINT_BORDER);
+    break;
   case Env_Line_Width:
     update_color ();
     break;
@@ -787,7 +1090,7 @@ edit_env_rep::update (string s) {
     update_dash_style();
     break;
   case Env_Dash_Style_Unit:
-    dash_style_unit= get_length (DASH_STYLE_UNIT);
+    update_dash_style_unit ();
     break;
   case Env_Fill_Color:
     update_color ();
@@ -795,12 +1098,18 @@ edit_env_rep::update (string s) {
   case Env_Line_Arrows:
     update_line_arrows();
     break;
+  case Env_Line_Portion:
+    line_portion= get_double (LINE_PORTION);
+    break;
   case Env_Text_At_Halign:
     text_at_halign= get_string (TEXT_AT_HALIGN);
     break;
   case Env_Text_At_Valign:
     text_at_valign= get_string (TEXT_AT_VALIGN);
     break;
+  case Env_Doc_At_Valign:
+    doc_at_valign= get_string (DOC_AT_VALIGN);
+    break;
   case Env_Src_Style:
     update_src_style ();
     break;
@@ -813,5 +1122,9 @@ edit_env_rep::update (string s) {
   case Env_Src_Close:
     update_src_close ();
     break;
+  case Env_Src_Color:
+    src_tag_color= get_string (SRC_TAG_COLOR);
+    src_tag_col= named_color (src_tag_color);
+    break;
   }
 }
