@@ -21,7 +21,8 @@ array<line_item> typeset_concat (edit_env env, tree t, path ip);
 void hyphenate (line_item item, int pos, line_item& item1, line_item& item2);
 array<path>
 line_breaks (array<line_item> a, int start, int end,
-	     SI line_width, SI first_spc, SI last_spc, bool ragged);
+	     SI line_width, SI large_width,
+             SI first_spc, SI last_spc, bool ragged);
 
 /******************************************************************************
 * Constructor
@@ -43,6 +44,7 @@ lazy_paragraph_rep::lazy_paragraph_rep (edit_env env2, path ip):
   mode       = as_string (env->read (PAR_MODE));
   flexibility= as_double (env->read (PAR_FLEXIBILITY));
   hyphen     = as_string (env->read (PAR_HYPHEN));
+  min_pen    = as_double (env->read (PAR_MIN_PENALTY));
   left       = env->get_length (PAR_LEFT);
   right      = env->get_length (PAR_RIGHT);
   bot        = 0;
@@ -55,6 +57,20 @@ lazy_paragraph_rep::lazy_paragraph_rep (edit_env env2, path ip):
   line_sep   = env->get_vspace (PAR_LINE_SEP);
   par_sep    = env->get_vspace (PAR_PAR_SEP);
   nr_cols    = env->get_int (PAR_COLUMNS);
+  swell      = array<SI> ();
+
+  SI sw= env->get_length (PAR_SWELL);
+  if (sw > 0)
+    swell << sw
+          << env->get_length (MATH_TOP_SWELL_START)
+          << env->get_length (MATH_TOP_SWELL_END)
+          << env->get_length (MATH_BOT_SWELL_START)
+          << env->get_length (MATH_BOT_SWELL_END);
+
+  string kr= as_string (env->read (PAR_KERNING_REDUCE));
+  if (kr == "auto") kreduce= 0.4 / 40.0;
+  else if (is_double (kr)) kreduce= as_double (kr);
+  else kreduce= 0.0;
 
   string ks= as_string (env->read (PAR_KERNING_STRETCH));
   if (ks == "auto") {
@@ -72,6 +88,28 @@ lazy_paragraph_rep::lazy_paragraph_rep (edit_env env2, path ip):
   if (ps == "true") protrusion= WESTERN_PROTRUSION;
   else protrusion= 0;
 
+  string cf= as_string (env->read (PAR_CONTRACTION));
+  if (cf == "auto") contraction= 1.0 / 40.0;
+  else if (is_double (cf)) contraction= as_double (cf);
+  else contraction= 0.0;
+
+  string ef= as_string (env->read (PAR_EXPANSION));
+  if (ef == "auto") {
+    double cpl= min (max (((double) width) / max (env->fn->wfn, 1), 10.0), 40.0);
+    expansion= 0.7 / cpl;
+  }
+  else if (ef == "tolerant") {
+    double cpl= min (max (((double) width) / max (env->fn->wfn, 1), 10.0), 40.0);
+    expansion= 1.75 / cpl;
+  }
+  else if (is_double (ef)) expansion= as_double (ef);
+  else expansion= 0.0;
+
+  //expansion= contraction= kstretch= kreduce= 0.0; // FIXME
+  //expansion= contraction= kreduce= 0.0; // FIXME
+  //contraction= kreduce= 0.0; // FIXME
+  //expansion= contraction= 0.0; // FIXME
+
   string sm= as_string (env->read (PAR_SPACING));
   if (sm == "plain");
   else if (sm == "quanjiao") protrusion += QUANJIAO;
@@ -105,7 +143,7 @@ lazy_paragraph_rep::line_print (line_item item) {
     else if (is_func (item->t, VAR_VSPACE) || is_func (item->t, VSPACE)) {
       space vspc= env->as_vspace (item->t[0]);
       if (is_func (item->t, VAR_VSPACE))
-	sss->vspace_before (vspc);
+        sss->vspace_before (vspc);
       else sss->vspace_after (vspc);
     }
     else if (L(item->t) == DATOMS)
@@ -114,6 +152,14 @@ lazy_paragraph_rep::line_print (line_item item) {
       sss->no_page_break_before ();
     else if (item->t == NO_PAGE_BREAK)
       sss->no_page_break_after ();
+    else if (item->t == VAR_NO_BREAK_HERE)
+      sss->no_break_before ();
+    else if (item->t == NO_BREAK_HERE)
+      sss->no_break_after ();
+    else if (item->t == NO_BREAK_START)
+      sss->no_break_start ();
+    else if (item->t == NO_BREAK_END) {
+      sss->no_break_end (); min_pen= 0; }
     else if (is_tuple (item->t, "env_page") ||
 	     (item->t == PAGE_BREAK) ||
 	     (item->t == NEW_PAGE) ||
@@ -228,8 +274,9 @@ total_width (array<box> bs) {
 }
 
 void
-lazy_paragraph_rep::adjust_kerning (SI dw, SI the_width) {
-  // attempt to add dw space by adjusting the kerning of the current line unit
+lazy_paragraph_rep::increase_kerning (SI dw, SI the_width) {
+  // attempt to add dw space by increasing the kerning of the current line unit
+  if (the_width <= 0) return;
   SI tot_spc= 0;
   for (int i=cur_start; i<N(items)-1; i++)
     tot_spc += spcs[i]->max;
@@ -252,21 +299,114 @@ lazy_paragraph_rep::adjust_kerning (SI dw, SI the_width) {
   }
 }
 
+void
+lazy_paragraph_rep::decrease_kerning (SI dw, SI the_width) {
+  // try to subtract dw space by reducing the kerning of the current line unit
+  if (the_width <= 0) return;
+  SI tot_spc= 0;
+  for (int i=cur_start; i<N(items)-1; i++)
+    tot_spc += spcs[i]->min;
+  dw= (((long int) dw) * (the_width - tot_spc)) / the_width;
+
+  int first, last;
+  find_first_last_text (first, last);
+  SI ref_w= total_width (range (items, cur_start, N(items)));
+  SI obj_w= ref_w - dw;
+  SI def_w= total_width (adjusted (0.0, first, last));
+  SI min_w= total_width (adjusted (-kreduce, first, last));
+  if (obj_w <= def_w && min_w < def_w) {
+    double ratio= ((double) (def_w - obj_w)) / ((double) (def_w - min_w));
+    ratio= min (ratio, 1.0);
+    array<box> bs= adjusted (-kreduce * ratio, first, last);
+    for (int i=0; i<N(bs); i++) {
+      cur_w += bs[i]->w() - items[cur_start + i]->w();
+      items[cur_start + i]= bs[i];
+    }
+  }
+}
+
+void
+lazy_paragraph_rep::expand_glyphs (SI dw, SI the_width) {
+  // attempt to add dw space by expanding the glyphs of the current line unit
+  if (expansion <= 0.0 || the_width <= 0) return;
+  SI tot_spc= 0;
+  for (int i=cur_start; i<N(items)-1; i++)
+    tot_spc += spcs[i]->max;
+  dw= (((long int) dw) * (the_width - tot_spc)) / the_width;
+  SI xdw= (SI) (dw * (expansion / (kstretch + expansion)));
+  SI mdw= (SI) (expansion * the_width);
+  int stages= 8;
+  int stage = min (((3 * stages + 1) * xdw) / (3 * mdw), stages);
+  if (stage <= 0) return;
+  double expansion_factor= (expansion * stage) / stages;
+  array<box> bs;
+  for (int i=cur_start; i<N(items); i++) {
+    box b= items[i]->expand_glyphs (0, expansion_factor);
+    cur_w += b->w() - items[i]->w();
+    items[cur_start + i]= b;
+  }
+}
+
+void
+lazy_paragraph_rep::contract_glyphs (SI dw, SI the_width) {
+  // try to subtract dw space by glyph contraction for the current line unit
+  if (contraction <= 0.0 || the_width <= 0) return;
+
+  // NOTE: adjust dw in presence of ligatures
+  int first, last;
+  find_first_last_text (first, last);
+  SI ref_w= total_width (range (items, cur_start, N(items)));
+  SI def_w= total_width (adjusted (0.0, first, last));
+  dw += (def_w - ref_w);
+  // END NOTE
+
+  SI tot_spc= 0;
+  for (int i=cur_start; i<N(items)-1; i++)
+    tot_spc += spcs[i]->min;
+  dw= (((long int) dw) * (the_width - tot_spc)) / the_width;
+  SI xdw= (SI) (dw * (contraction / (kreduce + contraction)));
+  SI mdw= (SI) (contraction * the_width);
+  int stages= 8;
+  int stage = min (((3 * stages + 2) * xdw) / (3 * mdw), stages);
+  if (stage <= 0) return;
+  double contraction_factor= (contraction * stage) / stages;
+  array<box> bs;
+  for (int i=cur_start; i<N(items); i++) {
+    box b= items[i]->expand_glyphs (0, -contraction_factor);
+    cur_w += b->w() - items[i]->w();
+    items[cur_start + i]= b;
+  }
+}
+
 /******************************************************************************
 * Typesetting a line
 ******************************************************************************/
 
+bool
+lazy_paragraph_rep::non_trailing_tabs () {
+  for (int i=0; i<N(tabs); i++) {
+    int pos= tabs[i]->pos;
+    bool b1= false, b2= false;
+    for (int k=cur_start; k<pos; k++)
+      if (items[k]->w () > 0) { b1= true; break; }
+    for (int k=pos+1; k<N(items); k++)
+      if (items[k]->w () > 0) { b2= true; break; }
+    if (b1 && b2) return true;
+  }
+  return false;
+}
+
 void
 lazy_paragraph_rep::make_unit (string mode, SI the_width, bool break_flag) {
   int i;
-
   // format tabs
   //cout << "      " << N(tabs) << "] " << (cur_w->def/PIXEL)
   //     << " < " << (the_width/PIXEL) << "? (" << break_flag << ")\n";
   //cout << mode << ", " << break_flag << ", " << N(tabs)
   //     << ", " << cur_w->def << " -- " << cur_w->max
   //     << ", " << the_width << "\n";
-  if (break_flag && (N(tabs)>0) && (cur_w->def<the_width)) {
+  if (break_flag && (N(tabs)>0) && (cur_w->def<the_width) &&
+      (mode != "center" || non_trailing_tabs ())) {
     double tot_weight= 0.0;
     int pos_first= -1, pos_last=-1;
     int num_hflush= 0;
@@ -274,21 +414,21 @@ lazy_paragraph_rep::make_unit (string mode, SI the_width, bool break_flag) {
       tab& tab_i= tabs[i];
       tot_weight += tab_i->weight;
       if (tab_i->kind == tab_first && pos_first < 0) {
-	num_hflush++; pos_first= i; }
+        num_hflush++; pos_first= i; }
       else if (tab_i->kind == tab_last) {
-	if (pos_last < 0) num_hflush++;
-	pos_last= i; 
+        if (pos_last < 0) num_hflush++;
+        pos_last= i;
       }
       else if (tab_i->kind == tab_all && tab_i->weight == 0.0)
-	num_hflush++;
+        num_hflush++;
     }
     for (i=cur_start; i<N(items)-1; i++) items_sp << spcs[i]->def;
     for (i=0; i<N(tabs); i++) {
       double part;
       if (tot_weight==0.0) {
-	if (i==pos_first || i==pos_last || tabs[i]->kind==tab_all)
-	  part= 1.0 / num_hflush;
-	else part= 0.0;
+        if (i==pos_first || i==pos_last || tabs[i]->kind==tab_all)
+          part= 1.0 / num_hflush;
+        else part= 0.0;
       }
       else part= tabs[i]->weight / tot_weight;
       items_sp[tabs[i]->pos] += (SI) (part * (the_width- cur_w->def));
@@ -313,11 +453,13 @@ lazy_paragraph_rep::make_unit (string mode, SI the_width, bool break_flag) {
     if (cur_w->max > cur_w->def)
       f= ((double) (the_width - cur_w->def)) /
          ((double) (cur_w->max - cur_w->def));
-    if (f > 1.0 && kstretch > 0.0) {
+    if (f > 1.0 && (kstretch + expansion) > 0.0) {
       space backup_cur_w= cur_w;
       double backup_f= f;
       array<box> backup= range (items, cur_start, N(items));
-      adjust_kerning (the_width - cur_w->max, the_width);
+      expand_glyphs (the_width - cur_w->max, the_width);
+      if (the_width > cur_w->max)
+        increase_kerning (the_width - cur_w->max, the_width);
       if (cur_w->max > cur_w->def)
         f= ((double) (the_width - cur_w->def)) /
            ((double) (cur_w->max - cur_w->def));
@@ -344,10 +486,29 @@ lazy_paragraph_rep::make_unit (string mode, SI the_width, bool break_flag) {
     double f=
       ((double) (cur_w->def - the_width)) /
       ((double) (cur_w->def - cur_w->min));
-    if (f>1.0) f=1.0;
+    if (f > 1.0 && (kreduce + contraction) > 0.0) {
+      space backup_cur_w= cur_w;
+      double backup_f= f;
+      array<box> backup= range (items, cur_start, N(items));
+      contract_glyphs (cur_w->min - the_width, the_width);
+      if (cur_w->min > the_width)
+        decrease_kerning (cur_w->min - the_width, the_width);
+      if (cur_w->def > cur_w->min)
+        f= ((double) (cur_w->def - the_width)) /
+           ((double) (cur_w->def - cur_w->min));
+      else if (cur_w->min <= the_width + PIXEL)
+        f= 1.0;
+      if (f < 0) {
+        cur_w= backup_cur_w;
+        f= backup_f;
+        for (i=0; i<N(backup); i++)
+          items[cur_start + i]= backup[i];
+      }
+    }
+    if (f > 1.0) f= 1.0;
     for (i=cur_start; i<N(items)-1; i++)
       items_sp <<
-	(spcs[i]->def- ((SI) (f*((double) spcs[i]->def- spcs[i]->min))));
+        (spcs[i]->def- ((SI) (f*((double) spcs[i]->def- spcs[i]->min))));
     return;
   }
 
@@ -504,7 +665,7 @@ lazy_paragraph_rep::line_end (space spc, int penalty) {
   box b= phrase_box (sss->ip, items, items_sp);
   sss->print (b, fl, nr_cols);
   sss->print (spc);
-  sss->penalty (penalty);
+  sss->penalty (max (penalty, min_pen));
   sss->flush ();
 }
 
@@ -524,7 +685,12 @@ lazy_paragraph_rep::line_units (
 
   int i;
   bool ragged= (hyphen == "normal");
-  array<path> hyphs= line_breaks (a, start, end, the_right-the_left,
+  SI line_width= the_right - the_left;
+  SI large_width= (SI) (line_width / (1.0 - 0.5 * (kreduce + contraction)));
+  // FIXME: the factor 0.5 is somewhat arbitrary and should be taken small
+  // enough so as to compensate for content that cannot be contracted
+  // on the line such as whitespace, images, and other miscellaneous objects.
+  array<path> hyphs= line_breaks (a, start, end, line_width, large_width,
 				  the_first, the_last, ragged);
   for (i=0; i<N(hyphs)-1; i++) {
     if (i>0) line_start ();
@@ -581,19 +747,19 @@ lazy_paragraph_rep::format_paragraph () {
     if (no_first) style (PAR_FIRST)= "0cm";
     for (j=start; j<i; j++)
       if (a[j]->type == CONTROL_ITEM)
-	if (is_tuple (a[j]->t, "env_par")) {
-	  if (a[j]->t[1]->label == PAR_FIRST) {
-	    for (k=j-1; k>=start; k--)
-	      if (a[k]->b->w () != 0) break;
-	    if (k >= start) continue;
-	  }
-	  style (a[j]->t[1]->label)= a[j]->t[2];
-	}
+        if (is_tuple (a[j]->t, "env_par")) {
+          if (a[j]->t[1]->label == PAR_FIRST) {
+            for (k=j-1; k>=start; k--)
+              if (a[k]->b->w () != 0) break;
+            if (k >= start) continue;
+          }
+          style (a[j]->t[1]->label)= a[j]->t[2];
+        }
     no_first= (style [PAR_NO_FIRST] == "true");
     if (no_first) env->monitored_write_update (PAR_NO_FIRST, "true");
     if (mode == "center") first= 0;
     else first= env->as_length (style [PAR_FIRST]);
-    sss->set_env_vars (height, sep, hor_sep, ver_sep, bot, top);
+    sss->set_env_vars (height, sep, hor_sep, ver_sep, bot, top, swell);
 
     // typeset paragraph unit
     format_paragraph_unit (start, i);
@@ -697,15 +863,15 @@ lazy_paragraph_rep::query (lazy_type request, format fm) {
     if (no_first) style (PAR_FIRST)= "0cm";
     for (int j=0; j<N(a); j++)
       if (a[j]->type == CONTROL_ITEM)
-	if (is_tuple (a[j]->t, "env_par")) {
-	  if (a[j]->t[1]->label == PAR_FIRST) {
+        if (is_tuple (a[j]->t, "env_par")) {
+          if (a[j]->t[1]->label == PAR_FIRST) {
             int k;
-	    for (k=j-1; k>=0; k--)
-	      if (a[k]->b->w () != 0) break;
-	    if (k >= 0) continue;
+            for (k=j-1; k>=0; k--)
+              if (a[k]->b->w () != 0) break;
+            if (k >= 0) continue;
             else first= env->as_length (a[j]->t[2]);
-	  }
-	}
+          }
+        }
     if (mode == "center") first= 0;
 
     SI w= left + first;
@@ -736,13 +902,18 @@ lazy_paragraph_rep::produce (lazy_type request, format fm) {
     int i, n= N(sss->l);
     if (hidden)
       for (i=0; i<n; i++) {
-	box b= sss->l[i]->b;
-	sss->l[i]->type= PAGE_HIDDEN_ITEM;
-	sss->l[i]->b   = resize_box (ip, b, b->x1, 0, b->x2, 0);
-	sss->l[i]->spc = space (0, 0, 0);
+        box b= sss->l[i]->b;
+        sss->l[i]->type= PAGE_HIDDEN_ITEM;
+        sss->l[i]->b   = resize_box (ip, b, b->x1, 0, b->x2, 0);
+        sss->l[i]->spc = space (0, 0, 0);
       }
     /* End hiding code */
     return lazy_vstream (ip, "", sss->l, sss->sb);
   }
   return lazy_rep::produce (request, fm);
 }
+
+void
+lazy_paragraph_rep::propagate () {
+  style (PAR_NO_FIRST)= env->read (PAR_NO_FIRST);
+}
